Package: gplots
Title: Various R Programming Tools for Plotting Data
Description: Various R programming tools for plotting data, including:
  - calculating and plotting locally smoothed summary function as
    ('bandplot', 'wapply'),
  - enhanced versions of standard plots ('barplot2', 'boxplot2',
    'heatmap.2', 'smartlegend'),
  - manipulating colors ('col2hex', 'colorpanel', 'redgreen',
    'greenred', 'bluered', 'redblue', 'rich.colors'),
  - calculating and plotting two-dimensional data summaries ('ci2d',
    'hist2d'),
  - enhanced regression diagnostic plots ('lmplot2', 'residplot'),
  - formula-enabled interface to 'stats::lowess' function ('lowess'),
  - displaying textual data in plots ('textplot', 'sinkplot'),
  - plotting dots whose size reflects the relative magnitude of the
    elements ('balloonplot', 'bubbleplot'),
  - plotting "Venn" diagrams ('venn'),
  - displaying Open-Office style plots ('ooplot'),
  - plotting multiple data on same region, with separate axes
    ('overplot'),
  - plotting means and confidence intervals ('plotCI', 'plotmeans'),
  - spacing points in an x-y plot so they don't overlap ('space').
Depends: R (>= 3.0)
Imports: gtools, stats, caTools, KernSmooth, methods
Suggests: grid, MASS, knitr, rmarkdown, r2d2, nlme, dendextend,
        heatmaply, RColorBrewer
LazyData: yes
VignetteBuilder: knitr
Version: 3.3.0
Date: 2025-11-29
Authors@R: c(
    person("Gregory R.", "Warnes", , role = "aut"),
    person("Ben", "Bolker", , role = "aut"),
    person("Lodewijk", "Bonebakker", , role = "aut"),
    person("Robert", "Gentleman", role = "aut"),
    person("Wolfgang", "Huber", role = "aut"),
    person("Andy", "Liaw", role = "aut"),
    person("Thomas", "Lumley", role = "aut"),
    person("Martin", "Maechler", role = "aut"),
    person("Arni", "Magnusson", role = "aut"),
    person("Steffen", "Moeller", role = "aut"),
    person("Marc", "Schwartz", role = "aut"),
    person("Bill", "Venables", role = "aut"),
    person("Tal", "Galili", , "tal.galili@gmail.com", c("aut", "cre"))
    )
License: GPL-2
URL: https://github.com/talgalili/gplots,
        https://talgalili.github.io/gplots/
BugReports: https://github.com/talgalili/gplots/issues
NeedsCompilation: no
Packaged: 2025-11-29 18:15:47 UTC; talgalili
Author: Gregory R. Warnes [aut],
  Ben Bolker [aut],
  Lodewijk Bonebakker [aut],
  Robert Gentleman [aut],
  Wolfgang Huber [aut],
  Andy Liaw [aut],
  Thomas Lumley [aut],
  Martin Maechler [aut],
  Arni Magnusson [aut],
  Steffen Moeller [aut],
  Marc Schwartz [aut],
  Bill Venables [aut],
  Tal Galili [aut, cre]
Maintainer: Tal Galili <tal.galili@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-30 06:10:15 UTC
Built: R 4.6.0; ; 2026-04-26 10:11:01 UTC; unix

export(
       adjust_hsv,
       angleAxis,
       balloonplot,
       bandplot,
       barplot2,
       bluered,
       boxplot2,
       bubbleplot,
       ci2d,
       col2hex,
       colorpanel,
       greenred,
       heatmap.2,
       hist2d,
       lmplot2,
       lowess,
       ooplot,
       overplot,
       panel.overplot,
       plot.venn,
       plotCI,
       plotLowess,
       plotmeans,
       qqnorm.aov,
       redblue,
       redgreen,
       residplot,
       rich.colors,
       sinkplot,
       space,
       textplot,
       venn,
       wapply,
       reorder.factor
)

importFrom(gtools, invalid)
importFrom(gtools, odd)
importFrom(gtools, mixedsort)

importFrom(stats, reorder)
importFrom(stats, na.omit)
importFrom(stats, nobs)

importFrom(caTools, runsd)
importFrom(caTools, runmean)

importFrom(KernSmooth, bkde2D)
importFrom(KernSmooth, dpik)

importFrom(utils, type.convert)

S3method(balloonplot, default)
S3method(balloonplot, table)

S3method(bandplot, default)
S3method(bandplot, formula)

S3method(barplot2, default)

S3method(bubbleplot, default)
S3method(bubbleplot, formula)

S3method(lowess, default)
S3method(lowess, formula)

S3method(ooplot, default)

S3method(plot, lowess)
S3method(plot, venn)

S3method(print, ci2d)
S3method(print, hist2d)

S3method(qqnorm, aov)

S3method(textplot, character)
S3method(textplot, data.frame)
S3method(textplot, default)
S3method(textplot, matrix)

S3method(reorder, factor)

# imports recommended by R CMD check
importFrom("grDevices", "col2rgb", "contourLines", "dev.flush",
           "dev.hold", "dev.interactive", "heat.colors", "hsv", "rgb",
           "rgb2hsv", "topo.colors")

importFrom("graphics", "abline", "arrows", "axTicks", "axis",
           "barplot", "contour", "filled.contour", "grid", "hist",
           "identify", "image", "layout", "legend", "lines",
           "matlines", "matpoints", "mtext", "panel.smooth", "par",
           "plot", "plot.default", "plot.new", "plot.window", "points",
           "polygon", "rect", "segments", "strheight", "strwidth",
           "text", "title", "xinch", "yinch")

importFrom("stats", "as.dendrogram", "cooks.distance", "density",
           "deviance", "df.residual", "dist", "dnorm", "effects",
           "expand.model.frame", "fitted", "hclust", "is.leaf",
           "lm.influence", "median", "model.frame", "order.dendrogram",
           "ppoints", "predict", "qnorm", "qqline", "qqnorm", "qt",
           "residuals", "sd", "var", "weights")

importFrom("utils", "str")

importFrom("methods", "is")

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/gplots)](https://cran.r-project.org/package=gplots)
[![CRAN Monthly](https://cranlogs.r-pkg.org/badges/gplots)](https://cranlogs.r-pkg.org/badges/gplots?color=yellow)
[![CRAN Total](https://cranlogs.r-pkg.org/badges/grand-total/gplots)](https://cranlogs.r-pkg.org/badges/grand-total/gplots?color=yellowgreen)
[![codecov.io](https://codecov.io/github/talgalili/gplots/coverage.svg?branch=master)](https://app.codecov.io/github/talgalili/gplots?branch=master)
[![R-CMD-check](https://github.com/talgalili/gplots/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/talgalili/gplots/actions/workflows/R-CMD-check.yaml)

**[gplots website](https://talgalili.github.io/gplots/) (built using [pkgdown](https://pkgdown.r-lib.org/))**

# gplots


## Introduction

This repo is to keep the gplots package alive. I don't plan to develop new features, but if you'll send pull requests I'm willing to review them.


### Latest news

You can see the most recent changes to the package in the [NEWS.md file](https://talgalili.github.io/gplots/news/index.html)



### Code of conduct

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/talgalili/gplots/blob/master/CODE_OF_CONDUCT.md). By participating in this project you agree to abide by its terms.



## Installation

To install the stable version on CRAN:

```r
install.packages('gplots')
```


And then you may load the package using:

```R
library("gplots")
```

## Usage

TODO


## Contact

You are welcome to:

* Ask questions on: <https://stackoverflow.com/questions/tagged/gplots>
* Submit suggestions and bug-reports at: <https://github.com/talgalili/gplots/issues>
* Send a pull request on: <https://github.com/talgalili/gplots/>
* Compose a friendly e-mail to: <tal.galili@gmail.com>

layout_set <- function(mat, index)
  {
    mfg <- par("mfg")
    mfg[1:2] = which(mat==index, arr.ind=TRUE)[1,]
    par(mfg=mfg)
    invisible(mfg)
  }

layout_show <- function(mat)
  {
    graphics::layout.show( max(mat) )
    mat
  }

# make stats::lowess into a generic base-function
lowess.default <- function (x, y = NULL,
                            f = 2/3,
                            iter = 3L,
                            delta = 0.01 * diff(range(x)),
                            ...)
  {
    m <- match.call()
    m[[1L]] <- quote(stats::lowess)
    retval <- eval(m, envir=parent.frame())
    class(retval) <- "lowess"
    retval$call <- match.call()
    retval
  }

lowess  <- function(x,...)
  UseMethod("lowess")


"lowess.formula" <-  function (formula,
                               data = parent.frame(),
                               ...,
                               subset,
                               f=2/3,
                               iter=3,
                               delta=.01*diff(range(mf[-response]))
                               )
{
  if (missing(formula) || (length(formula) != 3))
    stop("formula missing or incorrect")

  m <- match.call(expand.dots = FALSE)
  eframe <- parent.frame()
  md <- eval(m$data, eframe)
  if (is.matrix(md))
    m$data <- md <- as.data.frame(data)
  dots <- lapply(m$..., eval, md, eframe)
  nmdots <- names(dots)
  m$...  <- m$f <- m$iter <- m$delta <- NULL
  subset.expr <- m$subset
  m$subset <- NULL
  m <- as.list(m)
  m[[1L]] <- stats::model.frame.default
  m <- as.call(c(m, list(na.action = NULL)))
  mf <- eval(m, eframe)
  if (!missing(subset)) {
    s <- eval(subset.expr, data, eframe)
    l <- nrow(mf)
    dosub <- function(x) if (length(x) == l)
      x[s]
    else x
    dots <- lapply(dots, dosub)
    mf <- mf[s, ]
  }

  mf <- na.omit(mf)

  response <- attr(attr(mf, "terms"), "response" )
  retval <- stats::lowess(mf[[-response]], mf[[response]], f=f, iter=iter, delta=delta)
  class(retval) <- "lowess"
  retval$call <- match.call()

  retval
}

# transform base
#   v = value of base 10 to be transformed
#   b = new base
#   l = minimal length of returned array (default is 1)
# return value: array of factors, highest exponent first
baseOf<-function(v,b,l=1) {
	remainder<-v
	i<-l
	ret<-NULL
	while(remainder>0 || i>0) {
		#print(paste("i=",i," remainder=",remainder))
		m<-remainder%%b
		if (is.null(ret)) {
			ret<-m
		}
		else {
			ret<-c(m,ret)
		}
		remainder <- remainder %/% b
		i<-i-1
	}
	return(ret)
}
                                   

suppressMessages(library(gplots))
data(mtcars)

x <- as.matrix(mtcars)

## draws expected image
testHeatmap <- heatmap(x, Colv=NA, col=bluered(256), scale="column",
                       keep.dendro=TRUE)

# to prove this dendro is OK, redraw with same function:
heatmap(x, Colv=NA, col=bluered(256), scale="column",
        keep.dendro=TRUE, Rowv=testHeatmap$Rowv)

# but it doesn't work with heatmap.2()
heatmap.2(x, Colv=NA, col=bluered(256), scale="column",
          Rowv=testHeatmap$Rowv, dendrogram="row")

rich.colors <- function(n,
                        palette="temperature",
                        alpha=1,
                        rgb=FALSE,
                        plot=FALSE)
{
  if(n <= 0)
    return(character(0))

  palette <- match.arg(palette, c("temperature","blues"))
  x <- seq(0, 1, length=n)

  if(palette == "temperature")
  {
    r <- 1 / (1+exp(20-35*x))
    g <- pmin(pmax(0,-0.8+6*x-5*x^2), 1)
    b <- dnorm(x,0.25,0.15) / max(dnorm(x,0.25,0.15))
  }
  else
  {
    r <-        0.6*x + 0.4*x^2
    g <-        1.5*x - 0.5*x^2
    b <- 0.36 + 2.4*x - 2.0*x^2
    b[x>0.4] <- 1
  }

  rgb.m <- matrix(c(r,g,b), ncol=3,
                  dimnames=list(NULL,c("red","green","blue")))
  col <- mapply(rgb, r, g, b, alpha)

  if(rgb) 
    attr(col, "rgb") <- cbind(rgb.m, alpha)
  
  if(plot)
  {
    opar <- par("fig", "plt")
    par(fig=c(0,1,0,0.7), plt=c(0.15,0.9,0.2,0.95))
    plot(NA, xlim=c(-0.01,1.01), ylim=c(-0.01,1.01), xlab="Spectrum", ylab="",
         xaxs="i", yaxs="i", axes=FALSE)
    title(ylab="Value", mgp=c(3.5,0,0))
    matlines(x, rgb.m, col=colnames(rgb.m), lty=1, lwd=3)
    matpoints(x, rgb.m, col=colnames(rgb.m), pch=16)
    axis(1, at=0:1)
    axis(2, at=0:1, las=1)
    par(fig=c(0,1,0.75,0.9), plt=c(0.08,0.97,0,1), new=TRUE)
    midpoints <- barplot(rep(1,n), col=col, border=FALSE, space=FALSE,
                         axes=FALSE)
    axis(1, at=midpoints, labels=1:n, lty=0, cex.axis=0.6)
    par(opar)
  }

  return(col)
}

# $Id: ooplot.R 1557 2012-06-08 17:56:37Z warnes $

ooplot <- function(data, ...) UseMethod("ooplot")

ooplot.default <- function(data, width=1, space=NULL, names.arg=NULL,
                           legend.text=NULL, horiz=FALSE,
                           density=NULL, angle=45, kmg="fpnumkMGTP",
                           kmglim=TRUE,
                           type=c("xyplot", "linear", "barplot", "stackbar"),
                           col=heat.colors(NC), prcol=NULL,
                           border=par("fg"), main=NULL, sub=NULL,
                           xlab=NULL, ylab=NULL, xlim=NULL, ylim=NULL,
                           xpd=TRUE, log="", axes=TRUE,
                           axisnames=TRUE, prval=TRUE, lm=FALSE,
                           cex.axis=par("cex.axis"),
                           cex.names=par("cex.axis"),
                           cex.values=par("cex"),inside=TRUE,
                           plot=TRUE, axis.lty=0, plot.ci=FALSE,
                           ci.l=NULL, ci.u=NULL, ci.color="black",
                           ci.lty="solid", ci.lwd=1, plot.grid=FALSE,
                           grid.inc=NULL, grid.lty="dotted",
                           grid.lwd=1, grid.col="black", add=FALSE,
                           by.row=FALSE, ...)
{

  ##
  ##  oopplot function block
  ##
  ## this is the location of the helper functions for this method
  ##


  optlim <- function(lim, log=FALSE) {
    ## define xlim and ylim, adjusting for log-scale if needed
    factor <- 1.05
    if (log) {
      min <- 10^floor(log10(lim[1]/factor))
      max <- 10^ceiling(log10(factor*lim[2]))
    } else {
      range <- (factor*lim[2]-lim[1]/factor)
      if (range>0) {
        ## we know the range, now find the optimal start and endpoints
        scale <- 10^floor(log10(range))
        min <- scale*(floor((lim[1]/factor)/scale))
        max <- scale*(ceiling(factor*lim[2]/scale))
      } else {
        min=0
        max=1
      }
    }
    if (type=="barplot" || type=="stackbar") max=max+1

    return(c(min, max))
  }

  linearfitplot <- function(x,y,xlim,col) {
    ## calculate a linear fit through the datapoints and plot
    local <- data.frame(x=x,y=y)
    local.lm <- lm(y ~ x, data=local)
    summary <- summary(local.lm)
    xmin=min(x)
    xmax=max(x)
    if (xlim[1]<xmin) {
      x1=mean(xlim[1],xmin)
    } else {
      x1=max(min(x),xlim[1])
    }
    if (xlim[2]>xmax) {
      x2=mean(xlim[2],xmax)
    } else {
      x2=min(max(x),xlim[2])
    }
    y1=summary$coefficients[2,1]*x1+summary$coefficients[1,1]
    y2=summary$coefficients[2,1]*x2+summary$coefficients[1,1]
    p1=c(x1,x2)
    p2=c(y1,y2)
    lines(p1,p2,col=col,lty=2,lwd=2)
  }


  castNA <- function(matrix) {
    newmatrix <- matrix
    for (j in 1:ncol(matrix)) {
      for (i in 1:nrow(matrix)) {
        newmatrix[i,j] <- ifelse(is.na(matrix[i,j]),0,matrix[i,j])
      }
    }
    return(newmatrix)
  }
  ##
  ## End of function block
  ##
  ## In R, most people think about data in columns rather than rows
  if(by.row)
    data <- as.matrix(data)
  else
    data <- t(as.matrix(data))

  ## make sure we only accept the supported plot options
  type <- match.arg(type)
  ## check data validity
  if( nrow(data) < 2 ) stop("At least 2 columns are required.")
  ## check defaults
  if (!missing(inside)) .NotYetUsed("inside", error=FALSE)# -> help(.)
  ## set the beside parameter
  if (type=="stackbar") {
    beside <- FALSE
    data <- castNA(data)
  }
  else
    beside <- TRUE

  ## split the data into x and y values
  height <- data[-1,,drop=FALSE]


  heightscale <- ""
  heightsymbol <- ""
  ##
  if ((kmg!="") && (kmglim==TRUE)){
    ##
    ## auto scale the parameters
    ##
    ## now scale the data, valid factors are
    ## P : peta=1E15
    ## T : tera=1E12
    ## G : giga=1E09
    ## M : mega=1E06
    ## k : kilo=1E03
    ## m : milli=1E-03
    ## u : micro=1E-06
    ## n : nano=1E-09
    ## p : pico=1E-12
    ## f : femto=1E-15
    maxheight <- max(abs(height), na.rm=TRUE)
    heightfactor <- 1
    if (maxheight>10E15 && any(grep("P", kmg))) {
      heightfactor <- 1E15
      heightscale <- "Peta"
      heightsymbol <- "P"
    }
    else if (maxheight>1E12 && any(grep("T", kmg))) {
      heightfactor <- 1E12
      heightscale <- "Tera"
      heightsymbol <- "T"
    }
    else if (maxheight>1E09 && any(grep("G", kmg))) {
      heightfactor <- 1E09
      heightscale <- "Giga"
      heightsymbol <- "G"
    }
    else if (maxheight>1E06 && any(grep("M", kmg))) {
      heightfactor <- 1E06
      heightscale <- "Mega"
      heightsymbol <- "M"
    }
    else if (maxheight>1E03 && any(grep("k", kmg))) {
      heightfactor <- 1E03
      heightscale <- "Kilo"
      heightsymbol <- "k"
    }
    else if (maxheight<1E-15 && any(grep("f", kmg))) {
      heightfactor <- 1E-15
      heightscale <- "Femto"
      heightsymbol <- "f"
    }
    else if (maxheight<1E-12 && any(grep("p", kmg))) {
      heightfactor <- 1E-12
      heightscale <- "Pico"
      heightsymbol <- "p"
    }
    else if (maxheight<1E-09 && any(grep("n", kmg))) {
      heightfactor <- 1E-09
      heightscale <- "Nano"
      heightsymbol <- "n"
    }
    else if (maxheight<1E-06 && any(grep("u", kmg))) {
      heightfactor <- 1E-06
      heightscale <- "Micro"
      heightsymbol <- "u"
    }
    else if (maxheight<1E-03 && any(grep("m", kmg))) {
      heightfactor <- 1E-03
      heightscale <- "Milli"
      heightsymbol <- "m"
    }

    height <- height/heightfactor

  }

  ## fill the xaxis data set and matching rownames
  xaxis <- data[1, ]
  rownames <- rownames(data)

  ##
  if (missing(space))
    space <- if (is.matrix(height) && beside) c(0, 1) else 0.2
  space <- space * mean(width)
  ##
  if (plot && axisnames && missing(names.arg)) {
    if (type=="xyplot")
      names.arg <- colnames(height)
    else if (type=="linear")
      names.arg <- xaxis
    else if (type=="barplot")
      names.arg <- xaxis
    else if (type=="stackbar")
      names.arg <- xaxis
    else names.arg <- xaxis
  }


  ## set the legend text if it is null
  if(is.logical(legend.text))
    legend.text <- rownames[-c(1)]

  ##  if(legend.text && is.matrix(height)) rownames(height)
  ##  else colnames(height)

  if (is.vector(height) || is.array(height))
    {
      height <- rbind(height)
    }
  else if (!is.matrix(height))
    stop("`height' must be a vector or a matrix")

  ## Check for log scales
  logx <- FALSE
  logy <- FALSE

  if (log !="")
    {
      if (any(grep("x", log)))
        logx <- TRUE
      if (any(grep("y", log)))
        logy <- TRUE
    }

  ## Cannot "hatch" with rect() when log scales used
  if ((logx || logy) && !is.null(density))
    stop("Cannot use shading lines in bars when log scale is used")

  ## Set the size of Rows and Columns
  NR <- nrow(height)
  NC <- ncol(height)
  ## w.r, w.l, w.m are the x-axis coordinates

  if (type=="barplot") {
    if (NR<1) NR <- 1
    if (NC<1) NC <- 1
    if (length(space)==2) {
      space <- rep.int(c(space[2], rep.int(space[1], NR - 1)), NC)
    }
    width <- rep(width, length=NR * NC)
  } else if (type=="stackbar") {
    width <- rep(width, length=NC)
  }  else {
    width <- rep(width, length=NC)
    delta <- width / 2
  }

  ## set the proper x-axis scale
  ## linear is a switch between equidistant and scaled
  if (type=="xyplot") {
    delta <- 0
    w.m <- xaxis
    w.r <- w.m + delta
    w.l <- w.m - delta
  }
  else if (type=="linear") {
    w.r <- cumsum(width)
    w.m <- w.r - delta
    w.l <- w.m - delta
    xaxis <- w.m
  }
  else if(type=="barplot" || type=="stackbar") {
    delta <- width / 2
    w.r <- cumsum(space + width)
    w.m <- w.r - delta
    w.l <- w.m - delta
    ##if graphic will be stacked bars, do not plot ci
    if (beside && (NR > 1) && plot.ci)
      plot.ci=FALSE
  }
  else stop("Unkown plot type")


  ## error check ci arguments
  if (plot && plot.ci)
    {
      if ((missing(ci.l)) || (missing(ci.u)))
        stop("confidence interval values are missing")

      if (is.vector(ci.l))
        ci.l <- cbind(ci.l)
      else if (is.array(ci.l) && (length(dim(ci.l))==1))
        ci.l <- rbind(ci.l)
      else if (!is.matrix(ci.l))
        stop("`ci.l' must be a vector or a matrix")

      if (is.vector(ci.u))
        ci.u <- cbind(ci.u)
      else if (is.array(ci.u) && (length(dim(ci.u))==1))
        ci.u <- rbind(ci.u)
      else if (!is.matrix(ci.u))
        stop("`ci.u' must be a vector or a matrix")

      if ( any(dim(height) !=dim(ci.u) ) )
        stop("'height' and 'ci.u' must have the same dimensions.")
      else if ( any( dim(height) !=dim(ci.l) ) )
        stop("'height' and 'ci.l' must have the same dimensions.")
    }

  if (beside)
    w.m <- matrix(w.m, ncol=NC)

  ## check height/ci.l if using log scale to prevent log(<=0) error
  ## adjust appropriate ranges and bar base values
  if ((logx && horiz) || (logy && !horiz))
    {
      if (min(height, na.rm=TRUE) <=0)
        stop("log scale error: at least one 'height' value <=0")

      if (plot.ci && (min(ci.l) <=0))
        stop("log scale error: at least one lower c.i. value <=0")

      if (logx && !is.null(xlim) && (xlim[1] <=0))
        stop("log scale error: 'xlim[1]' <=0")

      if (logy && !is.null(ylim) && (ylim[1] <=0))
        stop("'log scale error: 'ylim[1]' <=0")

      ## arbitrary adjustment to display some of bar for min(height)
      ## or min(ci.l)
      if (plot.ci)
        rectbase <- rangeadj <- (0.9 * min(c(height, ci.l)))
      else
        rectbase <- rangeadj <- (0.9 * min(height))

      ## if axis limit is set to < above, adjust bar base value
      ## to draw a full bar
      if (logy && !is.null(ylim))
        rectbase <- ylim[1]
      else if (logx && !is.null(xlim))
        rectbase <- xlim[1]

      ## if stacked bar, set up base/cumsum levels, adjusting for log scale
      if (type=="stackbar") {
        heightdata <- height ## remember the original values
        height <- rbind(rectbase, apply(height, 2, cumsum))
      }

      ## if plot.ci, be sure that appropriate axis limits are set to
      ## include range(ci)
      lim <-
        if (plot.ci)
          c(height, ci.l, ci.u)
        else
          height
    }
  else
    {
      ## Use original bar base value
      rectbase <- 0

      ## if stacked bar, set up base/cumsum levels
      if (type=="stackbar") {
        heightdata <- height ## remember the original values

        height <- rbind(rectbase, apply(height, 2, cumsum))
      }

      ## if plot.ci, be sure that appropriate axis limits are set to
      ## include range(ci)
      lim <-
        if (plot.ci)
          c(height, ci.l, ci.u)
        else
          height

      ## use original range adjustment factor
      rangeadj <- (-0.01 * lim)
    }


  ## calculate the ranges ourselves
  if (missing(xlim)) xlim <- optlim(range(w.l, w.r, na.rm=TRUE), logx)
  if (missing(ylim)) ylim <- optlim(range(height, na.rm=TRUE), logy)
  ##
  if(plot) ##-------- Plotting :
    {
      if (type=="barplot" || type=="stackbar") {
        if (horiz)

          rectbase <- xlim[1] ## make sure the xlimit and the rectbase
                              ## are in sync
        else
          rectbase <- ylim[1] ## make sure the ylimit and rectbase are in sync
        opar <-
          if (horiz)
            par(xaxs="i", xpd=xpd)
          else
            par(yaxs="i", xpd=xpd)
      } else {
        opar <-
          if (horiz)
            par(xaxs="r", yaxs="r", xpd=xpd)
          else
            par(xaxs="r", yaxs="r", xpd=xpd)
      }
      on.exit(par(opar))

      ## If add=FALSE open new plot window
      ## else allow for adding new plot to existing window
      if (!add)
        {
          plot.new()
          plot.window(xlim, ylim, log=log, ...)
        }

      ## Set plot region coordinates
      usr <- par("usr")

      ## adjust par("usr") values if log scale(s) used
      if (logx)
        {
          usr[1] <- 10 ^ usr[1]
          usr[2] <- 10 ^ usr[2]
        }

      if (logy)
        {
          usr[3] <- 10 ^ usr[3]
          usr[4] <- 10 ^ usr[4]
        }

      ## if prcol specified, set plot region color
      if (!missing(prcol))
        rect(usr[1], usr[3], usr[2], usr[4], col=prcol)

      ## if plot.grid, draw major y-axis lines if vertical or x axis
      ## if horizontal R V1.6.0 provided axTicks() as an R equivalent
      ## of the C code for CreateAtVector.  Use this to determine
      ## default axis tick marks when log scale used to be consistent
      ## when no grid is plotted.  Otherwise if grid.inc is specified,
      ## use pretty()

      if (plot.grid)
      {
        par(xpd=FALSE)

        if (is.null(grid.inc))
        {
          if (horiz)
          {
            grid <- axTicks(1)
            abline(v=grid, lty=grid.lty, lwd=grid.lwd, col=grid.col)
          }
          else
          {
            grid <- axTicks(2)
            abline(h=grid, lty=grid.lty, lwd=grid.lwd, col=grid.col)
          }
        }
        else
        {
          if (horiz)
          {
            grid <- pretty(xlim, n=grid.inc)
            abline(v=grid, lty=grid.lty, lwd=grid.lwd, col=grid.col)
          }
          else
          {
            grid <- pretty(ylim, n=grid.inc)
            abline(h=grid, lty=grid.lty, lwd=grid.lwd, col=grid.col)
          }
        }

         par(xpd=xpd)
      }

      ##
      ## end of the general setup, now get ready to plot the elements
      ##
      ## cycle through all the sets and plot the lines
      if (type=="xyplot" || type=="linear") {
        pch <- c()
        for (i in 1:NR) {
          list <- is.finite(height[i, ])
          xrange <- xaxis[list]
          yrange <- height[i, list]
          lines(xrange, yrange, col=col[i])
          if ((type=="xyplot") && (lm==TRUE)) linearfitplot(xrange,yrange,xlim,col[i])
          symbol=21+(i %% 5)
          points(xrange, yrange, pch=symbol, bg=col[i], col=col[i])
          pch <- c(pch, symbol)
          ##
          if (prval) {
            values=paste(format(yrange, digits=4), heightsymbol, sep="")
            chh <- par()$cxy[2]
            chw <- par()$cxy[1]
            if (logy) {
              factor=(yrange+chh)/yrange
              text(xrange, 1.1*yrange, labels=values, adj=c(0, 1),
	           srt=0, cex=cex.values, bg="white", col="navy")
              } else {
              text(xrange, yrange+chh, labels=values, adj=c(0, 1),
	           srt=0, cex=cex.values, bg="white", col="navy")
            }
          }
        }
      }

      if (type=="barplot" || type=="stackbar") {
        xyrect <- function(x1, y1, x2, y2, horizontal=TRUE, ...)
          {
            if(horizontal)
              rect(x1, y1, x2, y2, ...)
            else
              rect(y1, x1, y2, x2, ...)
          }

        chh <- par()$cxy[2]
        chw <- par()$cxy[1]
        if (type=="barplot") {
          xyrect(rectbase, w.l, c(height), w.r, horizontal=horiz,
                 angle=angle, density=density, col=col, border=border)
          if (prval) {
            values=paste(format(c(height), digits=4), heightsymbol, sep="")
                if (horiz) {
                   text(c(height)+chw, w.l, labels=values, adj=c(0, 1),
                        srt=0, cex=cex.values, bg="white", col="navy")
                } else {
                  text(w.l, c(height)+chh, labels=values, adj=c(0, 1),
                       srt=0, cex=cex.values, bg="white", col="navy")
         }
          }
        } else if (type=="stackbar") {
          for (i in 1:NC) {
            xyrect(height[1:NR, i], w.l[i], height[-1, i], w.r[i],
                   horizontal=horiz, angle=angle, density=density,
                   col=col, border=border)
            if (prval) {
              for (j in 1:NR) {
                values=paste(format(c(heightdata[j, i]), digits=4),
                  heightsymbol, sep="")
                if (horiz) {
                   text(c(height[j, i])+chw, w.m[i], labels=values,
                        adj=c(0, 1), srt=0, cex=cex.values, bg="white",
                        col="navy")
                } else {
                text(w.m[i], c(height[j, i])+chh, labels=values, adj=c(0, 1),
                     srt=0, cex=cex.values, bg="white", col="navy")
              }
              }
            }
          }
        }
      }

      if (plot.ci)
        {
          ## CI plot width=barwidth / 2
          ci.width=width / 4

          if (horiz)
            {
              segments(ci.l, w.m, ci.u, w.m, col=ci.color, lty=ci.lty,
                       lwd=ci.lwd)
              segments(ci.l, w.m - ci.width, ci.l, w.m + ci.width,
                       col=ci.color, lty=ci.lty, lwd=ci.lwd)
              segments(ci.u, w.m - ci.width, ci.u, w.m + ci.width,
                       col=ci.color, lty=ci.lty, lwd=ci.lwd)
            }
          else
            {
              segments(w.m, ci.l, w.m, ci.u, col=ci.color, lty=ci.lty,
                       lwd=ci.lwd)
              segments(w.m - ci.width, ci.l, w.m + ci.width, ci.l,
                       col=ci.color, lty=ci.lty, lwd=ci.lwd)
              segments(w.m - ci.width, ci.u, w.m + ci.width, ci.u,
                       col=ci.color, lty=ci.lty, lwd=ci.lwd)
            }
        }


      if (axisnames && !is.null(names.arg)) # specified or from {col}names
        {
          at.l <-
            if (length(names.arg) !=length(w.m))
              {
                if (length(names.arg)==NC) # i.e. beside (!)
                  colMeans(w.m)
                else
                  if ((type=="barplot") && (NR==1)) {
                    median(w.m)
                  } else if ((type=="linear") && (NR==1)) {
                    median(w.m)
                  } else {
                    stop("incorrect number of names now")
                  }
              }
            else w.m

          ##axis(if(horiz) 2 else 1, at=at.l, labels=names.arg,
          ##     lty=axis.lty, cex.axis=cex.names, ...)
        }


      if(!is.null(legend.text))
        {
          legend.col <- rep(col, length=length(legend.text))

          if((horiz & beside) || (!horiz & !beside))
            {
              legend.text <- rev(legend.text)
              legend.col <- rev(legend.col)
              density <- rev(density)
              angle <- rev(angle)
            }

          ## adjust legend x and y values if log scaling in use
          if (logx)
            legx <- usr[2] - ((usr[2] - usr[1]) / 10)
          else
            legx <- usr[2] - xinch(0.1)

          if (logy)
            legy <- usr[4] - ((usr[4] - usr[3]) / 10)
          else
            legy <- usr[4] - yinch(0.1)

          if (type=="barplot" || type=="stackbar") {
            legend(legx, legy,
                   legend=legend.text, angle=angle, density=density,
                   fill=legend.col, xjust=1, yjust=1)
          } else {
            legend(legx, legy,
                   legend=legend.text,
                   ##angle=angle,
                   ##density=density,
                   lwd=1,
                   col=legend.col,
                   pch=pch,
                   pt.bg=legend.col,
                   xjust=1,
                   yjust=1)
          }
        }

      title(main=main, sub=sub, xlab=xlab, ylab=paste(heightscale, ylab), ...)

      ## if axis is to be plotted, adjust for grid "at" values
      if (axes)
        {
          par(lab=c(10, 10, 7))
          if (type=="barplot" || type=="stackbar") {
            axis(if(horiz) 2 else 1, at=at.l, labels=names.arg, lty=axis.lty,
                 cex.axis=cex.names, ...)
            if(plot.grid)
              axis(if(horiz) 1 else 2, at=grid, cex.axis=cex.axis, ...)
            else
              axis(if(horiz) 1 else 2, cex.axis=cex.axis, ...)
          }
          else if (type=="linear") {
            if(plot.grid) {
              axis(if(horiz) 1 else 2, at=grid, cex.axis=cex.axis, ...)
              axis(if(horiz) 2 else 1, at=at.l, labels=names.arg,
                   lty=axis.lty, cex.axis=cex.names, ...)
            } else {
              axis(if(horiz) 1 else 2, pos=0, cex.axis=cex.axis, ...)
              axis(if(horiz) 2 else 1, pos=ylim[1], at=at.l, labels=names.arg,
                   cex.axis=cex.names, ...)
            }
          }
          else if (type=="xyplot") {
            if(plot.grid) {
              axis(if(horiz) 1 else 2, at=grid, cex.axis=cex.axis, ...)
              axis(if(horiz) 2 else 1, cex.axis=cex.axis, )
            } else {
              if (horiz) {
                axis(1, pos=ylim[1], cex.axis=cex.axis, ...)
                axis(2, pos=xlim[1], cex.axis=cex.axis, ...)
              } else {
                axis(2, pos=xlim[1], cex.axis=cex.axis, ...)
                axis(1, pos=ylim[1], cex.axis=cex.axis, ...)
              }
            }
          }
        }

      invisible(w.m)

    }

    else w.m
}

.onAttach <- function(lib, pkg, ...) {
  packageStartupMessage(dendextendWelcomeMessage())
}



dendextendWelcomeMessage <- function() {
  paste0(
    "\n",
    "---------------------\n",
    "gplots ", utils::packageDescription("gplots")$Version, " loaded:\n",
    "  * Use citation('gplots') for citation info.\n",
    "  * Homepage: https://talgalili.github.io/gplots/\n",
    "  * Report issues: https://github.com/talgalili/gplots/issues\n",
    "  * Ask questions: https://stackoverflow.com/questions/tagged/gplots\n",
    "  * Suppress this message with: suppressPackageStartupMessages(library(gplots))\n",
    "---------------------\n"
  )
}




# Make sure to have: 
  # sudo apt-get install texinfo
  # sudo apt-get install texlive

# shell('git log --graph --stat --date=short --pretty=format:"%ad(%an) %s |%h" > ChangeLog', intern = TRUE)
# system('git log --graph --stat --date=short --pretty=format:"%ad(%an) %s |%h" > ChangeLog', intern = TRUE)
# file.copy("NEWS", "NEWS.md",overwrite = TRUE)
# devtools::check_win_devel()
# pkgdown::build_home()
# pkgdown::build_site(run_dont_run = TRUE)
# pkgdown::build_site()
# release()

## $Id: plotmeans.R 1912 2015-01-02 20:10:21Z warnes $

plotmeans  <- function (formula, data = NULL, subset, na.action,
                        bars=TRUE, p=0.95,
                        minsd=0, minbar=NULL, maxbar=NULL,
                        xlab=names(mf)[2], ylab=names(mf)[1],
                        mean.labels=FALSE,
                        ci.label=FALSE,
                        n.label=TRUE,
                        text.n.label="n=",
                        digits=getOption("digits"), col="black",
                        barwidth=1,
                        barcol="blue",
                        connect=TRUE,
                        ccol=col,
                        legends=names(means),
                        xaxt,
                        use.t = TRUE,
                        lwd=par("lwd"),
                        ...)
{
  if (invalid(formula) || (length(formula) != 3))
    stop("formula missing or incorrect")
  if (invalid(na.action))
    na.action <- options("na.action")

  mf <- match.call(expand.dots = FALSE)
  if (is.matrix(eval(mf$data, parent.frame())))
        mf$data <- as.data.frame(data)

  m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0L)

  mf <- mf[c(1L, m)]

  mf[[1L]] <- quote(stats::model.frame)
  mf <- eval(mf, parent.frame())
  response <- attr(attr(mf, "terms"), "response")

  ## drop unused levels in factors!!!

  wFact <- which(attr(attr(mf, "terms"),"dataClasses") == "factor")
  for(i in wFact)
    mf[,i] <- factor(mf[,i])

  means  <-  sapply(split(mf[[response]], mf[[-response]]), mean, na.rm=TRUE)
  ns     <-  sapply(sapply(split(mf[[response]], mf[[-response]]), na.omit,
                           simplify=FALSE), length )
  xlim  <-  c(0.5, length(means)+0.5)



  if(!bars)
    {
      plot( means, ..., col=col, xlim=xlim)
    }
  else
    {

      myvar  <-  function(x) var(x[!is.na(x)])
      vars <- sapply(split(mf[[response]], mf[[-response]]), myvar)

      ## apply minimum variance specified by minsd^2
      vars <- ifelse( vars < (minsd^2), (minsd^2), vars)

      if(use.t)
        ci.width  <- qt( (1+p)/2, ns-1 ) * sqrt(vars/ns)
      else
        ci.width  <- qnorm( (1+p)/2 ) * sqrt(vars/ns)

      if(length(mean.labels)==1)
        {
          if (mean.labels==TRUE)
            mean.labels  <-  format( round(means, digits=digits ))
          else if (mean.labels==FALSE)
            mean.labels  <- NULL
        }

      plotCI(x=1:length(means), y=means, uiw=ci.width, xaxt="n",
             xlab=xlab, ylab=ylab, labels=mean.labels, col=col, xlim=xlim,
             lwd=barwidth, barcol=barcol, minbar=minbar, maxbar=maxbar, ... )
      if(invalid(xaxt) || xaxt!="n")
        axis(1, at = 1:length(means), labels = legends, ...)

      if(ci.label)
        {
          ci.lower <- means-ci.width
          ci.upper <- means+ci.width

          if(!invalid(minbar))
            ci.lower <- ifelse(ci.lower < minbar, minbar, ci.lower)
          if(!invalid(maxbar))
            ci.upper <- ifelse(ci.upper > maxbar, maxbar, ci.upper)

          labels.lower <- paste( " \n", format(round(ci.lower, digits=digits)),
                                sep="")
          labels.upper <- paste( format(round(ci.upper, digits=digits)), "\n ",
                                sep="")

          text(x=1:length(means),y=ci.lower, labels=labels.lower, col=col)
          text(x=1:length(means),y=ci.upper, labels=labels.upper, col=col)
        }

    }


  if(n.label)
      {
          text(x=1:length(means),
               y=par("usr")[3],
               labels=paste(text.n.label, ns, "\n", sep="")
               )
      }

  if(!invalid(connect) & !identical(connect,FALSE))
    {
      if(is.list(connect))
        {
          if(length(ccol)==1)
            ccol  <-  rep(ccol, length(connect) )

          for(which in 1:length(connect))
            lines(x=connect[[which]],y=means[connect[[which]]],col=ccol[which])
        }
      else
        lines(means, ..., lwd=lwd, col=ccol)
    }

}

# $Id: balloonplot.R 1302 2008-08-14 20:06:22Z warnes $

balloonplot <- function(x,...)
  UseMethod("balloonplot",x)

balloonplot.table <- function(x, xlab, ylab, zlab, show.zeros = FALSE, 
                              show.margins = TRUE, ... )
  {
    obj <- x
    tmp <- as.data.frame(x)
    x <- tmp[,1]
    y <- tmp[,2]
    z <- tmp[,3]
    tableflag <- TRUE

    if(missing(xlab)) xlab <- names(dimnames(obj))[1]
    if(missing(ylab)) ylab <- names(dimnames(obj))[2]
    if(missing(zlab)) zlab <- "Freq"

    balloonplot.default(x, y, z, xlab=xlab, ylab=ylab, zlab=zlab, 
                        show.zeros = show.zeros, 
                        show.margins = show.margins, ...)
  }



balloonplot.default <- function(x,y,z,
                                xlab,
                                ylab,
                                zlab=deparse(substitute(z)),
                                dotsize=2/max(strwidth(19),strheight(19)),
                                dotchar=19,
                                dotcolor="skyblue",
                                text.size=1,
                                text.color=par("fg"),
                                main,
                                label=TRUE,
                                label.digits=2,
                                label.size=1,
                                label.color=par("fg"),
                                scale.method=c("volume","diameter"),
                                scale.range=c("absolute","relative"),
                                colsrt=par("srt"),
                                rowsrt=par("srt"),
                                colmar=1,
                                rowmar=2,
                                show.zeros=FALSE,
                                show.margins=TRUE,
                                cum.margins=TRUE,
                                sorted=TRUE,
                                label.lines=TRUE,
                                fun=function(x)sum(x,na.rm=T),
                                hide.duplicates=TRUE,
                                ... )
{

  if(is.null(names(x)))
    {
      xnames <- as.character(substitute(x))
      if(length(xnames)>1) xnames <- xnames[-1]
    }
  else
     xnames <- names(x)

  if(is.null(names(y)))
    {
      ynames <- as.character(substitute(y))
      if(length(ynames)>1) ynames <- ynames[-1]
    }
  else
     ynames <- names(y)

  ####
  ## Handle arguments
  ####
  
  scale.method <- match.arg(scale.method)
  scale.range  <- match.arg(scale.range)

  if( scale.method=="absolute" && any(z < 0, na.rm=TRUE )  )
    warning("z value(s) below zero detected.",
            " No balloons will be displayed for these cells.")
  
  if(missing(main))
    {
      if(scale.method=="volume")
        main <- paste("Balloon Plot for ",
                      paste(xnames, collapse=", "),
                      " by ",
                      paste(ynames, collapse=", "),
                      ".\nArea is proportional to ", zlab, ".", sep='')
      else
        main <- paste("Balloon Plot for ",
                      paste(ynames, collapse=", "),
                      " by ",
                      paste(ynames, collapse=", "),
                      ".\nDiameter is proportional to ", zlab, ".", sep='')
      }

  if(length(dotcolor)<length(z))
    dotcolor <- rep(dotcolor, length=length(z))

  ####
  ## Make sure x and y are lists
  ####
  
  if(is.list(x))
    {
      xlabs <- x
      x$sep=":"
      x <- do.call(paste, x)
    }
  else
      xlabs <- list(x)

  if(is.list(y))
    {
      ylabs <- y
      y$sep=":"
      y <- do.call(paste, y)
      ylab <-  paste( names(y) )      
    }
  else
    ylabs <- list(y)


  ####
  ## sort everything into a useful order
  ####
  if(sorted)
    {
      ord.x <- do.call(order, xlabs)
      ord.y <- do.call(order, ylabs)
    }
  else
    ord.x <- ord.y <- 1:length(x)


  forceOrder <- function(X, sord, lord)
    factor(X[sord], levels=unique(X[lord]))
  
  x <- forceOrder(x, ord.y, ord.y)
  y <- forceOrder(y, ord.y, ord.y)
  z <- as.numeric(z[ord.y])
  dotcolor <- dotcolor[ord.y]

  xlabs <- unique(data.frame(lapply(xlabs, forceOrder,
                                    sord=ord.y, lord=ord.y)))
  ylabs <- unique(data.frame(lapply(ylabs, forceOrder,
                                    sord=ord.y, lord=ord.y)))

  ####
  ## Function to scale circles to fill the containing box
  ####
  myscale <- function(X, min=0, max=16, scale.method, scale.range)
    {
      if(scale.method=="volume")
        {
          X[X<0] <- 0
          X <- sqrt(X)
        }

      if(scale.range=="relative")
          X <- (X-min(X, na.rm=TRUE)) # put min to 0
      X <- X / max(X, na.rm=TRUE )    # put max to 1
      X <- min + X  * (max - min)     # now to [min,max]

      cin.x <- par("cin")[1]
      cin.y <- par("cin")[2]
      if(cin.x < cin.y) X <- X * cin.x/cin.y
      X
    }

  nlabels.y <- length(ylabs)
  nlabels.x <- length(xlabs)




  ####
  ## Combine duplicate entries
  ####
  # Do twice, once for data, once for colors

  tab1 <- split( data.frame(z,dotcolor,x,y), f=list(x,y) )
  ztab <- do.call(rbind,
                  lapply(
                         tab1,
                         FUN=function(X) cbind(z=fun(X[,1]),X[1,-1])
                         )
                  )
  ####
  ## Do the plotting
  ###

  oldpar <- par("xpd","mar")
  on.exit( par(oldpar) )
  #par(xpd=NA, mar=c(1,1,5,1)+0.1)   # clip drawing to device region

  if(!show.margins)
    {
      xlim=c(-0.5,nlevels(x)+nlabels.y*rowmar-0.25)   # extra space on either
                                                      # end of plot for labels
      ylim=c(0.50,nlevels(y)+nlabels.x*colmar+1)      # and so dots don't cross
                                                      # into margins,
    }
  else
    {
      xlim=c(-0.5,nlevels(x)+nlabels.y*rowmar+1)      # extra space on either
                                                      # end of plot for labels
      ylim=c(0,nlevels(y)+nlabels.x*colmar+1)         # and so dots don't cross
                                                      # into margins,
    }

  
  plot(x=nlabels.y*rowmar+0.25 + as.numeric(ztab$x) - 1,
       y=nlevels(y) - as.numeric(ztab$y) + 1,
       cex=myscale(ztab$z, max=dotsize, scale.method=scale.method, scale.range=scale.range),
       pch=dotchar, # plot character
       col=as.character(ztab$dotcolor), # dot color
       xlab="",
       ylab="",
       xaxt="n", # no x axis lables
       yaxt="n", # no y axis lables
       bty="n",  # no box around the plot
       xaxs = "i",
       yaxs = "i",
       xlim=xlim,
       ylim=ylim,
       ...
     )

  ny <- nlevels(ztab$y)
  nx <- nlevels(ztab$x)


  sumz    <- sum(ztab$z, na.rm=TRUE)
  colsumz <- sapply(split( ztab$z, ztab$y), sum, na.rm=TRUE) # works
  rowsumz <- sapply(split( ztab$z, ztab$x), sum, na.rm=TRUE) # broken
  
  if(show.margins)
    {
      ## column totals
      text(
           x=(1:nx) + nlabels.y*rowmar + 0.25 -1,
           y=0.25,
           labels=format(c(sumz, rowsumz), digits=label.digits)[-1],
           font=1,
           adj=c(0.5,0.0),
           col=text.color,
           cex=text.size
           )

      ## row totals
      rowlabs <- format(c(sumz, colsumz), digits=label.digits)[-1]
      width <- max(strwidth(rowlabs),na.rm=TRUE)
      text(
           x=nx + nlabels.y*rowmar-0.25+width,
           y= (ny:1),
           labels=rowlabs,
           font=1,
           adj=c(1.0,0.5),
           col=text.color,
           cex=text.size           
           )

      ## overall total
      text(
           x=nx + nlabels.y*rowmar-0.25+width,
           y=0.25,
           labels=sumz,
           font=1,
           adj=c(1.0,0.0),
           col=text.color,
           cex=text.size           
           )
    }
     
  if(cum.margins)
    {
      ## Row Sums at left
      cx <- c(0, cumsum(rowsumz) / sumz)
      rect(xleft   = nlabels.y*rowmar - 1 - 0.25 + 1:nx,
           xright  = nlabels.y*rowmar - 1 + 0.75 + 1:nx,
           ybottom = ny+0.75+cx[1:nx]*colmar*nlabels.x,
           ytop    = ny+0.75+cx[2:(nx+1)]*colmar*nlabels.x,
           col     = "lightgray",
           border  = NA)

      ## Col Sums at top
      cy <- c(0, cumsum(colsumz) / sumz)
      rect(xleft   = -0.5 +rowmar*cy[ny:1]*nlabels.y,
           xright  = -0.5 +rowmar*cy[(ny+1):2]*nlabels.y,
           ybottom = 1:ny-0.5,
           ytop    = 1:ny+0.5,
           col     = "lightgray",
           border  = NA)
      
      
      tx <- paste(levels(x),"\n[",rowsumz,"]")
      ty <- paste(levels(y),"\n[",colsumz,"]")
    }

  
  ###
  ## Horizontal borders between cells
  ###
  segments(
           x0=nlabels.y*rowmar-0.25,
           x1=nx+nlabels.y*rowmar-0.25,
           y0=(0:ny)+0.5,
           y1=(0:ny)+0.5
           )
  
  ###
  ## Vertical borders between cells
  ###
  segments(
           x0=(0:nx)+nlabels.y*rowmar-0.25,
           x1=(0:nx)+nlabels.y*rowmar-0.25,
           y0= 0.5,
           y1=ny+0.5,
           )


  if(hide.duplicates)
    undupe <- function(X) 
      {
                                        # convert duplicates into blanks
        X <- as.character(X)
        c(X[1], ifelse(X[-1] == X[-length(X)], "", X[-1]))
      }
  else
    undupe <- function(X) X

  ### 
  ## Column labels
  ###
  for(i in 1:nlabels.x)
    {
      y <- ny + 0.75 + (nlabels.x - i + .5)*colmar
      text(
           x= (1:nx) + nlabels.y*rowmar + 0.25 - 1,
           y= y,
           labels=undupe(xlabs[,i]),
           srt=colsrt,
           font=1,
           col=text.color,
           cex=text.size
           )
    }

  ### 
  ## Row labels
  ###
  for(i in 1:length(ylabs))
    {
      text(
           y=ny:1,
           x= (i-0.5)*rowmar-0.5,
           labels=undupe(ylabs[,i]),
           srt=rowsrt,
           font=1,
           col=text.color,
           cex=text.size
           )
    }

  ####
  ## Column headers for row labels
  ####
  if(missing(ylab) || length(ylab)==0)
    text(
         x=((1:length(ylabs))-0.5)*rowmar-0.5,
         y=ny+0.5,
         labels=ynames,
         srt=colsrt,
         font=2,
         adj=c(0.5,0.0),
         col=text.color,
         cex=text.size
         )
  else
    text( 
         x=((1:length(ylab))-0.5)*rowmar-0.5,
         y=ny+0.5,
         labels=ylab,
         srt=colsrt,
         font=2,
         adj=c(0.5,0.0),
         col=text.color,
         cex=text.size
         )

  ####
  ## Row headers for column labels
  ####
  if(missing(xlab) || length(xlab)==0)
    text(
         x= nlabels.y*rowmar - 0.25 - strwidth(','),
         y= ny + 0.75 + ((nlabels.x:1) - 1 + .5)*colmar,
         labels=xnames,
         srt=colsrt,
         font=2,
         adj=c(1,0.5),
         col=text.color,
         cex=text.size
         )
  else
    text(
         x= nlabels.y*rowmar - 0.25 - strwidth(','),
         y= ny + 0.75 + ((length(xlab):1) - 1 + .5)*colmar,
         labels=xlab,
         srt=colsrt,
         font=2,
         adj=c(1,0.5),
         col=text.color,
         cex=text.size
         )

  ###
  ## add borders to row and column headers
  ###
  if(label.lines)
    {
      segments(                          # left: vertical lines
               x0=(0:nlabels.y)*rowmar-0.5,
               x1=(0:nlabels.y)*rowmar-0.5,
               y0=0.5,
               y1=ny+0.5
               )
      
      segments(
               x0=nlabels.y*rowmar-0.25,        # top: horizontal lines
               x1=nlabels.y*rowmar + nx - 0.25,
               y0=(0:nlabels.x)*colmar  +ny+0.75,
               y1=(0:nlabels.x)*colmar  +ny+0.75
               )
    }


  ####
  ## annotate cells with actual values
  ####
  if(label){
    if(show.zeros) 
     indiv <- 1:length(ztab$y) 
    else 
      indiv <- which(ztab$z != 0)
    
    text(x=as.numeric(ztab$x[indiv])+ nlabels.y*rowmar - 0.75,     # as.numeric give numeric values
         y=ny - as.numeric(ztab$y[indiv]) + 1,
         labels=format(ztab$z[indiv], digits=label.digits),       # label value
         font=2,
         adj=c(0.5,0.5),
         col=label.color,
         cex=label.size
         )
  }
  # put a nice title
  title(main=main)
}

# $Id: textplot.R 1213 2007-11-01 20:20:10Z warnes $

textplot <- function(object, halign="center", valign="center", cex, ... )
  UseMethod('textplot')


textplot.default <- function(object,
                             halign=c("center","left","right"),
                             valign=c("center","top","bottom"),
                             cex, ... )
{

  if (is.matrix(object) || (is.vector(object) && length(object)>1) )
    return(textplot.matrix(object, halign, valign, cex, ... ))

  halign <- match.arg(halign)
  valign <- match.arg(valign)

  textplot.character(object, halign,  valign, cex, ...)
}


textplot.data.frame <- function(object,
                             halign=c("center","left","right"),
                             valign=c("center","top","bottom"),
                             cex, ... )
    textplot.matrix(object, halign, valign, cex, ... )


textplot.matrix <- function(object,
                            halign=c("center","left","right"),
                            valign=c("center","top","bottom"),
                            cex, cmar=2, rmar=0.5,
                            show.rownames=TRUE, show.colnames=TRUE,
                            hadj=1,
                            vadj=1,
                            mar= c(1,1,4,1)+0.1,
                            col.data=par("col"),
                            col.rownames=par("col"),
                            col.colnames=par("col"),
                            ... )
{

  if(is.vector(object))
    object <- t(as.matrix(object))
  else
    object <- as.matrix(object)

  # check dimensions of col.data, col.rownames, col.colnames
  if(length(col.data)==1)
    col.data <- matrix(col.data, nrow=nrow(object), ncol=ncol(object))
  else
    if( nrow(col.data)!=nrow(object) || ncol(col.data)!=ncol(object) )
      stop("Dimensions of 'col.data' do not match dimensions of 'object'.")

  if(length(col.rownames)==1)
      col.rownames <- rep(col.rownames, nrow(object))      

  if(length(col.colnames)==1)
    if(show.rownames)
      col.colnames <- rep(col.colnames, ncol(object)+1)
    else
      col.colnames <- rep(col.colnames, ncol(object))
  
  halign=match.arg(halign)
  valign=match.arg(valign)

  opar <- par()[c("mar","xpd","cex")]
  on.exit( par(opar) )
  par(mar=mar, xpd=FALSE )

  # setup plot area
  plot.new()
  plot.window(xlim=c(0,1),ylim=c(0,1), log = "", asp=NA)



  # add 'r-style' row and column labels if not present
  if( is.null(colnames(object) ) )
    colnames(object) <- paste( "[,", 1:ncol(object), "]", sep="" )
  if( is.null(rownames(object)) )
    rownames(object) <- paste( "[", 1:nrow(object), ",]", sep="")


  # extend the matrix to include row and column labels
  if( show.rownames )
    {
      object <- cbind( rownames(object), object )
      col.data <- cbind( col.rownames, col.data )
      
    }
  if( show.colnames )
    {
      object <- rbind( colnames(object), object )
      col.data <- rbind( col.colnames, col.data )
    }

  # set the character size
  if( missing(cex) )
    {
      cex <- 1.0
      lastloop <- FALSE
    }
  else
    {
      lastloop <- TRUE
    }

  for (i in 1:20)
    {
      oldcex <- cex

      width  <- sum(
                    apply( object, 2,
                          function(x) max(strwidth(x,cex=cex) ) )
                    ) +
                      strwidth('M', cex=cex) * cmar * ncol(object)

      height <- strheight('M', cex=cex) * nrow(object) * (1 + rmar)

      if(lastloop) break

      cex <- cex / max(width,height)

      if (abs(oldcex - cex) < 0.001)
        {
          lastloop <- TRUE
        }
    }


  # compute the individual row and column heights
  rowheight<-strheight("W",cex=cex) * (1 + rmar)
  colwidth<- apply( object, 2, function(XX) max(strwidth(XX, cex=cex)) ) +
               strwidth("W")*cmar


  width  <- sum(colwidth)
  height <- rowheight*nrow(object)

  # setup x alignment
  if(halign=="left")
    xpos <- 0
  else if(halign=="center")
    xpos <- 0 + (1-width)/2
  else #if(halign=="right")
    xpos <- 0 + (1-width)

  # setup y alignment
  if(valign=="top")
    ypos <- 1
  else if (valign=="center")
    ypos <- 1 - (1-height)/2
  else #if (valign=="bottom")
    ypos <- 0 + height

  x <- xpos
  y <- ypos

  # iterate across elements, plotting them
  xpos<-x
  for(i in 1:ncol(object)) {
    xpos <- xpos + colwidth[i]
    for(j in 1:nrow(object)) {
      ypos<-y-(j-1)*rowheight
      if( (show.rownames && i==1) || (show.colnames && j==1) )
        text(xpos, ypos, object[j,i], adj=c(hadj,vadj), cex=cex, font=2,
             col=col.data[j,i], ... )
      else
        text(xpos, ypos, object[j,i], adj=c(hadj,vadj), cex=cex, font=1,
             col=col.data[j,i], ... )
    }
  }

  par(opar)
}

textplot.character <- function (object,
                                halign = c("center", "left", "right"),
                                valign = c("center", "top", "bottom"),
                                cex, fixed.width=TRUE,
                                cspace=1,
                                lspace=1,
                                mar=c(0,0,3,0)+0.1,
                                tab.width=8,
                                ...)
  {
    object <- paste(object,collapse="\n",sep="")
    object <- replaceTabs(object, width=tab.width)

    halign = match.arg(halign)
    valign = match.arg(valign)
    plot.new()

    opar <- par()[c("mar","xpd","cex","family")]
    on.exit( par(opar) )

    par(mar=mar,xpd=FALSE )
    if(fixed.width)
        par(family="mono")

    plot.window(xlim = c(0, 1), ylim = c(0, 1), log = "", asp = NA)

    slist   <- unlist(lapply(object, function(x) strsplit(x,'\n')))
    slist   <- lapply(slist, function(x) unlist(strsplit(x,'')))

    slen    <- sapply(slist, length)
    slines  <- length(slist)

    if (missing(cex))
      {
        lastloop <- FALSE
        cex <- 1
      }
    else
      lastloop <- TRUE


    for (i in 1:20)
      {
        oldcex <- cex
        #cat("cex=",cex,"\n")
        #cat("i=",i,"\n")
        #cat("calculating width...")
        cwidth  <- max(sapply(unlist(slist), strwidth,  cex=cex)) * cspace
        #cat("done.\n")
        #cat("calculating height...")
        cheight <- max(sapply(unlist(slist), strheight, cex=cex)) * ( lspace + 0.5 )
        #cat("done.\n")

        width <- strwidth(object, cex=cex)
        height <- strheight(object, cex=cex)

        if(lastloop) break

        cex <- cex  / max(width, height)

        if (abs(oldcex - cex) < 0.001)
          {
            lastloop <- TRUE
          }

      }

    if (halign == "left")
        xpos <- 0
    else if (halign == "center")
        xpos <- 0 + (1 - width)/2
    else xpos <- 0 + (1 - width)

    if (valign == "top")
        ypos <- 1
    else if (valign == "center")
        ypos <- 1 - (1 - height)/2
    else ypos <- 1 - (1 - height)

    text(x=xpos, y=ypos, labels=object, adj=c(0,1),
             cex=cex, ...)

    par(opar)
    invisible(cex)
}

here <- function() {}

plot.dendrogram <- stats:::plot.dendrogram
environment(plot.dendrogram) <- environment(here)

plotNodeLimit <- stats:::plotNodeLimit
environment(plotNodeLimit) <- environment(here)

# .memberDend <- stats:::.memberDend
.memberDend <- function (x) 
{
    r <- attr(x, "x.member")
    if (is.null(r)) {
        r <- attr(x, "members")
        if (is.null(r)) 
            r <- 1L
    }
    r
}
environment(.memberDend) <- environment(here)

# .midDend <- stats:::.midDend
.midDend <- function (x) {
    if (is.null(mp <- attr(x, "midpoint"))) 0 else mp
}
environment(.midDend) <- environment(here)

unByteCode <- function(fun)
    {
        FUN <- eval(parse(text=deparse(fun)))
        environment(FUN) <- environment(fun)
        FUN
    }

plotNode <- unByteCode(stats:::plotNode)
environment(plotNode) <- environment(here)

## Test that plotmeans works properly when bars=F
## Bug repoted by Alan Yong, Research Geophysicist,
## US DEPARTMENT OF THE INTERIOR

suppressMessages(library(gplots))
data(state)
plotmeans(state.area ~ state.region, bars=FALSE)

# $Id: hist2d.R 1555 2012-06-07 16:08:18Z warnes $

hist2d <- function(x,
                   y=NULL,
                   nbins=200,
                   same.scale=FALSE,
                   na.rm=TRUE,
                   show=TRUE,
                   col=c("black", heat.colors(12)),
                   FUN=base::length,
                   xlab,
                   ylab,
                   ... )
  {
    if(is.null(y))
      {
        if(ncol(x) != 2) stop("If y is ommitted, x must be a 2 column matirx")
        y <- x[,2]
        x <- x[,1]
      }

    if(length(nbins)==1)
      nbins <- rep(nbins,2)

    nas <- is.na(x) | is.na(y)

    if(na.rm)
      {
        x <- x[!nas]
        y <- y[!nas]
      }
    else
      stop("missinig values not permitted if na.rm=FALSE")

    if(same.scale)
      {
        x.cuts <- seq( from=min(x,y), to=max(x,y), length=nbins[1]+1)
        y.cuts <- seq( from=min(x,y), to=max(x,y), length=nbins[2]+1)
      }
    else
      {
        x.cuts <- seq( from=min(x), to=max(x), length=nbins[1]+1)
        y.cuts <- seq( from=min(y), to=max(y), length=nbins[2]+1)
      }

    index.x <- cut( x, x.cuts, include.lowest=TRUE)
    index.y <- cut( y, y.cuts, include.lowest=TRUE)

    ## tapply is faster than old for() loop, and allows
    ## use of any user-specified summary function 
    m <- tapply(x,list(index.x,index.y),FUN)

    ## If we're using length, set empty cells to 0 instead of NA
    if(identical(FUN,base::length))
      m[is.na(m)] <- 0

    if(missing(xlab)) xlab <- deparse(substitute(xlab))
    if(missing(ylab)) ylab <- deparse(substitute(ylab))
    
    if(show)
      image( x.cuts, y.cuts, m, col=col, xlab=xlab, ylab=ylab, ...)

    midpoints <- function(x) (x[-1]+x[-length(x)])/2
    
    retval <- list()
    retval$counts <- m
    retval$x.breaks = x.cuts
    retval$y.breaks = y.cuts
    retval$x = midpoints(x.cuts)
    retval$y = midpoints(y.cuts)
    retval$nobs = length(x)
    retval$call <- match.call()
    class(retval) <- "hist2d"
    retval
  }

# $Id: hist2d.R 1471 2011-08-16 01:03:31Z warnes $

print.hist2d <- function(x, ...)
  {
    cat("\n")
    cat("----------------------------\n")    
    cat("2-D Histogram Object\n")
    cat("----------------------------\n")    
    cat("\n")
    cat("Call: ")
    print(x$call)
    cat("\n")
    cat("Number of data points: ", x$nobs, "\n")
    cat("Number of grid bins: ", length(x$x), "x", length(x$y), "\n")
    cat("X range: (", min(x$x.breaks), ",", max(x$x.breaks), ")\n")
    cat("Y range: (", min(x$y.breaks), ",", max(x$y.breaks), ")\n")        
    cat("\n")
  }

adjust_hsv <- function(col, h=NULL, s=NULL, v=NULL, alpha=NULL)
{
  # Recursion if vector of colors is passed
  if(length(col) > 1)
  {
    unname(sapply(col, adjust_hsv, h=h, s=s, v=v, alpha=alpha))
  }
  else
  {
    # Convert color to HSV space
    col.rgb <- col2rgb(col, alpha=TRUE)
    col.hsv <- rgb2hsv(col.rgb[1:3,])

    # Adjust h, s, v, a
    h <- if(is.null(h)) col.hsv["h",] else h
    s <- if(is.null(s)) col.hsv["s",] else s
    v <- if(is.null(v)) col.hsv["v",] else v
    a <- if(is.null(alpha)) col.rgb["alpha",]/255 else alpha

    # Convert new color to hexadecimal RGB format
    col.new <- hsv(h, s, v, a)

    # Remove alpha if original color did not have it and user didn't specify one
    if(!grepl("#.{8}", col) && is.null(alpha))
      col.new <- substring(col.new, 1, 7)

    col.new
  }
}

# $Id: ci2d.R 1471 2011-08-16 01:03:31Z warnes $

## first(...) selects the first element of which(...)
first <- function(x,...)
  {
  w <- which(x,...)
  if(length(x)>1)
    w[1]
  else
    w
}

## first(...) selects the first element of which(...)
last <- function(x,...)
  {
  w <- which(x,...)
  if(length(x)>1)
    rev(w)[1]
  else
    w
}

## non-parametric 2 dimensional approximate confidence interval
ci2d <- function(x,
                 y = NULL,
                 nbins=51,
                 method=c("bkde2D","hist2d"),
                 bandwidth,
                 factor=1.0,
                 
                 ci.levels=c(0.50,0.75,0.90,0.95,0.975),
                 
                 show=c("filled.contour","contour","image","none"),
                 col=topo.colors(length(breaks)-1),
                 show.points=FALSE,
                 pch=par("pch"),
                 points.col="red",
                 xlab, ylab, 
                 ...)
  {

    show <- match.arg(show)
    method <- match.arg(method)
    breaks <- unique(c(0, ci.levels, 1.0))

    # get labels for x and y
    if (missing(xlab)) 
      xlab <- if (missing(x)) 
        ""
      else deparse(substitute(x))
    if (missing(ylab)) 
      ylab <- if (missing(y)) 
        ""
      else deparse(substitute(y))

    if(!is.null(y))
      x <- cbind(x,y)

    if(method=="hist2d")
      {
        h2d <- hist2d(x,
                      show=FALSE,
                      nbins=nbins,
                      ...)
        ## normalize
        h2d$density <- h2d$counts / sum(h2d$counts, na.rm=TRUE)
      }
    else if (method=="bkde2D")
      {
        if(length(nbins)==1)
          nbins <- c(nbins, nbins)

        if(missing(bandwidth))
          {
            h.x = dpik(x[,1])
            h.y = dpik(x[,2])
            bandwidth <-  c(h.x, h.y)
          }

        est <- bkde2D(x,
                      bandwidth=bandwidth*factor,
                      gridsize=nbins,
                      ...
                      )

        h2d <- list()
        h2d$x <- est$x1
        h2d$y <- est$x2
        h2d$counts <- est$fhat
        h2d$nobs <- nrow(x)
        h2d$density <- est$fhat / sum(est$fhat) # normalize
      }
    else
      stop("Unknown method: '", method, "'")

    uniqueVals <- rev(unique(sort(h2d$density)))
    cumProbs <- sapply(uniqueVals,
                       function(val) sum( h2d$density[h2d$density>=val] ) )
    names(cumProbs) <- uniqueVals
    h2d$cumDensity <- matrix(nrow=nrow(h2d$density), ncol=ncol(h2d$density))
    h2d$cumDensity[] <- cumProbs[as.character(h2d$density)]

    if(show=="image")
      {
        image(h2d$x, h2d$y, h2d$cumDensity,
              xlab=xlab, ylab=ylab,
              breaks=breaks, col=col)
        if(show.points)
            points(x[,1], x[,2], pch=pch, col=points.col);          
      }
    else if(show=="filled.contour")
      {
        if(show.points)
          plot.title <- function() {
            points(x[,1], x[,2], pch=pch, col=points.col);
          }
        else
          plot.title <- function() {}

        
        filled.contour(h2d$x, h2d$y, h2d$cumDensity,
                       levels=breaks,
                       col=col,
                       xlab=xlab,
                       ylab=ylab,
                       plot.title=plot.title(),
                       key.title=title("\nCI Level"),
                       key.axes=axis(4, at=breaks)
                       )
       }
    else if(show=="contour")
      {
        tmpBreaks <- breaks[breaks<1]  # avoid having 1.0 line
        contour(h2d$x, h2d$y, h2d$cumDensity,
                levels=tmpBreaks,
                labels=tmpBreaks,
                xlab=xlab,
                ylab=ylab,
                nlevels=length(tmpBreaks),
                col=col
                )
        if(show.points)
            points(x[,1], x[,2], pch=pch, col=points.col);          
      }

    h2d$contours <- contourLines(h2d$x, h2d$y, h2d$cumDensity,
                                 levels=breaks, nlevels=length(breaks))

    # use the confidence level value as the name in the contour list
    names(h2d$contours) <- sapply(h2d$contours, function(x) x$level)

    # convert each contour into a (x,y) dataframe 
    h2d$contours <- lapply( h2d$contours,
                            function(J) data.frame(x=J$x, y=J$y) )

    h2d$call <- match.call()
    class(h2d) <- "ci2d"
    
    invisible(h2d)
  }

# Revision 2.1 2005/06/06
# - Modified default behavior with 0's and NA's in
#   'height' so that these values are not plotted.
# - Warning messages added in the case of the above.

# Revision 2.0 2005/04/27
# - Added panel.first and panel.last arguments
# - As per R 2.0.0, the default barplot() method by default uses a
#   gamma-corrected grey palette (rather than the heat color
#   palette) for coloring its output when given a matrix.

barplot2 <- function(height, ...) UseMethod("barplot2")

barplot2.default <-
    function(
        height,
        width = 1,
        space = NULL,
        names.arg = NULL,
        legend.text = NULL,
        beside = FALSE,
        horiz = FALSE,
        density = NULL,
        angle = 45,
        col = NULL,
        prcol = NULL,
        border = par("fg"),
        main = NULL,
        sub = NULL,
         xlab = NULL,
         ylab = NULL,

        xlim = NULL,
        ylim = NULL,
        xpd = TRUE,
        log = "",

        axes = TRUE,
        axisnames = TRUE,

        cex.axis = par("cex.axis"),
        cex.names = par("cex.axis"),

        inside = TRUE,
        plot = TRUE,
        axis.lty = 0,
        offset = 0,

        plot.ci = FALSE,
        ci.l = NULL,
        ci.u = NULL,

        ci.color = "black",
        ci.lty = "solid",
        ci.lwd = 1,
        ci.width = 0.5,

        plot.grid = FALSE,
        grid.inc = NULL,

        grid.lty = "dotted",
        grid.lwd = 1,
        grid.col = "black",

        add = FALSE,
        panel.first = NULL,
        panel.last = NULL,
        ...)
{
    if (!missing(inside)) .NotYetUsed("inside", error = FALSE)# -> help(.)

    if (missing(space))
      space <- if (is.matrix(height) && beside) c(0, 1) else 0.2
    space <- space * mean(width)

    if (plot && axisnames && missing(names.arg))
      names.arg <-
          if(is.matrix(height)) colnames(height) else names(height)

    if (is.vector(height)
        || (is.array(height) && (length(dim(height)) == 1))) {
        ## Treat vectors and 1-d arrays the same.
	height <- cbind(height)
	beside <- TRUE
        ## The above may look strange, but in particular makes color
        ## specs work as most likely expected by the users.
        if(is.null(col)) col <- "grey"
    } else if (is.matrix(height)) {
        ## In the matrix case, we use " heat colors" by default.
        if(is.null(col)) col <- heat.colors(nrow(height))
    }
    else
	stop(paste(sQuote("height"), "must be a vector or a matrix"))

    if(is.logical(legend.text))
      legend.text <-
        if(legend.text && is.matrix(height)) rownames(height)

    # Check for log scales
    logx <- FALSE
    logy <- FALSE

    if (log != "")
    {
      if (any(grep("x", log)))
        logx <- TRUE

      if (any(grep("y", log)))
        logy <- TRUE
    }

    # Cannot "hatch" with rect() when log scales used
    if ((logx || logy) && !is.null(density))
      stop("Cannot use shading lines in bars when log scale is used")

    NR <- nrow(height)
    NC <- ncol(height)

    if (beside) {
	if (length(space) == 2)
	    space <- rep.int(c(space[2], rep.int(space[1], NR - 1)), NC)
	width <- rep(width, length.out = NR)
    } else
	width <- rep(width, length.out = NC)

    offset <- rep(as.vector(offset), length.out = length(width))

    delta <- width / 2
    w.r <- cumsum(space + width)
    w.m <- w.r - delta
    w.l <- w.m - delta

    #if graphic will be stacked bars, do not plot ci
    if (!beside && (NR > 1) && plot.ci)
      plot.ci = FALSE

    # error check ci arguments
    if (plot && plot.ci)
    {
      if ((missing(ci.l)) || (missing(ci.u)))
        stop("confidence interval values are missing")

      if (is.vector(ci.l)
        || (is.array(ci.l) && (length(dim(ci.l)) == 1)))
        ci.l <- cbind(ci.l)
      else if (!is.matrix(ci.l))
        stop(paste(sQuote("ci.l"), "must be a vector or a matrix"))

      if (is.vector(ci.u)
        || (is.array(ci.u) && (length(dim(ci.u)) == 1)))
        ci.u <- cbind(ci.u)
      else if (!is.matrix(ci.u))
        stop(paste(sQuote("ci.u"), "must be a vector or a matrix"))

      if (any(dim(height) != dim(ci.u)))
        stop(paste(sQuote("height"), "and", sQuote("ci.u"),
                   "must have the same dimensions."))
      else if (any(dim(height) != dim(ci.l)))
        stop(paste(sQuote("height"), "and", sQuote("ci.l"),
                   "must have the same dimensions."))
    }

    # check height + offset/ci.l if using log scale to prevent log(<=0) error
    # adjust appropriate ranges and bar base values
    if ((logx && horiz) || (logy && !horiz))
    {
      # Check for NA values and issue warning if required
      height.na <- sum(is.na(height))
      if (height.na > 0)
      {
         warning(sprintf("%.0f values == NA in 'height' omitted from logarithmic plot",
                          height.na), domain = NA)
      }

      # Check for 0 values and issue warning if required
      # _FOR NOW_ change 0's to NA's so that other calculations are not
      # affected. 0's and NA's affect plot output in the same way anyway,
      # except for stacked bars, so don't change those.
      height.lte0 <- sum(height <= 0, na.rm = TRUE)
      if (height.lte0 > 0)
      {
        warning(sprintf("%0.f values <=0 in 'height' omitted from logarithmic plot",
                         height.lte0), domain = NA)

        # If NOT stacked bars, modify 'height'
        if (beside)
          height[height <= 0] <- NA
      }

      if (plot.ci && (min(ci.l) <= 0))
        stop("log scale error: at least one lower c.i. value <= 0")

      if (logx && !is.null(xlim) && (xlim[1] <= 0))
        stop("log scale error: 'xlim[1]' <= 0")

      if (logy && !is.null(ylim) && (ylim[1] <= 0))
        stop("'log scale error: 'ylim[1]' <= 0")

      # arbitrary adjustment to display some of bar for min(height) since
      # 0 cannot be used with log scales. If plot.ci, also check ci.l
      if (plot.ci)
      {
        rectbase <- c(height[is.finite(height)], ci.l)
        rectbase <- min(0.9 * rectbase[rectbase > 0])
      }
      else
      {
        rectbase <- height[is.finite(height)]
        rectbase <- min(0.9 * rectbase[rectbase > 0])
      }

      # if axis limit is set to < above, adjust bar base value
      # to draw a full bar
      if (logy && !is.null(ylim) && !horiz)
        rectbase <- ylim[1]
      else if (logx && !is.null(xlim) && horiz)
        rectbase <- xlim[1]

      # if stacked bar, set up base/cumsum levels, adjusting for log scale
      if (!beside)
        height <- rbind(rectbase, apply(height, 2, cumsum))

      # if plot.ci, be sure that appropriate axis limits are set to include range(ci)
      lim <-
        if (plot.ci)
          c(height, ci.l, ci.u)
        else
          height

      rangeadj <- c(0.9 * lim + offset, lim + offset)
      rangeadj <- rangeadj[rangeadj > 0]
    }
    else
    {
      # Use original bar base value
      rectbase <- 0

      # if stacked bar, set up base/cumsum levels
      if (!beside)
        height <- rbind(rectbase, apply(height, 2, cumsum))

      # if plot.ci, be sure that appropriate axis limits are set to include range(ci)
      lim <-
        if (plot.ci)
          c(height, ci.l, ci.u)
        else
          height

      # use original range adjustment factor
      rangeadj <- c(-0.01 * lim + offset, lim + offset)
    }

    # define xlim and ylim, adjusting for log-scale if needed
    if (horiz)
    {
      if (missing(xlim)) xlim <- range(rangeadj, na.rm=TRUE)
      if (missing(ylim)) ylim <- c(min(w.l), max(w.r))
    }
    else
    {
      if (missing(xlim)) xlim <- c(min(w.l), max(w.r))
      if (missing(ylim)) ylim <- range(rangeadj, na.rm=TRUE)
    }

    if (beside)
      w.m <- matrix(w.m, ncol = NC)

    if(plot) ##-------- Plotting :
    {
      opar <-
        if (horiz) par(xaxs = "i", xpd = xpd)
        else       par(yaxs = "i", xpd = xpd)
      on.exit(par(opar))

      # If add = FALSE open new plot window
      # else allow for adding new plot to existing window
      if (!add)
      {
        plot.new()
        plot.window(xlim, ylim, log = log, ...)
      }

      # Execute the panel.first expression. This will work here
      # even if 'add = TRUE'
      panel.first

      # Set plot region coordinates
      usr <- par("usr")

      # adjust par("usr") values if log scale(s) used
      if (logx)
      {
        usr[1] <- 10 ^ usr[1]
        usr[2] <- 10 ^ usr[2]
      }

      if (logy)
      {
        usr[3] <- 10 ^ usr[3]
        usr[4] <- 10 ^ usr[4]
      }

      # if prcol specified, set plot region color
      if (!missing(prcol))
        rect(usr[1], usr[3], usr[2], usr[4], col = prcol)

      # if plot.grid, draw major y-axis lines if vertical or x axis if horizontal
      # R V1.6.0 provided axTicks() as an R equivalent of the C code for
      # CreateAtVector.  Use this to determine default axis tick marks when log
      # scale used to be consistent when no grid is plotted.
      # Otherwise if grid.inc is specified, use pretty()

      if (plot.grid)
      {
        par(xpd = FALSE)

        if (is.null(grid.inc))
        {
          if (horiz)
          {
            grid <- axTicks(1)
            abline(v = grid, lty = grid.lty, lwd = grid.lwd, col = grid.col)
          }
          else
          {
            grid <- axTicks(2)
            abline(h = grid, lty = grid.lty, lwd = grid.lwd, col = grid.col)
          }
        }
        else
        {
          if (horiz)
          {
            grid <- pretty(xlim, n = grid.inc)
            abline(v = grid, lty = grid.lty, lwd = grid.lwd, col = grid.col)
          }
          else
          {
            grid <- pretty(ylim, n = grid.inc)
            abline(h = grid, lty = grid.lty, lwd = grid.lwd, col = grid.col)
          }
        }

         par(xpd = xpd)
      }

      xyrect <- function(x1,y1, x2,y2, horizontal = TRUE, ...)
      {
        if(horizontal)
          rect(x1,y1, x2,y2, ...)
        else
          rect(y1,x1, y2,x2, ...)
      }

      if (beside)
        xyrect(rectbase + offset, w.l, c(height) + offset, w.r, horizontal=horiz,
               angle = angle, density = density, col = col, border = border)
      else
      {
        for (i in 1:NC)
          xyrect(height[1:NR, i] + offset[i], w.l[i], height[-1, i] + offset[i], w.r[i],
                 horizontal=horiz, angle = angle, density = density,
                 col = col, border = border)
      }

      # Execute the panel.last expression here
      panel.last

      if (plot.ci)
      {
        # CI plot width = barwidth / 2
        half.ci.width = width * ci.width / 2

        if (horiz)
        {
          segments(ci.l, w.m, ci.u, w.m, col = ci.color, lty = ci.lty, lwd = ci.lwd)
          segments(ci.l, w.m - half.ci.width, ci.l, w.m + half.ci.width, col = ci.color, lty = ci.lty, lwd = ci.lwd)
          segments(ci.u, w.m - half.ci.width, ci.u, w.m + half.ci.width, col = ci.color, lty = ci.lty, lwd = ci.lwd)
        }
        else
        {
          segments(w.m, ci.l, w.m, ci.u, col = ci.color, lty = ci.lty, lwd = ci.lwd)
          segments(w.m - half.ci.width, ci.l, w.m + half.ci.width, ci.l, col = ci.color, lty = ci.lty, lwd = ci.lwd)
          segments(w.m - half.ci.width, ci.u, w.m + half.ci.width, ci.u, col = ci.color, lty = ci.lty, lwd = ci.lwd)
        }
      }

      if (axisnames && !is.null(names.arg)) # specified or from {col}names
      {
        at.l <-
        if (length(names.arg) != length(w.m))
        {
          if (length(names.arg) == NC) # i.e. beside (!)
            colMeans(w.m)
          else
            stop("incorrect number of names")
        }
        else w.m

        axis(if(horiz) 2 else 1, at = at.l, labels = names.arg, lty = axis.lty, cex.axis = cex.names, ...)
      }

      if(!is.null(legend.text))
      {
        legend.col <- rep(col, length = length(legend.text))

        if((horiz & beside) || (!horiz & !beside))
        {
          legend.text <- rev(legend.text)
          legend.col <- rev(legend.col)
          density <- rev(density)
          angle <- rev(angle)
        }

        # adjust legend x and y values if log scaling in use
        if (logx)
          legx <- usr[2] - ((usr[2] - usr[1]) / 10)
        else
          legx <- usr[2] - xinch(0.1)

        if (logy)
          legy <- usr[4] - ((usr[4] - usr[3]) / 10)
        else
          legy <- usr[4] - yinch(0.1)

        legend(legx, legy,
        legend = legend.text, angle = angle, density = density,
        fill = legend.col, xjust = 1, yjust = 1)
      }

      title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)

      # if axis is to be plotted, adjust for grid "at" values
      if (axes)
      {
        if(plot.grid)
          axis(if(horiz) 1 else 2, at = grid, cex.axis = cex.axis, ...)
        else
          axis(if(horiz) 1 else 2, cex.axis = cex.axis, ...)
      }

      invisible(w.m)

    }

    else w.m
}

# $Id: boxplot2.R 2052 2015-06-02 19:03:20Z warnes $

boxplot2  <- function( ..., top=FALSE, shrink=1.0, textcolor=NULL )
  {
    boxcall <- match.call()           # get call
    boxcall$top  <- boxcall$shrink  <- boxcall$textcolor  <- NULL
    boxcall[[1]]  <- as.name("boxplot")

    box <- eval(boxcall, parent.frame())
    mids <- 1:length(box$n)

    if(top)
      {
        where  <- par("usr")[4]
        adj  <- c(0.5,1)
      }
    else
      {
        where  <- par("usr")[3]
        adj  <- c(0.5,0)
      }
    tcex <- par("cex")
    par(cex=shrink*tcex)

    text(x=mids, y=where, labels=paste("n=",box$n,sep=""), adj=adj,
         col=textcolor)

    par(cex=tcex)

    invisible(box)
  }

lmplot2 <- function(
                    x,
                    which = 1:5,
                    caption = c("Residuals vs Fitted", "Normal Q-Q plot",
                                "Scale-Location plot", "Cook's distance plot"),
                    panel = panel.smooth,
                    sub.caption = deparse(x$call),
                    main = "",
                    ask = interactive() && nb.fig < length(which)
                    && .Device != "postscript",
                    ...,
                    id.n = 3,
                    labels.id = names(residuals(x)),
                    cex.id = 0.75,
                    band=TRUE,
                    rug=TRUE,
                    width=1/10,
                    max.n=5000
                    )
{
  if (!inherits(x, "lm"))
    stop("Use only with 'lm' objects")
  show <- rep(FALSE, 5)
  if(!is.numeric(which) || any(which < 1) || any(which > 5))
    stop("`which' must be in 1:5")
  show[which] <- TRUE
  r <- residuals(x)
  n <- length(r)
  if(inherits(x,"glm"))
    yh <- predict(x) # != fitted() for glm
  else
    yh <- fitted(x)
  if (any(show[2:4]))
    s <- if(inherits(x, "rlm")) x$s else sqrt(deviance(x)/df.residual(x))
  if (any(show[2:3]))
    {
      ylab23 <- if(inherits(x, "glm"))
        "Std. deviance resid." else "Standardized residuals"
      hii <- lm.influence(x)$hat
      w <- weights(x)
                                        # r.w := weighted.residuals(x):
      r.w <- if(is.null(w)) r else (sqrt(w)*r)[w!=0]
      rs <- r.w/(s * sqrt(1 - hii))
    }
  if (any(show[c(1,3)]))
    l.fit <- if(inherits(x,"glm"))
      "Predicted values" else "Fitted values"
  if (is.null(id.n))
    id.n <- 0
  else {
    id.n <- as.integer(id.n)
    if(id.n < 0 || id.n > n)
      stop(paste("`id.n' must be in { 1,..,",n,"}"))
  }
  if(id.n > 0) {
    if(is.null(labels.id))
      labels.id <- paste(1:n)
    iid <- 1:id.n
    show.r <- order(-abs(r))[iid]
    if(any(show[2:3]))
      show.rs <- order(-abs(rs))[iid]
    text.id <- function(x,y, ind, adj.x = FALSE)
      text(x - if(adj.x) strwidth(" ")*cex.id else 0, y, labels.id[ind],
           cex = cex.id, xpd = TRUE, adj = if(adj.x) 1)
  }
  nb.fig <- prod(par("mfcol"))
  one.fig <- prod(par("mfcol")) == 1
  if (ask) {
    op <- par(ask = TRUE)
    on.exit(par(op))
  }

  ##---------- Do the individual plots : ----------
  if (show[1]) {
    ylim <- range(r)
    if(id.n > 0)
      ylim <- ylim + c(-1,1)* 0.08 * diff(ylim)
    plot(yh, r, xlab = l.fit, ylab = "Residuals", main = main,
         ylim = ylim, type = "n", ...)
    panel(yh, r, ...)
    if(rug)  rug(yh)                                 ## GRW 2001-06-08
    if(band) bandplot(yh,r,add=TRUE,width=width)    ## GRW 2001-06-08
    if (one.fig)
      title(sub = sub.caption, ...)
    mtext(caption[1], 3, 0.25)
    if(id.n > 0) {
      y.id <- r[show.r]
      y.id[y.id < 0] <- y.id[y.id < 0] - strheight(" ")/3
      text.id(yh[show.r], y.id, show.r, adj.x = TRUE)
    }
    abline(h = 0, lty = 3, col = "gray")
  }
  if (show[2]) {
    ylim <- range(rs)
    ylim[2] <- ylim[2] + diff(ylim) * 0.075
    qq <- qqnorm(rs, main = main, ylab = ylab23, ylim = ylim, ...)
    qqline(rs)
    if (one.fig)
      title(sub = sub.caption, ...)
    mtext(caption[2], 3, 0.25)
    if(id.n > 0)
      text.id(qq$x[show.rs], qq$y[show.rs], show.rs, adj.x = TRUE)
  }
  if (show[3]) {
    sqrtabsr <- sqrt(abs(rs))
    ylim <- c(0, max(sqrtabsr))
    yl <- as.expression(substitute(sqrt(abs(YL)), list(YL=as.name(ylab23))))
    yhn0 <- if(is.null(w)) yh else yh[w!=0]
    plot(yhn0, sqrtabsr, xlab = l.fit, ylab = yl, main = main,
         ylim = ylim, type = "n", ...)
    panel(yhn0, sqrtabsr, ...)

    abline(h=mean(sqrtabsr),lty = 3, col = "gray")
    if(rug)  rug(yh)                             ## GRW 2001-06-08
    if(band) bandplot(yhn0,sqrtabsr,add=TRUE) ## GRW 2001-06-08

    if (one.fig)
      title(sub = sub.caption, ...)
    mtext(caption[3], 3, 0.25)
    if(id.n > 0)
      text.id(yhn0[show.rs], sqrtabsr[show.rs], show.rs, adj.x = TRUE)
  }
  if (show[4]) {
    cook <- cooks.distance(x, sd=s)
    if(id.n > 0) {
      show.r <- order(-cook)[iid]# index of largest `id.n' ones
      ymx <- cook[show.r[1]] * 1.075
    } else ymx <- max(cook)
    plot(cook, type = "h", ylim = c(0, ymx), main = main,
         xlab = "Obs. number", ylab = "Cook's distance", ...)
    if (one.fig)
      title(sub = sub.caption, ...)
    mtext(caption[4], 3, 0.25)
    if(id.n > 0)
      text.id(show.r, cook[show.r] + 0.4*cex.id * strheight(" "), show.r)
  }

  if (show[5])
  {
    ## plot residuals against each predictor ##
    data  <- model.frame(x)
    for( i in 1:ncol(data) )
    {
      test <- try(
      {
        plot.default( x=data[,i], y=r,
                     xlab=names(data)[i], ylab="Residuals", type="n")
        panel( data[,i], r, ... )
        if(rug)  rug(data[,i])
        if(band) bandplot(data[,i],r,add=TRUE)
        abline(h=0,lty = 3, col = "gray")
      }
      )
    }
  }

  if (!one.fig && par("oma")[3] >= 1)
    mtext(sub.caption, outer = TRUE, cex = 1.25)
  invisible()
}

plotLowess <- function (formula, data=parent.frame(), ..., subset=parent.frame(),
                        col.lowess="red",
                        lty.lowess=2  )
{
  m <- match.call(expand.dots=TRUE)
  m[[1]] <- as.name("plot")
  eframe <- parent.frame()
  eval(m, eframe)
  m[[1]] <- as.name("lowess")
  lw <- eval(m, eframe)
  lines(lw, col=col.lowess, lty=lty.lowess)
  grid()

  invisible(lw)
}

plot.lowess <- function(x, y, ..., col.lowess="red", lty.lowess=2)
{
  m <- x$call
  m[[1]] <- quote(plot)
  m <- as.call(append(as.list(m), list(...)))
  eval(m,  envir = parent.frame())

  lines(x$x, x$y, col=col.lowess, lty=lty.lowess)
  grid()

  invisible(x)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  echo = TRUE,
  warning = FALSE,
  message = FALSE,
  fig.width = 8,
  fig.height = 6,
  fig.align = "center"
)

## ----load-libraries-----------------------------------------------------------
# Load required libraries
library(gplots)  # for heatmap.2
library(RColorBrewer)  # for better color palettes

## -----------------------------------------------------------------------------
data(mtcars)
x  <- as.matrix(mtcars)
rc <- rainbow(nrow(x), start=0, end=.3)
cc <- rainbow(ncol(x), start=0, end=.3)


## -----------------------------------------------------------------------------
heatmap.2(x)                    ## default - dendrogram plotted and reordering done.
heatmap.2(x, dendrogram="none") ##  no dendrogram plotted, but reordering done.
heatmap.2(x, dendrogram="row")  ## row dendrogram plotted and row reordering done.
heatmap.2(x, dendrogram="col")  ## col dendrogram plotted and col reordering done.

heatmap.2(x, keysize=2)         ## default - dendrogram plotted and reordering done.

heatmap.2(x, Rowv=FALSE, dendrogram="both") ## generates a warning!
heatmap.2(x, Rowv=NULL, dendrogram="both")  ## generates a warning!
heatmap.2(x, Colv=FALSE, dendrogram="both") ## generates a warning!


## -----------------------------------------------------------------------------
heatmap.2(x, reorderfun=function(d, w) reorder(d, w, agglo.FUN = mean) )


## ----fig.keep='none'----------------------------------------------------------
full <- heatmap.2(x) # we use it to easily get the dendrograms

## -----------------------------------------------------------------------------
library(dendextend)  # for color_branches
heatmap.2(x, 
        Rowv=color_branches(full$rowDendrogram, k = 3),
        Colv=color_branches(full$colDendrogram, k = 2))
# Look at the vignette for more details:
# https://cran.r-project.org/web/packages/dendextend/vignettes/dendextend.html


## -----------------------------------------------------------------------------
full <- heatmap.2(x)
heatmap.2(x, Colv=full$colDendrogram[[2]], breaks=full$breaks)  # column subset
heatmap.2(x, Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # row subset
heatmap.2(x, Colv=full$colDendrogram[[2]],
            Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # both



## -----------------------------------------------------------------------------

heatmap.2(x, srtCol=NULL)
heatmap.2(x, srtCol=0,   adjCol = c(0.5,1) )
heatmap.2(x, srtCol=45,  adjCol = c(1,1)   )
heatmap.2(x, srtCol=135, adjCol = c(1,0)   )
heatmap.2(x, srtCol=180, adjCol = c(0.5,0) )
heatmap.2(x, srtCol=225, adjCol = c(0,0)   ) ## not very useful
heatmap.2(x, srtCol=270, adjCol = c(0,0.5) )
heatmap.2(x, srtCol=315, adjCol = c(0,1)   )
heatmap.2(x, srtCol=360, adjCol = c(0.5,1) )

heatmap.2(x, srtRow=45, adjRow=c(0, 1) )
heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=45, adjCol=c(1,1) )
heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=270, adjCol=c(0,0.5) )


## -----------------------------------------------------------------------------
## not also present)
heatmap.2(x, offsetRow=0, offsetCol=0)
heatmap.2(x, offsetRow=1, offsetCol=1)
heatmap.2(x, offsetRow=2, offsetCol=2)
heatmap.2(x, offsetRow=-1, offsetCol=-1)

heatmap.2(x, srtRow=0, srtCol=90, offsetRow=0, offsetCol=0)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=1, offsetCol=1)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=2, offsetCol=2)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=-1, offsetCol=-1)


## -----------------------------------------------------------------------------
lmat <- rbind( c(5,3,4), c(2,1,4) )
lhei <- c(1.5, 4)
lwid <- c(1.5, 4, 0.75)

myplot <- function() {
           oldpar <- par("mar")
           par(mar=c(5.1, 4.1, 0.5, 0.5))
           plot(mpg ~ hp, data=x)
         }

heatmap.2(x, lmat=lmat, lhei=lhei, lwid=lwid, key=FALSE, extrafun=myplot)


## -----------------------------------------------------------------------------

heatmap.2(x,
         key.title=NA, # no title
         key.xlab=NA,  # no xlab
         key.par=list(mgp=c(1.5, 0.5, 0),
                      mar=c(2.5, 2.5, 1, 0)),
         key.xtickfun=function() {
               breaks <- parent.frame()$breaks
               return(list(
                    at=parent.frame()$scale01(c(breaks[1],
                                                breaks[length(breaks)])),
                    labels=c(as.character(breaks[1]),
                             as.character(breaks[length(breaks)]))
                    ))
         })

heatmap.2(x,
        breaks=256,
        key.title=NA,
        key.xlab=NA,
        key.par=list(mgp=c(1.5, 0.5, 0),
                     mar=c(1, 2.5, 1, 0)),
        key.xtickfun=function() {
             cex <- par("cex")*par("cex.axis")
             side <- 1
             line <- 0
             col <- par("col.axis")
             font <- par("font.axis")
             mtext("low", side=side, at=0, adj=0,
                   line=line, cex=cex, col=col, font=font)
             mtext("high", side=side, at=1, adj=1,
                   line=line, cex=cex, col=col, font=font)
             return(list(labels=FALSE, tick=FALSE))
        })



## -----------------------------------------------------------------------------
##
##
hv <- heatmap.2(x, col=bluered, scale="column", tracecol="#303030")

###
## Look at the return values
###
names(hv)

## Show the mapping of z-score values to color bins
hv$colorTable

## Extract the range associated with white
hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",]

## Determine the original data values that map to white
whiteBin <- unlist(hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",1:2])
rbind(whiteBin[1] * hv$colSDs + hv$colMeans,
     whiteBin[2] * hv$colSDs + hv$colMeans )
##
## A more decorative heatmap, with z-score scaling along columns
##
hv <- heatmap.2(x, col=cm.colors(255), scale="column",
       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
       xlab="specification variables", ylab= "Car Models",
       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
       tracecol="green", density="density")
## Note that the breakpoints are now symmetric about 0

## Color the labels to match RowSideColors and ColSideColors
hv <- heatmap.2(x, col=cm.colors(255), scale="column",
       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
       xlab="specification variables", ylab= "Car Models",
       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
       tracecol="green", density="density", colRow=rc, colCol=cc,
       srtCol=45, adjCol=c(0.5,1))




## -----------------------------------------------------------------------------
# TODO: want example using the `add.exp' argument!

data(attitude)
round(Ca <- cor(attitude), 2)
symnum(Ca) # simple graphic

# with reorder
heatmap.2(Ca, 		 symm=TRUE, margin=c(6, 6), trace="none" )

# without reorder
heatmap.2(Ca, Rowv=FALSE, symm=TRUE, margin=c(6, 6), trace="none" )

## Place the color key below the image plot
heatmap.2(x, lmat=rbind( c(0, 3), c(2,1), c(0,4) ), lhei=c(1.5, 4, 2 ) )

## Place the color key to the top right of the image plot
heatmap.2(x, lmat=rbind( c(0, 3, 4), c(2,1,0 ) ), lwid=c(1.5, 4, 2 ) )


## -----------------------------------------------------------------------------
data(USJudgeRatings)
symnum( cU <- cor(USJudgeRatings) )

hU <- heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=topo.colors(16),
            distfun=function(c) as.dist(1 - c), trace="none")

## The Correlation matrix with same reordering:
hM <- format(round(cU, 2))
hM


## -----------------------------------------------------------------------------

heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=rev(heat.colors(16)),
           distfun=function(c) as.dist(1 - c), trace="none",
           cellnote=hM)


## -----------------------------------------------------------------------------
data(mtcars)
x  <- as.matrix(mtcars)

library(heatmaply)
# just use heatmaply instead of heatmap.2
heatmaply(x)


## ----fig.keep='none'----------------------------------------------------------
full <- heatmap.2(x) # we use it to easily get the dendrograms

## -----------------------------------------------------------------------------
## Color branches of dendrograms by cluster membership using dendextend:
heatmaply(x, 
        Rowv=color_branches(full$rowDendrogram, k = 3),
        Colv=color_branches(full$colDendrogram, k = 2))
# Look at the vignette for more details:
# https://cran.r-project.org/web/packages/heatmaply/vignettes/heatmaply.html


## ----session-info-------------------------------------------------------------
sessionInfo()

# $Id: smartlegend.R 2089 2016-03-24 13:47:48Z warnes $

smartlegend <- function(x=c("left","center","right"),
                        y=c("top","center","bottom"),
                        ..., inset=0.05 )
    .Defunct('legend', 'gplots')

## $Id: heatmap.2.R 2103 2016-03-25 17:11:26Z warnes $

heatmap.2 <- function (x,

                       ## dendrogram control
                       Rowv = TRUE,
                       Colv=if(symm)"Rowv" else TRUE,
                       distfun = dist,
                       hclustfun = hclust,
                       dendrogram = c("both","row","column","none"),
                       reorderfun = function(d, w) reorder(d, w),
                       symm = FALSE,

                       ## data scaling
                       scale = c("none","row", "column"),
                       na.rm=TRUE,

                       ## image plot
                       revC = identical(Colv, "Rowv"),
                       add.expr,

                       ## mapping data to colors
                       breaks,
                       symbreaks=any(x < 0, na.rm=TRUE) || scale!="none",

                       ## colors
                       col="heat.colors",

                       ## block sepration
                       colsep,
                       rowsep,
                       sepcolor="white",
                       sepwidth=c(0.05,0.05),

                       ## cell labeling
                       cellnote,
                       notecex=1.0,
                       notecol="cyan",
                       na.color=par("bg"),

                       ## level trace
                       trace=c("column","row","both","none"),
                       tracecol="cyan",
                       hline=median(breaks),
                       vline=median(breaks),
                       linecol=tracecol,

                       ## Row/Column Labeling
                       margins = c(5, 5),
                       ColSideColors,
                       RowSideColors,
                       cexRow = 0.2 + 1/log10(nr),
                       cexCol = 0.2 + 1/log10(nc),
                       labRow = NULL,
                       labCol = NULL,
                       srtRow = NULL,
                       srtCol = NULL,
                       adjRow = c(0,NA),
                       adjCol = c(NA,0),
                       offsetRow = 0.5,
                       offsetCol = 0.5,
                       colRow = NULL,
                       colCol = NULL,

                       ## color key + density info
                       key = TRUE,
                       keysize = 1.5,
                       density.info=c("histogram","density","none"),
                       denscol=tracecol,
                       symkey = any(x < 0, na.rm=TRUE) || symbreaks,
                       densadj = 0.25,
                       key.title = NULL,
                       key.xlab = NULL,
                       key.ylab = NULL,
                       key.xtickfun = NULL,
                       key.ytickfun = NULL,
                       key.par=list(),

                       ## plot labels
                       main = NULL,
                       xlab = NULL,
                       ylab = NULL,

                       ## plot layout
                       lmat = NULL,
                       lhei = NULL,
                       lwid = NULL,

                       ## extras
                       extrafun=NULL,
                       ...
                       )
{
  scale01 <- function(x, low=min(x), high=max(x) )
    {
      x <- (x-low)/(high - low)
      x
    }

  retval <- list()

  scale <- if(symm && missing(scale)) "none" else match.arg(scale)
  dendrogram <- match.arg(dendrogram)
  trace <- match.arg(trace)
  density.info <- match.arg(density.info)

  if(length(col)==1 && is.character(col) )
    col <- get(col, mode="function")

  if(!missing(breaks) && any(duplicated(breaks)) )
      stop("breaks may not contain duplicate values")

  if(!missing(breaks) && (scale!="none"))
    warning("Using scale=\"row\" or scale=\"column\" when breaks are",
            "specified can produce unpredictable results.",
            "Please consider using only one or the other.")

  if ( is.null(Rowv) || any(is.na(Rowv)) )
    Rowv <- FALSE
  if ( is.null(Colv) || any(is.na(Colv)) )
    Colv <- FALSE
  else if( all(Colv=="Rowv") )
    Colv <- Rowv


  if(length(di <- dim(x)) != 2 || !is.numeric(x))
    stop("`x' must be a numeric matrix")

  nr <- di[1]
  nc <- di[2]

  if(nr <= 1 || nc <= 1)
    stop("`x' must have at least 2 rows and 2 columns")

  if(!is.numeric(margins) || length(margins) != 2)
    stop("`margins' must be a numeric vector of length 2")

  if(missing(cellnote))
    cellnote <- matrix("", ncol=ncol(x), nrow=nrow(x))

  if(!inherits(Rowv, "dendrogram")) {
    ## Check if Rowv and dendrogram arguments are consistent
    if (
          (
             ( is.logical(Rowv) && !isTRUE(Rowv) )
              ||
             ( is.null(Rowv) )
          )
          &&
          ( dendrogram %in% c("both","row") )
       )
      {
        warning("Discrepancy: Rowv is FALSE, while dendrogram is `",
                dendrogram, "'. Omitting row dendogram.")

        if (dendrogram=="both")
          dendrogram <- "column"
        else
          dendrogram <- "none"

      }
  }

  if(!inherits(Colv, "dendrogram")) {
    ## Check if Colv and dendrogram arguments are consistent
    if (
         (
           (is.logical(Colv) && !isTRUE(Colv) )
             ||
            (is.null(Colv))
         )
         &&
         ( dendrogram %in% c("both","column")) )
      {
        warning("Discrepancy: Colv is FALSE, while dendrogram is `",
                dendrogram, "'. Omitting column dendogram.")

        if (dendrogram=="both")
          dendrogram <- "row"
        else
          dendrogram <- "none"

      }
  }


  ## by default order by row/col mean
  ## if(is.null(Rowv)) Rowv <- rowMeans(x, na.rm = na.rm)
  ## if(is.null(Colv)) Colv <- colMeans(x, na.rm = na.rm)

  ## get the dendrograms and reordering indices

  ## if( dendrogram %in% c("both","row") )
  ##  { ## dendrogram option is used *only* for display purposes
  if(inherits(Rowv, "dendrogram"))
    {
      ddr <- Rowv ## use Rowv 'as-is', when it is dendrogram
      rowInd <- order.dendrogram(ddr)
      if(length(rowInd)>nr || any(rowInd<1 | rowInd > nr ))
         stop("Rowv dendrogram doesn't match size of x")
      if (length(rowInd) < nr)
          nr <- length(rowInd)
    }
  else if (is.integer(Rowv))
    {
      ## Compute dendrogram and do reordering based on given vector
      distr <- distfun(x)
      hcr <- hclustfun(distr)
      ddr <- as.dendrogram(hcr)
      ddr <- reorderfun(ddr, Rowv)

      rowInd <- order.dendrogram(ddr)
      if(nr != length(rowInd))
        stop("row dendrogram ordering gave index of wrong length")
    }
  else if (isTRUE(Rowv))
    { ## If TRUE, compute dendrogram and do reordering based on rowMeans
      Rowv <- rowMeans(x, na.rm = na.rm)
      distr <- distfun(x)
      hcr <- hclustfun(distr)
      ddr <- as.dendrogram(hcr)
      ddr <- reorderfun(ddr, Rowv)

      rowInd <- order.dendrogram(ddr)
      if(nr != length(rowInd))
        stop("row dendrogram ordering gave index of wrong length")
    }
  else if(!isTRUE(Rowv))
    {
      rowInd <- nr:1
      ddr <- as.dendrogram(hclust(dist(diag(nr))))
    }
  else
    {
      rowInd <- nr:1
      ddr <- as.dendrogram(Rowv)
    }

  if(inherits(Colv, "dendrogram"))
    {
      ddc <- Colv ## use Colv 'as-is', when it is dendrogram
      colInd <- order.dendrogram(ddc)
      if(length(colInd)>nc || any(colInd<1 | colInd > nc ))
         stop("Colv dendrogram doesn't match size of x")
      if (length(colInd) < nc)
          nc <- length(colInd)
    }
  else if(identical(Colv, "Rowv")) {
    if(nr != nc)
      stop('Colv = "Rowv" but nrow(x) != ncol(x)')
    if(exists("ddr"))
      {
        ddc <- ddr
        colInd <- order.dendrogram(ddc)
      } else
    colInd <- rowInd
  } else if(is.integer(Colv))
    {## Compute dendrogram and do reordering based on given vector
      distc <- distfun(if(symm)x else t(x))
      hcc <- hclustfun(distc)
      ddc <- as.dendrogram(hcc)
      ddc <- reorderfun(ddc, Colv)

      colInd <- order.dendrogram(ddc)
      if(nc != length(colInd))
        stop("column dendrogram ordering gave index of wrong length")
    }
  else if (isTRUE(Colv))
    {## If TRUE, compute dendrogram and do reordering based on rowMeans
      Colv <- colMeans(x, na.rm = na.rm)
      distc <- distfun(if(symm)x else t(x))
      hcc <- hclustfun(distc)
      ddc <- as.dendrogram(hcc)
      ddc <- reorderfun(ddc, Colv)

      colInd <- order.dendrogram(ddc)
      if(nc != length(colInd))
        stop("column dendrogram ordering gave index of wrong length")
    }
  else if(!isTRUE(Colv))
    {
      colInd <- 1:nc
      ddc <- as.dendrogram(hclust(dist(diag(nc))))
    }
  else
    {
      colInd <- 1:nc
      ddc <- as.dendrogram(Colv)
    }

  retval$rowInd <- rowInd
  retval$colInd <- colInd
  retval$call <- match.call()


  ## reorder x & cellnote
  x <- x[rowInd, colInd]
  x.unscaled <- x
  cellnote <- cellnote[rowInd, colInd]

  if(is.null(labRow))
    labRow <- if(is.null(rownames(x))) (1:nr)[rowInd] else rownames(x)
  else
    labRow <- labRow[rowInd]

  if(is.null(labCol))
    labCol <- if(is.null(colnames(x))) (1:nc)[colInd] else colnames(x)
  else
    labCol <- labCol[colInd]

  if(!is.null(colRow))
    colRow <- colRow[rowInd]

  if(!is.null(colCol))
    colCol <- colCol[colInd]

  if(scale == "row") {
    retval$rowMeans <- rm <- rowMeans(x, na.rm = na.rm)
    x <- sweep(x, 1, rm)
    retval$rowSDs <-  sx <- apply(x, 1, sd, na.rm = na.rm)
    x <- sweep(x, 1, sx, "/")
  }
  else if(scale == "column") {
    retval$colMeans <- rm <- colMeans(x, na.rm = na.rm)
    x <- sweep(x, 2, rm)
    retval$colSDs <-  sx <- apply(x, 2, sd, na.rm = na.rm)
    x <- sweep(x, 2, sx, "/")
  }

  ## Set up breaks and force values outside the range into the endmost bins
  if(missing(breaks) || is.null(breaks) || length(breaks)<1 )
    {
      if( missing(col) ||  is.function(col) )
        breaks <- 16
      else
        breaks <- length(col)+1
    }

  if(length(breaks)==1)
    {
      if(!symbreaks)
        breaks <- seq( min(x, na.rm=na.rm), max(x,na.rm=na.rm), length=breaks)
      else
        {
          extreme <- max(abs(x), na.rm=TRUE)
          breaks <- seq( -extreme, extreme, length=breaks )
        }
    }

  nbr <- length(breaks)
  ncol <- length(breaks)-1

  if(is(col, "function"))
    col <- col(ncol)

  min.breaks <- min(breaks)
  max.breaks <- max(breaks)

  x[x<min.breaks] <- min.breaks
  x[x>max.breaks] <- max.breaks


  ## Calculate the plot layout
  if( missing(lhei) || is.null(lhei) )
    lhei <- c(keysize, 4)

  if( missing(lwid) || is.null(lwid) )
    lwid <- c(keysize, 4)

  if( missing(lmat) || is.null(lmat) )
    {
      lmat <- rbind(4:3, 2:1)

      if(!missing(ColSideColors)) { ## add middle row to layout
        if(!is.character(ColSideColors) || length(ColSideColors) != nc)
          stop("'ColSideColors' must be a character vector of length ncol(x)")
        lmat <- rbind(lmat[1,]+1, c(NA,1), lmat[2,]+1)
        lhei <- c(lhei[1], 0.2, lhei[2])
      }

      if(!missing(RowSideColors)) { ## add middle column to layout
        if(!is.character(RowSideColors) || length(RowSideColors) != nr)
          stop("'RowSideColors' must be a character vector of length nrow(x)")
        lmat <- cbind(lmat[,1]+1, c(rep(NA, nrow(lmat)-1), 1), lmat[,2]+1)
        lwid <- c(lwid[1], 0.2, lwid[2])
      }

      lmat[is.na(lmat)] <- 0
    }

  if(length(lhei) != nrow(lmat))
    stop("lhei must have length = nrow(lmat) = ", nrow(lmat))

  if(length(lwid) != ncol(lmat))
    stop("lwid must have length = ncol(lmat) =", ncol(lmat))

  ## Graphics `output' -----------------------

  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  layout(lmat, widths = lwid, heights = lhei, respect = FALSE)

  plot.index <- 1

  ## draw the side bars
  if(!missing(RowSideColors)) {
    par(mar = c(margins[1],0, 0,0.5))
    iy <- if (revC) nr:1 else 1:nr
    image(rbind(iy), col = RowSideColors[rowInd], axes = FALSE)
    plot.index <- plot.index + 1
  }
  if(!missing(ColSideColors)) {
    par(mar = c(0.5,0, 0,margins[2]))
    image(cbind(1:nc), col = ColSideColors[colInd], axes = FALSE)
    plot.index <- plot.index + 1
  }
  ## draw the main carpet
  par(mar = c(margins[1], 0, 0, margins[2]))
  #if(scale != "none" || !symm)
  #  {
      x <- t(x)
      cellnote <- t(cellnote)
  #  }
  if(revC)
    { ## x columns reversed
      iy <- nr:1
      if(exists("ddr"))
        ddr <- rev(ddr)
      x <- x[,iy]
      cellnote <- cellnote[,iy]
    }
  else iy <- 1:nr

  ## display the main carpet
  image(1:nc, 1:nr, x, xlim = 0.5+ c(0, nc), ylim = 0.5+ c(0, nr),
        axes = FALSE, xlab = "", ylab = "", col=col, breaks=breaks,
        ...)
  retval$carpet <- x
  if(exists("ddr"))
    retval$rowDendrogram <- ddr
  if(exists("ddc"))
    retval$colDendrogram <- ddc
  retval$breaks <- breaks
  retval$col <- col

  ## fill 'na' positions with na.color
  if(!invalid(na.color) & any(is.na(x)))
    {
      mmat <- ifelse(is.na(x), 1, NA)
      image(1:nc, 1:nr, mmat, axes = FALSE, xlab = "", ylab = "",
            col=na.color, add=TRUE)
    }

  ## add column labels
  if(is.null(srtCol) && is.null(colCol))
    axis(1,
         1:nc,
         labels= labCol,
         las= 2,
         line= -0.5 + offsetCol,
         tick= 0,
         cex.axis= cexCol,
         hadj=adjCol[1],
         padj=adjCol[2]
         )
  else
    {
      if(is.null(srtCol) || is.numeric(srtCol))
        {
          if(missing(adjCol) || is.null(adjCol))
            adjCol=c(1,NA)

          if(is.null(srtCol))
            srtCol <- 90

          xpd.orig <- par("xpd")
          par(xpd=NA)
          xpos <- axis(1, 1:nc, labels=rep("", nc), las=2, tick=0)
          text(x=xpos,
               y=par("usr")[3] - (1.0 + offsetCol) * strheight("M"),
               labels=labCol,
               ##pos=1,
               adj=adjCol,
               cex=cexCol,
               srt=srtCol,
               col=colCol
               )
          par(xpd=xpd.orig)
        }
      else
        warning("Invalid value for srtCol ignored.")
    }

  ## add row labels
  if(is.null(srtRow) && is.null(colRow))
    {
      axis(4,
           iy,
           labels=labRow,
           las=2,
           line=-0.5+offsetRow,
           tick=0,
           cex.axis=cexRow,
           hadj=adjRow[1],
           padj=adjRow[2]
           )
    }
  else
    {
      if(is.null(srtRow) || is.numeric(srtRow))
        {
          xpd.orig <- par("xpd")
          par(xpd=NA)
          ypos <- axis(4, iy, labels=rep("", nr), las=2, line= -0.5, tick=0)
          if (revC) ypos <- rev(ypos)
          text(x=par("usr")[2] + (1.0 + offsetRow) * strwidth("M"),
               y=ypos,
               labels=labRow,
               adj=adjRow,
               cex=cexRow,
               srt=srtRow,
               col=colRow
               )
          par(xpd=xpd.orig)
        }
      else
        warning("Invalid value for srtRow ignored.")
    }



  ## add row and column headings (xlab, ylab)
  if(!is.null(xlab)) mtext(xlab, side = 1, line = margins[1] - 1.25)
  if(!is.null(ylab)) mtext(ylab, side = 4, line = margins[2] - 1.25)

  ## perform user-specified function
  if (!missing(add.expr))
    eval(substitute(add.expr))

  ## add 'background' colored spaces to visually separate sections
  if(!missing(colsep))
      for(csep in colsep)
        rect(xleft =csep+0.5,               ybottom=0,
             xright=csep+0.5+sepwidth[1],   ytop=ncol(x)+1,
             lty=1, lwd=1, col=sepcolor, border=sepcolor)

  if(!missing(rowsep))
    for(rsep in rowsep)
      rect(xleft =0,          ybottom= (ncol(x)+1-rsep)-0.5,
           xright=nrow(x)+1,  ytop   = (ncol(x)+1-rsep)-0.5 - sepwidth[2],
           lty=1, lwd=1, col=sepcolor, border=sepcolor)


  ## show traces
  min.scale <- min(breaks)
  max.scale <- max(breaks)
  x.scaled  <- scale01(t(x), min.scale, max.scale)

  if(trace %in% c("both","column") )
    {
      retval$vline <- vline
      vline.vals <- scale01(vline, min.scale, max.scale)
      for( i in 1:length(colInd) )
        {
          if(!is.null(vline))
            {
              abline(v=i-0.5 + vline.vals, col=linecol, lty=2)
            }
          xv <- rep(i, nrow(x.scaled)) + x.scaled[,i] - 0.5
          xv <- c(xv[1], xv)
          yv <- 1:length(xv)-0.5
          lines(x=xv, y=yv, lwd=1, col=tracecol, type="s")
        }
    }


  if(trace %in% c("both","row") )
    {
      retval$hline <- hline
      hline.vals <- scale01(hline, min.scale, max.scale)
      for( i in 1:length(rowInd) )
        {
          if(!is.null(hline))
            {
              abline(h=i - 0.5 + hline.vals, col=linecol, lty=2)
            }
          yv <- rep(i, ncol(x.scaled)) + x.scaled[i,] - 0.5
          yv <- rev(c(yv[1], yv))
          xv <- length(yv):1-0.5
          lines(x=xv, y=yv, lwd=1, col=tracecol, type="s")
        }
    }



  if(!missing(cellnote))
    text(x=c(row(cellnote)),
         y=c(col(cellnote)),
         labels=c(cellnote),
         col=notecol,
         cex=notecex)

  plot.index <- plot.index + 1

  ## increment plot.index and then do
  ##   latout_set( lmat, plot.index )
  ## to set to the correct plot region, instead of
  ## relying on plot.new().

  ## the two dendrograms :
  par(mar = c(margins[1], 0, 0, 0))
  if( dendrogram %in% c("both","row") )
    {
        flag <- try(
            plot.dendrogram(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
            )
        if("try-error" %in% class(flag))
            {
                cond <- attr(flag, "condition")
                if(!is.null(cond) && conditionMessage(cond)=="evaluation nested too deeply: infinite recursion / options(expressions=)?")
                    stop('Row dendrogram too deeply nested, recursion limit exceeded.  Try increasing option("expressions"=...).')
            }
    }
  else
    plot.new()

  par(mar = c(0, 0, if(!is.null(main)) 5 else 0, margins[2]))

  if( dendrogram %in% c("both","column") )
    {
        flag <- try(
            plot.dendrogram(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
            )
        if("try-error" %in% class(flag))
            {
                cond <- attr(flag, "condition")
                if(!is.null(cond) && conditionMessage(cond)=="evaluation nested too deeply: infinite recursion / options(expressions=)?")
                    stop('Column dendrogram too deeply nested, recursion limit exceeded.  Try increasing option("expressions"=...).')
            }
    }
  else
    plot.new()

  ## title
  if(!is.null(main)) title(main, cex.main = 1.5*op[["cex.main"]])

  ## Add the color-key
  if(key)
    {
      mar <- c(5, 4, 2, 1)
      if (!is.null(key.xlab) && is.na(key.xlab))
          mar[1] <- 2
      if (!is.null(key.ylab) && is.na(key.ylab))
          mar[2] <- 2
      if (!is.null(key.title) && is.na(key.title))
          mar[3] <- 1
      par(mar = mar, cex=0.75, mgp=c(2, 1, 0))
      if (length(key.par) > 0)
          do.call(par, key.par)
      tmpbreaks <- breaks

      if(symkey)
        {
          max.raw <- max(abs(c(x,breaks)),na.rm=TRUE)
          min.raw <- -max.raw
          tmpbreaks[1] <- -max(abs(x), na.rm=TRUE)
          tmpbreaks[length(tmpbreaks)] <- max(abs(x), na.rm=TRUE)
        }
      else
        {
          min.raw <- min.breaks
          max.raw <- max.breaks
        }

      z <- seq(min.raw, max.raw, by=min(diff(breaks)/100))
      image(z=matrix(z, ncol=1),
            col=col, breaks=tmpbreaks,
            xaxt="n", yaxt="n")

      par(usr=c(0,1,0,1))
      if (is.null(key.xtickfun)) {
          lv <- pretty(breaks)
          xv <- scale01(as.numeric(lv), min.raw, max.raw)
          xargs <- list(at=xv, labels=lv)
      } else {
          xargs <- key.xtickfun()
      }
      xargs$side <- 1
      do.call(axis, xargs)
      if (is.null(key.xlab)) {
        if(scale=="row")
            key.xlab <- "Row Z-Score"
        else if(scale=="column")
           key.xlab <- "Column Z-Score"
        else
            key.xlab <- "Value"
      }
      if (!is.na(key.xlab)) {
          mtext(side=1, key.xlab, line=par("mgp")[1], padj=0.5, cex=par("cex") * par("cex.lab"))
      }

      if(density.info=="density")
        {
          dens <- density(x, adjust=densadj, na.rm=TRUE,
                          from=min.scale, to=max.scale)
          omit <- dens$x < min(breaks) | dens$x > max(breaks)
          dens$x <- dens$x[!omit]
          dens$y <- dens$y[!omit]
          dens$x <- scale01(dens$x, min.raw, max.raw)
          lines(dens$x, dens$y / max(dens$y) * 0.95, col=denscol, lwd=1)
          if (is.null(key.ytickfun)) {
              yargs <- list(at=pretty(dens$y)/max(dens$y) * 0.95, labels=pretty(dens$y))
          } else {
              yargs <- key.ytickfun()
          }
          yargs$side <- 2
          do.call(axis, yargs)
          if (is.null(key.title))
              key.title <- "Color Key\nand Density Plot"
          if (!is.na(key.title))
            title(key.title)
          par(cex=0.5)
          if (is.null(key.ylab))
              key.ylab <- "Density"
          if (!is.na(key.ylab))
            mtext(side=2,key.ylab, line=par("mgp")[1], padj=0.5, cex=par("cex") * par("cex.lab"))
        }
      else if(density.info=="histogram")
        {
          h <- hist(x, plot=FALSE, breaks=breaks)
          hx <- scale01(breaks, min.raw, max.raw)
          hy <- c(h$counts, h$counts[length(h$counts)])
          lines(hx, hy/max(hy)*0.95, lwd=1, type="s", col=denscol)
          if (is.null(key.ytickfun)) {
              yargs <- list(at=pretty(hy)/max(hy) * 0.95, labels=pretty(hy))
          } else {
              yargs <- key.ytickfun()
          }
          yargs$side <- 2
          do.call(axis, yargs)
          if (is.null(key.title))
              key.title <- "Color Key\nand Histogram"
          if (!is.na(key.title))
            title(key.title)
          par(cex=0.5)
          if (is.null(key.ylab))
              key.ylab <- "Count"
          if (!is.na(key.ylab))
            mtext(side=2,key.ylab, line=par("mgp")[1], padj=0.5, cex=par("cex") * par("cex.lab"))
        }
      else
        {
          if (is.null(key.title))
              key.title <- "Color Key"
          if (!is.na(key.title))
              title(key.title)
        }
      if(trace %in% c("both","column") )
          {
              vline.vals <- scale01(vline, min.raw, max.raw)
              if(!is.null(vline))
                  {
                      abline(v=vline.vals, col=linecol, lty=2)
                  }
          }


      if(trace %in% c("both","row") )
          {
              hline.vals <- scale01(hline, min.raw, max.raw)
              if(!is.null(hline))
                  {
                      abline(v=hline.vals, col=linecol, lty=2)

                  }
          }

    }
  else
    {
      par(mar=c(0, 0, 0, 0))
      plot.new()
    }
  ## Create a table showing how colors match to (transformed) data ranges
  retval$colorTable <- data.frame(
                             low=retval$breaks[-length(retval$breaks)],
                             high=retval$breaks[-1],
                             color=retval$col
                             )

  # Store layout information, suggested by Jenny Drnevich
  retval$layout <- list(lmat = lmat,
                        lhei = lhei,
                        lwid = lwid
                        )


  ## If user has provided an extra function, call it.
  if(!is.null(extrafun))
      extrafun()

  invisible( retval )
}

suppressMessages(library(gplots))
data(mtcars)
x  <- as.matrix(mtcars)
rc <- rainbow(nrow(x), start=0, end=.3)
cc <- rainbow(ncol(x), start=0, end=.3)

heatmap.2(x)  ## default - dendrogram plotted and reordering done.
heatmap.2(x, dendrogram="none") ##  no dendrogram plotted, but reordering done.
heatmap.2(x, dendrogram="row") ## row dendrogram plotted and row reordering done.
heatmap.2(x, dendrogram="col") ## col dendrogram plotted and col reordering done.

heatmap.2(x, keysize=2)  ## default - dendrogram plotted and reordering done.


heatmap.2(x, Colv=FALSE)
heatmap.2(x, Colv=NULL)

heatmap.2(x, Rowv=FALSE)
heatmap.2(x, Rowv=NULL)

heatmap.2(x, Rowv=FALSE, dendrogram="both") ## generate warning!
heatmap.2(x, Rowv=NULL, dendrogram="both")  ## generate warning!
heatmap.2(x, Colv=FALSE, dendrogram="both") ## generate warning!



heatmap.2(x,dendrogram="both", Rowv=FALSE) ## row and col reordering done, but only col dendrogram plotted 
heatmap.2(x,dendrogram="row",  Rowv=FALSE) 


heatmap.2(x,dendrogram="col",  Rowv=FALSE)
heatmap.2(x,dendrogram="none", Rowv=FALSE) 


heatmap.2(x,dendrogram="both", Colv=FALSE) ## row and col reordering done, but only row dendrogram plotted
heatmap.2(x,dendrogram="row",  Colv=FALSE)
heatmap.2(x,dendrogram="col",  Colv=FALSE)
heatmap.2(x,dendrogram="none", Colv=FALSE) 

## above statement is same as heatmap.2(x,Colv=FALSE)
heatmap.2(x,dendrogram="col", Colv=FALSE) ## col reordering done, but no dendrogram plotted 


hv <- heatmap.2(x, col=cm.colors(256), scale="column", 
	       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10), 
	       xlab="specification variables", ylab= "Car Models", 
	       main="heatmap(<Mtcars data>, \n        ..., scale=\"column\")", 
               tracecol="green", density="density")

## IGNORE_RDIFF_BEGIN
str(hv) # the two re-ordering index vectors
## IGNORE_RDIFF_END

## Test row and column separators 
heatmap.2(x,dendrogram="col", colsep=1) ## one colsep
heatmap.2(x,dendrogram="col", rowsep=1) ## one rowsep

heatmap.2(x,dendrogram="col", colsep=1:2) ## two colseps
heatmap.2(x,dendrogram="col", rowsep=1:2) ## two rowseps

suppressMessages(library(gplots))

dat <- as.matrix(read.csv(file="dat.csv", row.names=1))
dist2 <- function(x) as.dist(1-cor(t(x), method="pearson"))
hclust1 <- function(x) hclust(x, method = "single")

distance <- dist2(dat)
cluster  <- hclust1(distance)

dend <- as.dendrogram(cluster)

## R's default recursion limits will be exceeded when plotting this dendrogram
try( gplots:::plot.dendrogram(dend) )
try( heatmap.2(dat, Rowv=dend) )

## Increase them and try again
options("expressions"=20000)
gplots:::plot.dendrogram(dend)
heatmap.2(dat, Rowv=dend)

angleAxis <- function(side, labels, at=1:length(labels), srt=45, adj, xpd=TRUE, ...)
{
  usr <- par("usr")
  emH <- strheight("M")
  emW <- strwidth("M")
  
  if(missing(adj))
    adj <- switch(side, 1, 1, 0, 0)
  
  switch(side,
         #1 - below
         text(x=at,           y=usr[3]-emH/2, labels=labels, srt=srt, adj=adj, xpd=xpd, ...), 
         #2 - left
         text(x=usr[1]-emW/2, y=at,           labels=labels, srt=srt, adj=adj, xpd=xpd, ...), 
         #3 - above
         text(x=at,           y=usr[4]+emH/2, labels=labels, srt=srt, adj=adj, xpd=xpd, ...), 
         #4 - right
         text(x=usr[2]+emW/2, y=at,           labels=labels, srt=srt, adj=adj, xpd=xpd, ...)
  )       
  
  invisible(NULL)
}

# $Id: sinkplot.R 1673 2013-06-27 20:20:46Z warnes $
sinkplotEnv <-new.env()
  
sinkplot <- function(operation=c("start","plot","cancel"),...)
  {
    operation <- match.arg(operation)

    if( operation=="start" )
      {
        if (exists(".sinkplot.conn", envir=sinkplotEnv) &&
            get(".sinkplot.conn", envir=sinkplotEnv) )
          stop("sinkplot already in force")


        .sinkplot.conn <- textConnection(".sinkplot.data", "w", local=FALSE)
        assign(x=".sinkplot.conn", value=.sinkplot.conn, envir=sinkplotEnv)

        on.exit(sink())
        sink(.sinkplot.conn)
        on.exit()
      }
    else
      {
        if (!exists(".sinkplot.conn", envir=sinkplotEnv) ||
            !get(".sinkplot.conn", envir=sinkplotEnv) )
          stop("No sinkplot currently in force")

        sink()

        data <- get(".sinkplot.data", envir=sinkplotEnv)

        if( operation=="plot" )
            textplot( paste( data, collapse="\n"), ... )

        close(get(".sinkplot.conn", envir=sinkplotEnv))

        if(exists(".sinkplot.data", envir=sinkplotEnv, inherits=FALSE))
          rm(list=".sinkplot.data", pos=sinkplotEnv, inherits=FALSE)

        if(exists(".sinkplot.conn", envir=sinkplotEnv, inherits=FALSE))
          rm(list=".sinkplot.conn", pos=sinkplotEnv, inherits=FALSE)

        invisible(data)
      }
  }

# $Id: qqnorm.aov.R 625 2005-06-09 14:20:30Z nj7w $

qqnorm.aov <- function (y, full = FALSE, label = FALSE, omit = NULL,
                        xlab = paste(if(full) "" else "Half", " Normal plot"),
                        ylab = "Effects", ...)
{
    r <- y$rank
    eff <- if (full)
        effects(y, set.sign = TRUE)[1:r]
    else abs(effects(y))[1:r]
    na <- names(eff)
    int <- match("(Intercept)", na)
    if (!is.null(omit)) {
        if (is.character(omit)) {
            int <- c(int, match(omit, na))
        }
        else int <- c(int, omit)
    }
    int <- int[!is.na(int)]
    if (length(int))
        eff <- eff[-int]
    n <- length(eff)
    if (n <= 0)
        stop("Not enough effects")
    ord <- order(eff)
    na <- names(eff)
    P <- if (full)
        ppoints(n)
    else ((1:n) + n)/(2 * n + 1)
    Q <- qnorm(P)
    plot(x = Q, y = eff[ord], xlab = xlab, ylab = ylab, ...)
    if (label && dev.interactive())
        identify(Q, eff[ord], names(eff)[ord])
}

# This code plots Venn Diagrams for up to 5 sets. The
# function getVennCounts is passed a list of vectors.
# This is transformed into a table indicating the
# number of intersections for each intersection. This table
# is generated for any number of sets.

# The function drawVennDiagram plots circles (up to three
# sets) or ellipses (4 and 5 sets) to depict the sets.
# The sum of values placed is the number of entries of
# each set.

# Function to determine values of a venn diagram
# It works for an arbitrary large set of input sets.
#

getVennCounts <- function(l, universe, verbose=F, ...)
  UseMethod("getVennCounts")

getVennCounts.data.frame <- function(l, universe=NA, verbose=F, ...)
  {
    if (verbose) cat("Interpreting data as data.frame.\n")
    if( !all(unique(unlist(l)) %in% c(0,1))  )
      stop("Only indicator columns permitted")

    l <- lapply( l, function(x) which(as.logical(x)))
    getVennCounts.list(l, universe=universe, verbose=verbose)
  }

getVennCounts.matrix <- function(l, universe=NA, verbose=F, ...)
{
  getVennCounts.data.frame(as.data.frame(l), universe=NA, verbose=F, ...)
}

# l offers a list of arrays, their values are to
# be tested for the size of their intersects.
getVennCounts.list<-function(l, universe=NA, verbose=F, intersections=TRUE, ...) {
  if (verbose) cat("Interpreting data as list.\n")
  numSets<-length(l)
  result.table<-NULL
  result.table.names<-NULL

  memberList <- list()

  # Iteration over all possible intersections involving all sets
  # or the complement (negation) of those sets.
  for (i in 0:(-1 + 2^numSets)) {
    # i2 is a binary representation of that number
    i2<-baseOf(i,2,numSets)

    # some debug output
    #print(paste(i,":",paste(i2,collapse="",sep="")))

    # p.pos determines the position in number
    #       which is also the set that is inspected

    sel<-universe

    # positive selection first
    for (p.pos in which(1 == i2) ) {
      current.set<-l[[p.pos]]
      if (!is.null(dim(current.set))) {
        # circumventing strange experiences with data.frames
        warning(paste("List element [[",p.pos,"]] has dimensions, but all elements are considered.\n",sep=""))
        current.set<-as.character(as.matrix(current.set))
        dim(current.set)<-NULL
      }
      #print(paste("set ",p.pos,", val=1: ",paste(current.set,collapse=",")))
      if (is.null(sel)) {
        #print("Sel is null")
      } else if (1 == length(sel) && is.na(sel)) {
        sel<-current.set
      }
      else {
        w<-which(sel %in% current.set)
        if (length(w)>0) {
          sel<-sel[w]
        }
        else {
          sel<-NULL
        }
      }
    }

    # something should be in sel now, otherwise
    # the number will be 0

    # negative selection
    for (p.pos in which(0 == i2) ) {
      if (is.null(sel) || ( 1 == length(sel) && is.na(sel))) {
        # The complement is not known, hence no checks done
      }
      else {
        current.set<-l[[p.pos]]
        if (!is.null(dim(current.set))) {
          warning(paste("List element [[",p.pos,"]] has dimensions, but all elements are considered.\n",sep=""))
          current.set<-as.character(as.matrix(current.set))
          dim(current.set)<-NULL
        }
        w<-which( ! sel %in% current.set)
        #print(paste("set ",p.pos,", val=1: ",paste(current.set,collapse=",")))
        if (length(w)>0) {
          sel<-sel[w]
        }
        else {
          sel<-NULL
        }
      }
    }
    #print(paste("sel:",paste(sel,collapse=",")))

    if(is.null(sel) || (1 == length(sel) && is.na(sel))) {
      sel<-NULL
    }

    r.name<-paste(i2,collapse="")
    if (intersections) {
      memberList[[r.name]] <- sel
    }

    r<-length(sel)
    result.row<-c(r,i2)
    dim(result.row)<-c(1,length(result.row))
    rownames(result.row)<-c(r.name)
    #print(paste("Adding ",r.name))
    if (is.null(result.table)) {
      result.table<-result.row
    }
    else {
      result.table<-rbind(result.table,result.row)
    }
    #if (is.null(result.table)) {
    #	result.table<-r
    #	result.table.names<-r.name
    #}
    #else {
    #	result.table<-c(result.table,r)
    #	result.table.names<-c(result.table.names,r.name)
    #}
  }
  #names(result.table)<-result.table.names
  if (is.null(names(l))) {
    colnames(result.table)<-c("num",LETTERS[1:numSets])
  }
  else{
    colnames(result.table)<-c("num",names(l))
  }
  if (intersections) {
    attr(result.table,"intersections") <- memberList
  }
  class(result.table) <- "venn"
  return(result.table)
}

#print(getVennCounts(list(A,B,C,D)))
#print(getVennCounts(list(a=A,b=B,c=C,d=D)))

venn <- function(data,
                 universe=NA,
                 small=0.7,
                 showSetLogicLabel=FALSE,
                 simplify=FALSE,
                 show.plot=TRUE,
                 intersections=TRUE,
                 names,
                 ...
                 )
{
  counts <- getVennCounts(data,
                          universe=universe,
                          intersections=intersections
                          )

  if(show.plot)
    drawVennDiagram(data=counts,
                    small=small,
                    showSetLogicLabel=showSetLogicLabel,
                    simplify=simplify,
                    ...
                    )

  # use VennMemberNames to properly label and order the 'intersection' table
  if(intersections)
    attr(counts, "intersections") <- vennMembers(l=data,
                                                 universe=universe,
                                                 names=names
                                                 )

  invisible(counts)
}

# $Id: wapply.R 1012 2006-11-14 22:25:06Z ggorjan $

"wapply" <- function( x, y, fun=mean, method="range",
                    width, n=50, drop.na=TRUE, pts, ...)
{
  method <- match.arg(method, c("width","range","nobs","fraction"))
  if(missing(width))
    if( method=="nobs" ) width <- max(5, length(x)/10 )
  else
    width <- 1/10

  if(method == "width" || method == "range" )
    {
      if(method=="range")
        width <- width * diff(range(x))

      if(missing(pts))
        pts <- seq(min(x),max(x),length.out=n)
      
      result <- sapply( pts, function(pts,y,width,fun,XX,...)
                      {
                        low <- min((pts-width/2),max(XX))
                        high <- max((pts+width/2), min(XX))
                        return (fun(y[(XX>= low) & (XX<=high)],...))
                      },
                      y=y,
                      width=width,
                      fun=fun,
                      XX = x,
                      ...)
      if(drop.na)
        {
          missing <- is.na(pts) & is.na(result)
          pts <- pts[!missing]
          result <- result[!missing]
        }
      
      return(list(x=pts,y=result))
    }
  else # method=="nobs" || method=="fraction"
    {
      if( method=="fraction")
        width <- floor(length(x) * width)

      ord <- order(x)
      x  <- x[ord]
      y  <- y[ord]

      n  <- length(x)
      center  <- 1:n
      below <- sapply(center - width/2, function(XX) max(1,XX) )
      above <- sapply(center + width/2, function(XX) min(n,XX) )

      retval  <- list()
      retval$x  <- x
      retval$y  <- apply(cbind(below,above), 1,
                         function(x) fun(y[x[1]:x[2]],...) )
                         
      if(drop.na)
        {
          missing <- is.na(retval$x) | is.na(retval$y)
          retval$x <- retval$x[!missing]
          retval$y <- retval$y[!missing]
        }


      return(retval)
    }
      
}

plot.venn <- function(x, y, ...,
                      small=0.7,
                      showSetLogicLabel=FALSE,
                      simplify=FALSE
                      )
  {
    drawVennDiagram(
                    data=x,
                    small=small,
                    showSetLogicLabel=showSetLogicLabel,
                    simplify=simplify
                    )
  }

## data should be a matrix.
##   - The first column of the matrix is the
##     count of the number of objects with the specified pattern.
##   - The second and subsequent columns contain 0-1 indicators
##     giving the pattern of group membership


drawVennDiagram <-function(data,
                           small=0.7,
                           showSetLogicLabel=FALSE,
                           simplify=FALSE)
  {
	numCircles<-NA
	data.colnames<-NULL
	data.rownames<-NULL

	if(is.matrix(data)) {
		numCircles<-ncol(data)-1
		data.colnames<-colnames(data)[2:(ncol(data))]
		# Order is reverted since later indexing starts with
		# the "lowest bit" and that is expected at the left
		data.rownames<-rownames(data)
	}
	else {
		if (is.list(data)) {
			stop("gplots.drawVennDiagram: This internal function is used wrongly. ",
                             "Please call the function 'venn' with the same arguments, instead.\n")
		}

		warning("drawVennDiagram: Testing only, presuming first argument to specify",
		     "the number of circles to draw.\n")
		numCircles<-data
	}


	m<-(0:(-1+2^numCircles))

	if (! is.matrix(data)) {
		##cat("prepare randomised data\n")
		data<-t(sapply(X=m,FUN=function(v){
			l<-baseOf(v,2,numCircles)
			#print(l)
			return(l)
		}))

		#print(data)

		#data.names<-apply(data,1,function(X){
		#	return(paste(X),collapse="")
		#})
		for(i in m) {
			n<-paste(data[i+1,],collapse="")
			if (is.null(data.rownames)) {
				data.rownames<-n
			}
			else {
				data.rownames<-c(data.rownames,n)
			}
		}
		#print(data.rownames)
		data<-cbind(sample(1:100,size=2^numCircles,replace=TRUE),data)
		#print(data)
		rownames(data)<-data.rownames
		data.colnames<-LETTERS[1:numCircles]
		colnames(data)<-c("num",data.colnames)
	}


	plot.new()
	h<-400
	plot.window(c(0,h), c(0,h), ylab="", xlab="")

	if ((2 <= numCircles && numCircles <= 3) || (4 == numCircles && simplify)) {


		circle <- function(x,y=NULL,r=1) {
			elps=cbind(r*cos(seq(0,2*pi,len=1000)), r*sin(seq(0,2*pi,len=1000)));
			if (!is.null(y)) {
				if (length(x) != length(y))
				  stop("circle: both x and y need to be of same length")
				if (is.matrix(x) && ncol(x)>1)
				  stop("circle: if y is not NULL, then x must not be a matrix")
				x<-cbind(x,y)
			}
			for(i in 1:nrow(x)) {
				ax<-elps[,1]+rep(x[i,1],1000)
				ay<-elps[,2]+rep(x[i,2],1000)
				polygon(ax,ay)
			}
		}

		#nolongerreguired#require(grid)

		##cat("drawing circles\n")
		# draw circles with radius 1.7 equally distributed
		# with centers on a circle of radius 1

		degrees<-2*pi/numCircles*(1:numCircles)

		# scaling factor
		s<-1/8*h
		# radius for circles
		r<-3/12*h

		x<-sapply(degrees,FUN=sin)*s + 0.5*h
		y<-sapply(degrees,FUN=cos)*s + 0.5*h

		##cat("filling data\n")
		circle(x,y,r)

		distFromZero<-rep(NA,2^numCircles)
		degrees<-rep(NA,2^numCircles)

		degrees[(2^numCircles)]<-0
		distFromZero[(2^numCircles)]<-0

		for (i in 0:(numCircles-1)) {
			distFromZero[2^i+1] <- r
			degrees[2^i+1] <- 2*pi/numCircles*i
			d<-degrees[2^i+1]

			#print(data.colnames)

			text(
				# starting from the lowest bit, hence reading
				# lables from the right
				label=data.colnames[numCircles - i],
				x=sin(d)*5/12*h+0.5*h,
				y=cos(d)*5/12*h+0.5*h
			)

		}

		if (4==numCircles) {
			for (i in 0:(numCircles-1)) {
				# Current set bit plus the bit left of it and the bit right of it
				distFromZero[2^i
						+2^((i+numCircles-1)%%numCircles)
						+2^((i+1)%%numCircles)+1] <- 2/12*h
				distFromZero[2^i+1] <- 3.5/12*h
				degrees[2^i
						+2^((i+numCircles-1)%%numCircles)
						+2^((i+1)%%numCircles)+1] <- degrees[2^i+1]
			}
		}

				#degrees[2^i+1] + degrees[2^((i+1)%%numCircles)+1])/2

		if (3 <=numCircles) {
			for (i in 0:(numCircles-1)) {
				distFromZero[(2^i+2^((i+1)%%numCircles))+1]<- 2.2/12*h
				distFromZero[2^i+1] <- 3/12*h
				if (i == (numCircles-1)) {
					degrees[(2^i+2^((i+1)%%numCircles))+1] <- (
						degrees[2^i+1] + 2*pi+ degrees[1+1])/2
				}
				else {
					degrees[(2^i+2^((i+1)%%numCircles))+1] <- (
						degrees[2^i+1] + degrees[2^((i+1)%%numCircles)+1])/2
				}
			}
		}

		for(i in 1:2^numCircles) {
			n<-paste(baseOf((i-1),2,numCircles),collapse="")
			v<-data[n,1]
			d<-degrees[i]
			if (1 == length(d) && is.na(d)) {
				if (v>0) warning("Not shown: ",n," contains ",v,"\n")
			}
			else {
				l<-distFromZero[i]
				x<-sin(d)*l+0.5*h
				y<-cos(d)*l+0.5*h
				#cat("i=",i," x=",x," y=",y," label=",n,"\n")
				l<-v
				if (showSetLogicLabel) l<-paste(n,"\n",v,sep="")
				text(label=l,x=x,y=y)
			}
		}

	}
	else if ( (4 == numCircles && !simplify) || numCircles <= 5 ) {

		# Function to turn and move ellipses/circles
		relocate_elp <- function(e, alpha, x, y){
			phi=(alpha/180)*pi;
			xr=e[,1]*cos(phi)+e[,2]*sin(phi)
			yr=-e[,1]*sin(phi)+e[,2]*cos(phi)
			xr=x+xr;
			yr=y+yr;
			return(cbind(xr, yr))
		}

		lab<-function (identifier, data, showLabel=showSetLogicLabel) {
			r<-data[identifier,1]
			if (showLabel) {
				return(paste(identifier,r,sep="\n"))
			}
			else {
				return(r)
			}
		}

	    if (4 == numCircles) {
	        elps=cbind(162*cos(seq(0,2*pi,len=1000)), 108*sin(seq(0,2*pi,len=1000)));

		#plot(c(0, 400), c(0, 400), type="n", axes=F, ylab="", xlab="");
		polygon(relocate_elp(elps, 45,130,170));
		polygon(relocate_elp(elps, 45,200,200));
		polygon(relocate_elp(elps,135,200,200));
		polygon(relocate_elp(elps,135,270,170));

		text( 35, 315, data.colnames[1],cex=1.5)
		text(138, 347, data.colnames[2],cex=1.5)
		text(262, 347, data.colnames[3],cex=1.5)
		text(365, 315, data.colnames[4],cex=1.5)

	        elps <- cbind(130*cos(seq(0,2*pi,len=1000)),
			80*sin(seq(0,2*pi,len=1000)))

		text( 35, 250, lab("1000",data));
		text(140, 315, lab("0100",data));
		text(260, 315, lab("0010",data));
		text(365, 250, lab("0001",data));

		text( 90, 280, lab("1100",data), cex=small)
		text( 95, 110, lab("1010",data) )
		text(200,  50, lab("1001",data), cex=small)
		text(200, 290, lab("0110",data))
		text(300, 110, lab("0101",data))
		text(310, 280, lab("0011",data), cex=small)

		text(130, 230, lab("1110",data))
		text(245,  75, lab("1101",data),cex=small)
		text(155,  75, lab("1011",data),cex=small)
		text(270, 230, lab("0111",data))

		text(200,150,lab("1111",data))
	    }
	    else if (5 == numCircles) {

	        elps <- cbind(150*cos(seq(0,2*pi,len=1000)),
			60*sin(seq(0,2*pi,len=1000)))

		polygon(relocate_elp(elps, 90,200, 250))
		polygon(relocate_elp(elps, 162,250, 220))
		polygon(relocate_elp(elps, 234,250, 150))
		polygon(relocate_elp(elps, 306,180, 125))
		polygon(relocate_elp(elps, 378,145, 200))

		text( 20, 295, data.colnames[1],cex=1.5)
		text(140, 380, data.colnames[2],cex=1.5)
		text(350, 318, data.colnames[3],cex=1.5)
		text(350,   2, data.colnames[4],cex=1.5)
		text( 50,  10, data.colnames[5],cex=1.5)

		text( 61, 228, lab("10000",data));
		text(194, 329, lab("01000",data));
		text(321, 245, lab("00100",data));
		text(290,  81, lab("00010",data));
		text(132,  69, lab("00001",data));

		text(146, 250, lab("11000",data), cex=small)
		text(123, 188, lab("10100",data), cex=small)
		text(275, 152, lab("10010",data), cex=small)
		text(137, 146, lab("10001",data), cex=small)
		text(243, 268, lab("01100",data), cex=small)
		text(175, 267, lab("01010",data), cex=small)
		text(187, 117, lab("01001",data), cex=small)
		text(286, 188, lab("00110",data), cex=small)
		text(267, 235, lab("00101",data), cex=small)
		text(228, 105, lab("00011",data), cex=small)

		text(148, 210, lab("11100",data),cex=small)
		text(159, 253, lab("11010",data),cex=small)
		text(171, 141, lab("11001",data),cex=small)
		text(281, 175, lab("10110",data),cex=small)
		text(143, 163, lab("10101",data),cex=small)
		text(252, 145, lab("10011",data),cex=small)
		text(205, 255, lab("01110",data),cex=small)
		text(254, 243, lab("01101",data),cex=small)
		text(211, 118, lab("01011",data),cex=small)
		text(267, 211, lab("00111",data),cex=small)

		text(170, 231,lab("11110",data),cex=small)
		text(158, 169,lab("11101",data),cex=small)
		text(212, 139,lab("11011",data),cex=small)
		text(263, 180,lab("10111",data),cex=small)
		text(239, 232,lab("01111",data),cex=small)

		text(204,190,lab("11111",data))
	    }
	}
	else {
		stop(paste("Venn diagrams for ",numCircles," dimensions are not yet supported.\n"))
	}

}

# $Id: space.R 625 2005-06-09 14:20:30Z nj7w $

# When there are two or more points with the same (x,y) value (or
# within x+-s[1] and x+-s[2]), space these out in the y direction so
# that the points are separated by at least distance s.


space <-  function(x,y,s=1/50, na.rm=TRUE, direction="x")
  {
    if(direction!='x')
      {
        tmp <- y
        y <- x
        x <- tmp
      }

    if(na.rm)
      {
        ind <- is.na(x) | is.na(y)
        x <- x[!ind]
        y <- y[!ind]
      }

    if (length(s)==1) s <- c(s,s)

    spacing.x <- (max(x) - min(x))*s[1]
    spacing.y <- (max(y) - min(y))*s[2]

    within <- function(x,y,delta) { abs(x-y) < delta }

    # sort x,y so we can do the work
    ord <- order(x,y)
    undo <- order(ord)

    x <- x[ord]
    y <- y[ord]

    startsame <- 1
    same.x <- x[1]
    same.y <- y[1]

    for( i in 1:length(x) )
      {
        if(i>1 &&
           within(x[i],same.x,spacing.x) &&
           within(y[i],same.y,spacing.y) )
          {
            if(x[startsame] == same.x )
              x[startsame] <- x[startsame]

            cumrun <- i - startsame

            x[i] <- x[i] + (-1)^(cumrun+1) * floor((cumrun+1) /2) * spacing.x
          } else {
            startsame <- i
            same.x <- x[i]
            same.y <- y[i]
          }
      }

    if(direction!='x')
      {
        tmp <- y
        y <- x
        x <- tmp
      }

    return( list(x=x[undo], y=y[undo]) )
}

# $Id: colorpanel.R 736 2005-11-18 00:16:28Z warnes $

colorpanel <- function(n,low,mid,high)
  {
    if(missing(mid) || missing(high) )
      {
        ## convert to rgb
        low <- col2rgb(low)
        if(missing(high))
          high <- col2rgb(mid)
        else
          high <- col2rgb(high)
        
        red    <- seq(low[1,1], high[1,1], length=n)/255
        green  <- seq(low[3,1], high[3,1], length=n)/255
        blue   <- seq(low[2,1], high[2,1], length=n)/255
      }
    else # use a center color
      {
        isodd <- odd(n)
        if(isodd)
          {
            n <- n+1
          }

        ## convert to rgb
        low <- col2rgb(low)
        mid <- col2rgb(mid)
        high <- col2rgb(high)

        ## determine length of each component
        lower <- floor(n/2)
        upper <- n - lower
        
        red  <- c(
                  seq(low[1,1], mid [1,1], length=lower),
                  seq(mid[1,1], high[1,1], length=upper)
                  )/255
        
        green <- c(
                   seq(low[3,1], mid [3,1], length=lower),
                   seq(mid[3,1], high[3,1], length=upper)
                   )/255
        
        blue <- c(
                  seq(low[2,1], mid [2,1], length=lower),
                  seq(mid[2,1], high[2,1], length=upper)
                  )/255
        
        if(isodd)
          {
            red   <- red  [-(lower+1)]
            green <- green[-(lower+1)]
            blue  <- blue [-(lower+1)]
          }
      }
      
    rgb(red,blue,green)
  }



# Generate red-to-green colorscale
redgreen <- function(n) colorpanel(n, 'red', 'black', 'green')
greenred <- function(n) colorpanel(n, 'green', 'black', 'red' )
bluered  <- function(n) colorpanel(n, 'blue','white','red')
redblue  <- function(n) colorpanel(n, 'red','white','blue')

# Extract intersections
vennMembers <- function(l, universe=NA, names, ...)
{
  venn_object <- getVennCounts(l, universe, intersections=TRUE, ...)
  map <- attr(venn_object, "intersections")
  if(missing(names))
    names <- colnames(venn_object)[-1]

  if(is.matrix(l) || is.data.frame(l))
  {
    ids <- rownames(l)
    retval <- list()
    for(i in names(map))
      retval[[i]] <- ids[map[[i]]]
  }
  else if(is.list(l))
    retval <- map


  flags <- do.call(rbind, strsplit(names(map), character(0), fixed=TRUE))
  rownames(flags) <- names(map)
  colnames(flags) <- names
  nameList <- list()
  for(i in 1:nrow(flags))     nameList[[i]] <- ifelse(flags[i,]=="1", colnames(flags), "")
  nameList <- do.call(data.frame,nameList)
  nameList <- apply(nameList, 2, paste, collapse=":")
  nameList <- gsub('::+', ':', nameList)
  nameList <- gsub('^:+', '',  nameList)
  nameList <- gsub(':+$', '',  nameList)

  names(retval) <- nameList

  sortTab <- cbind(sapply(nameList, nchar), nameList)
  ord     <- order(sortTab[,1], sortTab[,2])

  retval <- retval[ord]

  retval <- lapply(retval, as.character)

  retval
}

# $Id: overplot.R 1947 2015-04-23 21:18:42Z warnes $

panel.overplot <- function(formula, data, subset, col, lty, ...)
  {
    m <- match.call()

    m[[1]] <- as.name("plot")
    eval(m, parent.frame() )

    m[[1]] <- as.name("lowess")
    tmp <- eval(m, parent.frame() )

    lines( tmp, col=col, lwd=2, lty=lty )
  }

overplot <- function (formula, data = parent.frame(),
                      same.scale=FALSE,
                      xlab, ylab,
                      xlim, ylim,
                      min.y, max.y,
                      log='',
                      panel='panel.overplot',
                      subset,
                      plot=TRUE,
                      groups,
                      main,
                      f=2/3,
                      ... )
{
  ###
  # check that the formula had the right form
  ###
  if(
     length(formula)!=3 ||
     length(formula[[3]]) != 3  ||
     formula[[3]][[1]] != as.name("|")
     )
    stop("Formula must be of the form y ~ x1 | x2")

  if(!missing(subset))
    {
      flag <- eval(substitute(subset), envir=data)
      data <- data[flag,]
    }

  ###
  # Get the actual formula values
  ###
  cond <- eval(formula[[3]][[3]], envir=data, parent.frame())
  x <- eval(formula[[3]][[2]], envir=data, parent.frame())
  y <- eval(formula[[2]], envir=data, parent.frame())

  #print(data.frame(cond,x,y))

  y.all.min <- min(y, na.rm=TRUE)
  y.all.max <- max(y, na.rm=TRUE)

  x.all.min <- min(x, na.rm=TRUE)
  x.all.max <- max(x, na.rm=TRUE)

  if (length(cond) == 0) {
    cond <- list(as.factor(rep(1, length(x))))
  }
  if (!is.factor(cond))
    {
      cond <- factor(cond)
    }


  ###
  # create a new call to the requested function
  ###
  mycall <- match.call(expand.dots=FALSE)
  mycall$panel <- mycall$plot <- mycall$groups <- mycall$same.scale <- NULL
  mycall$min.y <- mycall$max.y <- NULL
  mycall$data <- data

  # remove condition from formula
  mycall$formula[3] <- formula[[3]][2]

  # function name
  if(is.character(panel))
    panel <- as.name(panel)
  else
    panel <- deparse(substitute(panel))

  mycall[[1]] <- as.name(panel)

  # ylim
  if(same.scale)
    {
      if(missing(ylim))
        mycall$ylim <- range(y[y>0],na.rm=TRUE)
    }

  # xlim is always the same for all graphs
  if(missing(xlim))
    if(log %in% c("x","xy"))
      mycall$xlim <- range(x[x>0],na.rm=TRUE)
    else
      mycall$xlim <- range(x,na.rm=TRUE)



  ###
  # Only plot groups with non-na entries
  ##
  tmp <- na.omit(data.frame(x,y,cond))
  leveln <- sapply( split(tmp, tmp$cond), nrow )

  if(missing(groups)) groups <- names(leveln)
  ngroups <- length(groups)

  if(!missing(min.y) && length(min.y==1))
    min.y <- rep(min.y, length=ngroups)

  if(!missing(max.y) && length(max.y==1))
    max.y <- rep(max.y, length=ngroups)

  ###
  # Set up the plot regions
  ###
  oldpar <- par()['mar']
  on.exit(par(oldpar))

  par(mar=par("mar") +c(0,ngroups*2.5,0,0))

  ###
  # Ok. Now iterate over groups
  ##

    i <- 1
    for(level in groups)
      {

        if(i>1)
          {
            par(new=TRUE)
          }

        mycall$subset <- (cond==level)

        mycall$ylab <- ''
        mycall$xlab <- ''
        mycall$xaxt = 'n'
        mycall$yaxt = 'n'
        mycall$pch = i
        mycall$col = i
        mycall$lty = i

        tmp.y <- y[mycall$subset & cond==level]

        ## If nothing to plot, skip to next level
        if(!any(is.finite(tmp.y))) next
        
        min.tmp.y <- min(tmp.y, na.rm=TRUE)
        max.tmp.y <- max(tmp.y, na.rm=TRUE)

        if( !missing(min.y) || !missing(max.y) )
          {
            if(!missing(min.y) && !missing(max.y) )
              {
                mycall$ylim <- c(min.y[i], max.y[i])
              }
            else if(missing(min.y) && !missing(max.y))
              {
                if(same.scale)
                  mycall$ylim <- c(y.all.min, max.y[i])
                else
                  mycall$ylim <- c(min.tmp.y, max.y[i])
              }
            else # !missing(min.y) && missing(max.y)
              {
                if(same.scale)
                  mycall$ylim <- c(min.y[i], y.all.max)
                else
                  mycall$ylim <- c(min.y[i], max.tmp.y)
              }
          }

        if(plot)
          {
            status <- try(
                eval(mycall, parent.frame())
                )
            if('try-error' %in% class(status))
              break;

          }

        usr <- par("usr")

        axis(side=2,line=2.5*(i-1),lty=i,col=i,lwd=2)
        title(ylab=level, line=2.5*(i-1))

        if(i == 1)
          axis(side=1)

        i <- i+1

      }


  if(missing(xlab))
      xlab <- as.character(formula[[3]][[2]])

  if(missing(ylab))
      ylab <- as.character(formula[[2]])

  if(missing(main))
    main <- paste("plot of ", xlab, "vs.", ylab, "by",
                  as.character(formula[[3]][[3]]))

  if(i>1)
    {
      title(main=main,xlab=xlab)
      title(ylab=ylab, line=2.5*(i-1))
    }

  par(oldpar)

  invisible( split(data.frame(x,y),cond) )
}

## Function to replace all tabs in a string with an appropriate number of spaces.

# handle a single character string
replaceTabs.inner <-  function( text, width=8 )
{
  spaces <- "        "

  if(nchar(text)<1) return(text)
  
  text.split <- strsplit(text,"\t")[[1]]
  if(length(text.split)==1)
    return(text)
  else
    {
      nSpaceAdd <- 8 - nchar(text.split) %% 8
      nSpaceAdd[length(nSpaceAdd)] <- 0
      nSpaceAdd[nSpaceAdd==8] <- 0
      
      retval <- ""
      for(i in 1:length(text.split))
        {
          tmp.text <- chartr("\t"," ", text.split[i]) # one space here
          retval <- paste(retval, tmp.text, substr(spaces,0,nSpaceAdd[i]-1 ), sep='' ) # rest here
        }
      return(retval)
    }
}

replaceTabs <- function(text, width=8)
{
  text <- as.character(text)
  retval <- sapply(text, replaceTabs.inner) 
  names(retval) <- names(text)
  retval
}

## $Id$

print.ci2d <- function(x, ...)
  {
    cat("\n")
    cat("----------------------------\n")    
    cat("2-D Confidence Region Object\n")
    cat("----------------------------\n")    
    cat("\n")
    cat("Call: ")
    print(x$call)
    cat("\n")
    cat("Number of data points: ", x$nobs, "\n")
    cat("Number of grid points: ", length(x$x), "x", length(x$y), "\n")
    cat("Number of confidence regions:", length(x$contours), "\n")
    cat("\n")

    tab <- data.frame(
                      "Region"=1:length(x$contours),
                      "CI Level"=as.numeric(names(x$contours)),
                      "X Min"=sapply(x$contours, function(XX) min(XX$x)),
                      "X Max"=sapply(x$contours, function(XX) max(XX$x)),
                      "Y Min"=sapply(x$contours, function(XX) min(XX$y)),
                      "Y Max"=sapply(x$contours, function(XX) max(XX$y))
                      )

    print(tab, row.names=FALSE, ...)

    x$summary <- tab

    class(x) <- c("ci2d.summary", "ci2d")

    return(x)
  }

# $Id: bandplot.R 1906 2014-12-02 01:46:35Z warnes $

bandplot <- function(x, ...)
    UseMethod("bandplot")

bandplot.default  <-  function(x, y,
                       ...,
                       add=FALSE,
                       sd=c(-2:2),
                       sd.col=c("magenta","blue","red",
                                 "blue","magenta"),
                       sd.lwd=c(2,2,3,2,2),
                       sd.lty=c(2,1,1,1,2),
                       method="frac", width=1/5,
                       n=50
                       )
  {

    if(length(sd.col)<length(sd)) sd <-rep(sd.col,length=length(sd))
    if(length(sd.lwd)<length(sd)) sd <-rep(sd.lwd,length=length(sd))
    if(length(sd.lty)<length(sd)) sd <-rep(sd.lty,length=length(sd))
    if(!add)
      {
        m <- match.call(expand.dots = TRUE)
        m$width  <- m$add  <- m$sd  <- m$sd.col  <- NULL
        m$method <- m$n <- m$sd.lty <- m$sd.lwd  <- NULL
        m[[1]] <- as.name("plot")
        mf <- eval(m, parent.frame())
      }

    x  <- as.numeric(x)
    y  <- as.numeric(y)

    CL <- function(x,sd)
      if(length(x)<2)
        return( NA )
      else
        mean(x)+sd*sqrt(var(x))

    ord <- order(x)
    myx <- x[ord]
    myy <- y[ord]
    sdVec <- runsd(myy, k=floor(length(x)*width))
    meanVec <- runmean(myy, k=floor(length(x)*width) )

    stdevs <- list()
    for( i in 1:length(sd) )
      {
          lines(myx,
                meanVec + sdVec * sd[i],
                col=sd.col[i],
                lwd=sd.lwd[i],
                lty=sd.lty[i]
                )

      }

  }


bandplot.formula <- function(x,
                             data,
                             subset,
                             na.action,
                             ...,
                             xlab=NULL,
                             ylab=NULL,
                             add=FALSE,
                             sd=c(-2:2),
                             sd.col=c("magenta","blue","red",
                                      "blue","magenta"),
                             sd.lwd=c(2,2,3,2,2),
                             sd.lty=c(2,1,1,1,2),
                             method="frac", width=1/5,
                             n=50
                             )
    {
        if (missing(x) || (length(x) != 3))
            stop("x missing or incorrect")
        if (missing(na.action))
            na.action <- getOption("na.action")

        ## construct call to model.frame to generate model matrix
        mf <- match.call(expand.dots = FALSE)
        m <- match(c("x", "data", "subset", "na.action"), names(mf), 0L)
        mf <- mf[c(1L, m)]
        mf$drop.unused.levels <- TRUE
        mf[[1L]] <- quote(stats::model.frame)

        ## rename 'x' to 'formula'
        names(mf)[2] <- "formula"

        mf <- eval(mf, parent.frame())

        ## now extract x and y
        response <- attr(attr(mf, "terms"), "response")
        x <- mf[[-response]]
        y <- mf[[response]]

        ## get x and y axis labels
        sx <- substitute(x)

        if (is.null(xlab)) {
            if (mode(x) != "name")
                xlab <- deparse(sx[[3L]])
            else
                xlab <- "x"
        }

        if (is.null(ylab)) {
            if (mode(x) != "name")
                ylab <- deparse(sx[[2L]])
            else
                ylab <- "y"
        }

        ## now call the default method to actually do the work
        bandplot.default(x,
                 y,
                 ...,
                 xlab=xlab,
                 ylab=ylab,
                 add=add,
                 sd=sd,
                 sd.col=sd.col,
                 sd.lwd=sd.lwd,
                 sd.lty=sd.lty,
                 method=method,
                 width=width,
                 n=n
                 )
    }

# $Id: residplot.R 1012 2006-11-14 22:25:06Z ggorjan $

residplot  <-  function(model, formula, ...)
  {
    data  <- expand.model.frame( model, formula, na.expand=TRUE)

    newform  <- eval(parse( text=paste("as.call(", "resid(model) ~",
                        formula[-1],")" )))

    plot( newform, data=data, ylab="Residuals")
    lines(lowess( newform, data=data ), col="red")
    bandplot(newform,data=data)
  }

suppressMessages(library(gplots))

col <- "#123456"
col2 <- adjust_hsv(col, h=0.1)  # h
col3 <- adjust_hsv(col, s=0.1)  # s
col4 <- adjust_hsv(col, v=0.7)  # v
col5 <- adjust_hsv("blue")      # name
col6 <- adjust_hsv(4)           # integer
col7 <- adjust_hsv(palette(), alpha=0.1)  # multiple colors, alpha

cols <- c(col2, col3, col4, col5, col6, col7)

stopifnot(all(is.character(cols)))            # character
stopifnot(all(substring(cols, 1, 1) == "#"))  # start with #
stopifnot(all(nchar(cols) %in% c(7, 9)))      # either 7 or 9 chars long

barplot(rep(1, length(cols)), col=cols, space=0, border=NA)  # can be used

bubbleplot <- function(x, ...)
{
  UseMethod("bubbleplot")
}

bubbleplot.default <- function(x, y, z, std=TRUE, pow=0.5, add=FALSE, rev=FALSE,
                               type="p", ylim=NULL, xlab=NULL, ylab=NULL,
                               pch=c(16,1), cex.points=1, col="black",
                               bg=par("bg"), ...)
{
  pch <- rep(pch, length.out=2)
  col <- rep(col, length.out=2)
  bg <- rep(bg, length.out=2)

  if(is.matrix(x))  # matrix or table
  {
    dnn <- names(dimnames(x))
    if(is.null(xlab))
      xlab <- if(is.null(dnn)) "" else dnn[2]
    if(is.null(ylab))
      ylab <- if(is.null(dnn)) "" else dnn[1]
    y <- as.data.frame(as.table(x))[[1]]
    z <- as.data.frame(as.table(x))[[3]]
    x <- as.data.frame(as.table(x))[[2]]
  }

  if(is.list(x))  # data.frame or list
  {
    xyz <- all(c("x","y","z") %in% names(x))  # colnames include x,y,z
    if(is.null(xlab))
      xlab <- if(xyz) "x" else names(x)[1]
    if(is.null(ylab))
      ylab <- if(xyz) "y" else names(x)[2]
    if(xyz)
    {
      y <- x$y
      z <- x$z
      x <- x$x
    }
    else
    {
      y <- x[[2]]
      z <- x[[3]]
      x <- x[[1]]
    }
  }

  if(is.null(xlab))
    xlab <- deparse(substitute(x))
  if(is.null(ylab))
    ylab <- deparse(substitute(y))

  ## Convert factor to numeric
  x <- type.convert(as.character(x), as.is=TRUE)
  y <- type.convert(as.character(y), as.is=TRUE)

  if(is.null(ylim))
    ylim <- range(y)
  if(rev)
    ylim <- rev(ylim)

  if(std)
    mycex <- cex.points * (abs(z)/mean(abs(z)))^pow
  else
    mycex <- cex.points * abs(z)^pow

  if(!add)
    suppressWarnings(plot(x, y, type="n", ylim=ylim, xlab=xlab, ylab=ylab, ...))
  suppressWarnings(points(x[z>0], y[z>0], type=type, cex=mycex[z>0],
                          pch=pch[1], col=col[1], bg=bg[1], ...))
  suppressWarnings(points(x[z<0], y[z<0], type=type, cex=mycex[z<0],
                          pch=pch[2], col=col[2], bg=bg[2], ...))
}

bubbleplot.formula <- function(formula, data, subset, na.action=NULL, ...)
{
  m <- match.call(expand.dots=FALSE)
  if(is.matrix(eval(m$data,parent.frame())))
    m$data <- as.data.frame(data)
  m$... <- NULL
  m[[1L]] <- quote(model.frame)
  mf <- eval(m, parent.frame())

  if(ncol(mf) < 3)
    stop("formula must have the form z ~ x + y")

  bubbleplot.default(mf[c(2,3,1)], ...)
}

---
title: "Enhanced Heat Maps with heatmap.2"
date: "`r Sys.Date()`"
author: "Andy Liaw, original; R. Gentleman, M. Maechler, W. Huber, G. Warnes, revisions. Tal Galili"
output: 
  html_vignette: 
    toc: yes
    code_folding: show
editor_options: 
  chunk_output_type: console
---
<!--
  %\VignetteIndexEntry{Enhanced Heat Maps with heatmap.2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
-->


```{r setup, include=FALSE}
knitr::opts_chunk$set(
  echo = TRUE,
  warning = FALSE,
  message = FALSE,
  fig.width = 8,
  fig.height = 6,
  fig.align = "center"
)
```

# Introduction

The `heatmap.2` function provides an enhanced version of the standard R `heatmap` function with numerous additional features and customization options. This vignette demonstrates the key capabilities and usage patterns of `heatmap.2`. It is based on the manual for `?heatmap.2`, written by Andy Liaw, original; R. Gentleman, M. Maechler, W. Huber, G. Warnes, revisions.


# Details on Dendrograms, Scaling, and Plot Layout

### Dendrogram Behavior (`Rowv`, `Colv`)

* If either **`Rowv`** or **`Colv`** are already **dendrograms**, they are **honored** and their ordering is preserved (not reordered).
* Otherwise, dendrograms are computed as $dd \leftarrow \text{as.dendrogram}(\text{hclustfun}(\text{distfun}(X)))$, where $X$ is either $x$ or $t(x)$.
* If either is a **vector** (of "weights"), the appropriate dendrogram ($dd$) is **reordered** according to the supplied values, subject to the constraints imposed by the dendrogram, using $\texttt{reorder}(dd, Rowv)$ (in the row case).
* If either is **missing** (as by default), the ordering of the corresponding dendrogram is by the **mean value** of the rows/columns (e.g., $Rowv \leftarrow \text{rowMeans}(x, \text{na.rm=na.rm})$ in the row case).
* If either is **`NULL`**, **no reordering** will be done for the corresponding side.


### Data Scaling (`scale`)

* If **`scale="row"`** (or **`scale="col"`**), the rows (columns) are scaled to have **mean zero** and **standard deviation one**.
* This scaling has some empirical evidence of being useful in genomic plotting.

### Color Selection

* The **default colors** range from red to white ($\texttt{heat.colors}$) and are generally not considered aesthetically pleasing.
* Consider using enhancements like the **RColorBrewer** package to select better colors.

### Plot Layout (Default and Customization)

* By default, **four components** are displayed in the plot:
    * **Top left:** The **color key**.
    * **Top right:** The **column dendrogram**.
    * **Bottom left:** The **row dendrogram**.
    * **Bottom right:** The **image plot** (heatmap).
* When **`RowSideColor`** or **`ColSideColor`** are provided, an **additional row or column** is inserted in the appropriate location.
* This layout can be overridden by specifying appropriate values for **`lmat`**, **`lwid`**, and **`lhei`**:
    * **`lmat`** controls the **relative position** of each element.
    * **`lwid`** controls the **column width**.
    * **`lhei`** controls the **row height**.
* See the help page for **`layout`** for details on how to use these arguments.

## Note

* The original rows and columns are **reordered** to match the dendrograms **`Rowv`** and **`Colv`** (if present).
* **`heatmap.2()`** uses **`layout`** to arrange the plot elements. Consequentially, it **cannot** be used in a multi-column/row layout using **`layout(...)`**, **`par(mfrow=...)`** or **`(mfcol=...)`**.

# Load Data

```{r load-libraries}
# Load required libraries
library(gplots)  # for heatmap.2
library(RColorBrewer)  # for better color palettes
```

```{r}
data(mtcars)
x  <- as.matrix(mtcars)
rc <- rainbow(nrow(x), start=0, end=.3)
cc <- rainbow(ncol(x), start=0, end=.3)

```

# Dendrogram Options

## demonstrate the effect of row and column dendrogram options

```{r}
heatmap.2(x)                    ## default - dendrogram plotted and reordering done.
heatmap.2(x, dendrogram="none") ##  no dendrogram plotted, but reordering done.
heatmap.2(x, dendrogram="row")  ## row dendrogram plotted and row reordering done.
heatmap.2(x, dendrogram="col")  ## col dendrogram plotted and col reordering done.

heatmap.2(x, keysize=2)         ## default - dendrogram plotted and reordering done.

heatmap.2(x, Rowv=FALSE, dendrogram="both") ## generates a warning!
heatmap.2(x, Rowv=NULL, dendrogram="both")  ## generates a warning!
heatmap.2(x, Colv=FALSE, dendrogram="both") ## generates a warning!

```


## Reorder dendrogram by branch means rather than sums
```{r}
heatmap.2(x, reorderfun=function(d, w) reorder(d, w, agglo.FUN = mean) )

```


## Color dendrograms' branches (using dendextend)

```{r, fig.keep='none'}
full <- heatmap.2(x) # we use it to easily get the dendrograms
```


```{r}
library(dendextend)  # for color_branches
heatmap.2(x, 
        Rowv=color_branches(full$rowDendrogram, k = 3),
        Colv=color_branches(full$colDendrogram, k = 2))
# Look at the vignette for more details:
# https://cran.r-project.org/web/packages/dendextend/vignettes/dendextend.html

```


## plot a sub-cluster using the same color coding as for the full heatmap
```{r}
full <- heatmap.2(x)
heatmap.2(x, Colv=full$colDendrogram[[2]], breaks=full$breaks)  # column subset
heatmap.2(x, Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # row subset
heatmap.2(x, Colv=full$colDendrogram[[2]],
            Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # both


```


# Show effect of row and column label rotation
```{r}

heatmap.2(x, srtCol=NULL)
heatmap.2(x, srtCol=0,   adjCol = c(0.5,1) )
heatmap.2(x, srtCol=45,  adjCol = c(1,1)   )
heatmap.2(x, srtCol=135, adjCol = c(1,0)   )
heatmap.2(x, srtCol=180, adjCol = c(0.5,0) )
heatmap.2(x, srtCol=225, adjCol = c(0,0)   ) ## not very useful
heatmap.2(x, srtCol=270, adjCol = c(0,0.5) )
heatmap.2(x, srtCol=315, adjCol = c(0,1)   )
heatmap.2(x, srtCol=360, adjCol = c(0.5,1) )

heatmap.2(x, srtRow=45, adjRow=c(0, 1) )
heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=45, adjCol=c(1,1) )
heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=270, adjCol=c(0,0.5) )

```

# Show effect of offsetRow/offsetCol (only works when srtRow/srtCol is
```{r}
## not also present)
heatmap.2(x, offsetRow=0, offsetCol=0)
heatmap.2(x, offsetRow=1, offsetCol=1)
heatmap.2(x, offsetRow=2, offsetCol=2)
heatmap.2(x, offsetRow=-1, offsetCol=-1)

heatmap.2(x, srtRow=0, srtCol=90, offsetRow=0, offsetCol=0)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=1, offsetCol=1)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=2, offsetCol=2)
heatmap.2(x, srtRow=0, srtCol=90, offsetRow=-1, offsetCol=-1)

```

# Show how to use 'extrafun' to replace the 'key' with a scatterplot
```{r}
lmat <- rbind( c(5,3,4), c(2,1,4) )
lhei <- c(1.5, 4)
lwid <- c(1.5, 4, 0.75)

myplot <- function() {
           oldpar <- par("mar")
           par(mar=c(5.1, 4.1, 0.5, 0.5))
           plot(mpg ~ hp, data=x)
         }

heatmap.2(x, lmat=lmat, lhei=lhei, lwid=lwid, key=FALSE, extrafun=myplot)

```

# show how to customize the color key
```{r}

heatmap.2(x,
         key.title=NA, # no title
         key.xlab=NA,  # no xlab
         key.par=list(mgp=c(1.5, 0.5, 0),
                      mar=c(2.5, 2.5, 1, 0)),
         key.xtickfun=function() {
               breaks <- parent.frame()$breaks
               return(list(
                    at=parent.frame()$scale01(c(breaks[1],
                                                breaks[length(breaks)])),
                    labels=c(as.character(breaks[1]),
                             as.character(breaks[length(breaks)]))
                    ))
         })

heatmap.2(x,
        breaks=256,
        key.title=NA,
        key.xlab=NA,
        key.par=list(mgp=c(1.5, 0.5, 0),
                     mar=c(1, 2.5, 1, 0)),
        key.xtickfun=function() {
             cex <- par("cex")*par("cex.axis")
             side <- 1
             line <- 0
             col <- par("col.axis")
             font <- par("font.axis")
             mtext("low", side=side, at=0, adj=0,
                   line=line, cex=cex, col=col, font=font)
             mtext("high", side=side, at=1, adj=1,
                   line=line, cex=cex, col=col, font=font)
             return(list(labels=FALSE, tick=FALSE))
        })


```

# Show effect of z-score scaling within columns, blue-red color scale
```{r}
##
##
hv <- heatmap.2(x, col=bluered, scale="column", tracecol="#303030")

###
## Look at the return values
###
names(hv)

## Show the mapping of z-score values to color bins
hv$colorTable

## Extract the range associated with white
hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",]

## Determine the original data values that map to white
whiteBin <- unlist(hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",1:2])
rbind(whiteBin[1] * hv$colSDs + hv$colMeans,
     whiteBin[2] * hv$colSDs + hv$colMeans )
##
## A more decorative heatmap, with z-score scaling along columns
##
hv <- heatmap.2(x, col=cm.colors(255), scale="column",
       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
       xlab="specification variables", ylab= "Car Models",
       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
       tracecol="green", density="density")
## Note that the breakpoints are now symmetric about 0

## Color the labels to match RowSideColors and ColSideColors
hv <- heatmap.2(x, col=cm.colors(255), scale="column",
       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
       xlab="specification variables", ylab= "Car Models",
       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
       tracecol="green", density="density", colRow=rc, colCol=cc,
       srtCol=45, adjCol=c(0.5,1))



```


# Correlation Heatmaps
```{r}
# TODO: want example using the `add.exp' argument!

data(attitude)
round(Ca <- cor(attitude), 2)
symnum(Ca) # simple graphic

# with reorder
heatmap.2(Ca, 		 symm=TRUE, margin=c(6, 6), trace="none" )

# without reorder
heatmap.2(Ca, Rowv=FALSE, symm=TRUE, margin=c(6, 6), trace="none" )

## Place the color key below the image plot
heatmap.2(x, lmat=rbind( c(0, 3), c(2,1), c(0,4) ), lhei=c(1.5, 4, 2 ) )

## Place the color key to the top right of the image plot
heatmap.2(x, lmat=rbind( c(0, 3, 4), c(2,1,0 ) ), lwid=c(1.5, 4, 2 ) )

```



## For variable clustering, rather use distance based on cor():
```{r}
data(USJudgeRatings)
symnum( cU <- cor(USJudgeRatings) )

hU <- heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=topo.colors(16),
            distfun=function(c) as.dist(1 - c), trace="none")

## The Correlation matrix with same reordering:
hM <- format(round(cU, 2))
hM

```


now with the correlation matrix on the plot itself
```{r}

heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=rev(heat.colors(16)),
           distfun=function(c) as.dist(1 - c), trace="none",
           cellnote=hM)

```


# Interactive heatmap.2 using heatmaply
```{r}
data(mtcars)
x  <- as.matrix(mtcars)

library(heatmaply)
# just use heatmaply instead of heatmap.2
heatmaply(x)

```


The default are slightly different, but it supports most of the same arguments.
If you want the dendrograms to match perfectly, use this:

```{r, fig.keep='none'}
full <- heatmap.2(x) # we use it to easily get the dendrograms
```


```{r}
## Color branches of dendrograms by cluster membership using dendextend:
heatmaply(x, 
        Rowv=color_branches(full$rowDendrogram, k = 3),
        Colv=color_branches(full$colDendrogram, k = 2))
# Look at the vignette for more details:
# https://cran.r-project.org/web/packages/heatmaply/vignettes/heatmaply.html

```



# Session Information

```{r session-info}
sessionInfo()
```

# References

- Original `heatmap` function: R Core Team
- `heatmap.2` enhancements: Andy Liaw, R. Gentleman, M. Maechler, W. Huber, G. Warnes
- `gplots` package documentation
- RColorBrewer package for color palettes

---
title: "Venn Diagrams with `gplots`"
date: "`r Sys.Date()`"
author: "Steffen Möller"
output: 
  html_vignette: 
    toc: yes
editor_options: 
  chunk_output_type: console
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Venn Diagrams with `gplots`}
%\VignetteEncoding{UTF-8}
-->


Venn diagrams [Wikipedia](https://en.wikipedia.org/wiki/Venn_diagram) allow for a quick overview on the number of elements that multiple sets share. When those elements represent traits of real objects, like observations in biomedical sciences, marketing, etc., this may direct researchers to further investigations or decisions.

The `gplots` package provides Venn diagrams for up to five sets. The R code to produce the diagrams is straightforward. The plot function behaves the same, depending only on the number of overlapping circles to draw. Its input is a table that is produced by another function. The `venn()` function calls one after the other and is the only one to be seen by the user. The values shown are returned invisibly.

The `venn()` function accepts either a list of sets as an argument, or it takes a binary matrix—one column per set—indicating for every element, one per row, the membership with every set.

The common form with overlapping circles works with up to three sets, as seen here:

```{r, fig=TRUE, echo=TRUE, message=FALSE}
suppressMessages(library(gplots))
venn( list(A=1:5,B=4:6,C=c(4,8:10)) )
```


The names of columns or the list elements are the set names. To squeeze extra circles in, those circles need to become ellipses. This works for four sets:

```{r, fig=TRUE, echo=TRUE, message=FALSE}
v.table <- venn( list(A=1:5,B=4:6,C=c(4,8:10),D=c(4:12)) )
print(v.table)
```


And maybe even more impressively for five:

```{r, fig=TRUE, echo=TRUE, message=FALSE}
venn( list(A=1:5,B=4:6,C=c(4,8:10),D=c(4:12),E=c(2,4,6:9)) )
```

The man page of venn() lists options to change the appearance of the plots, e.g., the names of the sets may be omitted, and sizes changed. However, there is ample opportunity to extend the functionality of this package, such as:

* More dimensions
* Colors
* Variation of size of circles with the number of members in the set
* Density plot rather than numbers, identification of individual entries

The prime personal interest is in the increase of dimensions. Please send patches for features you are most interested in.

\documentclass[a4paper]{article}

%\VignetteIndexEntry{Venn Diagrams with gplots}

\title{Venn Diagrams with \texttt{gplots}}
\author{Steffen M{\"{o}}ller}

\begin{document}
\SweaveOpts{concordance=TRUE}

\maketitle

Venn diagrams\footnote{https://en.wikipedia.org/wiki/Venn\_diagram} 
allow for a quick overview on the number of elements that multiple
sets share. And when those elements are representing traits of
real objects, like observations in biomedical sciences/marketing/...,
then this may direct researchers to further investigations/decisions.

The {\tt gplots} package provides Venn diagrams for up to five sets.
The R code to produce the diagrams is not complicated. The plot
function behaves alwas the same, depending only on the number of
overlapping circles to draw. Its input is a table that is produced
by another function. The function {\tt venn()} calls one after the
other and is the only one to be seen by the user. The values shown
are returned invisibly.

The {\tt venn()} function accepts either a list of sets as an argument,
or it takes a binary matrix, one column per set, indicating for every
element, one per row, the membership with every set.

The common form with overlapping circles only works with up to three
sets, as seen here:
\begin{center}
<<fig=TRUE,echo=TRUE>>=
suppressMessages(library(gplots))
venn( list(A=1:5,B=4:6,C=c(4,8:10)) )
@
\end{center}

The names of columns or the list elements are the set names.
To squeeze extra circles in, those circles need to become ellipses.
This works for four sets
\begin{center}
<<fig=TRUE,echo=TRUE>>=
v.table<-venn( list(A=1:5,B=4:6,C=c(4,8:10),D=c(4:12)) )
print(v.table)
@
\end{center}

and maybe even more impressively also for five.
\begin{center}
<<fig=TRUE,echo=FALSE>>=
venn( list(A=1:5,B=4:6,C=c(4,8:10),D=c(4:12),E=c(2,4,6:9)) )
@
\end{center}

The man page of {\it venn()} lists options to change the appearance of
the plots, e.g., the names of the sets may be omitted and sizes changed.
However, there is ample of opportunity to extend the functionality of
this package.  To mind come

\begin{itemize}
\item more dimensions (next)
\item colors
\item variation of size of circles with the number of members the set has
\item density plot rather than numbers, identification of individual entries
\end{itemize}

The prime personal interest is more in the increase of dimensions. Please
send patches for features you are most interested in.

\end{document}

\name{venn}
\alias{venn}
\alias{plot.venn}
\title{Plot a Venn diagram}
\description{
  Plot a Venn diagrams for up to 5 sets
}
\usage{
venn(data, universe=NA, small=0.7, showSetLogicLabel=FALSE,
     simplify=FALSE, show.plot=TRUE, intersections=TRUE, names,
     ...)

\method{plot}{venn}(x, y, ..., small=0.7, showSetLogicLabel=FALSE,
     simplify=FALSE)
}
\arguments{
  \item{data,x}{Either a list list containing vectors of names or indices
    of group intersections, or a data frame containing boolean indicators of
    group intersectionship (see below)}
  \item{universe}{Subset of valid name/index elements. Values  ignore values
    in code{data} not in this list will be ignored. Use \code{NA} to
    use all elements of \code{data} (the default).}
  \item{small}{Character scaling of the smallest group counts}
  \item{showSetLogicLabel}{Logical flag indicating whether the internal
    group label should be displayed}
  \item{simplify}{Logical flag indicating whether unobserved groups
    should be omitted.}
  \item{show.plot}{Logical flag indicating whether the plot should be
    displayed.  If false, simply returns the group count matrix.}
  \item{intersections}{Logical flag indicating
    if the returned object should have the attribute
    "individuals.in.intersections" featuring for every set a list of
    individuals that are assigned to it.}
  \item{y}{Ignored}
  \item{...}{Optional graphical parameters.}
  \item{names}{Optional vector of group names.}
}
\details{
  \code{data} should be either a named list of vectors containing
  character string names ("GeneAABBB", "GeneBBBCY", .., "GeneXXZZ") or
  indexes of group intersections (1, 2, .., N), or a data frame containing
  indicator variables (TRUE, FALSE, TRUE, ..) for group intersectionship.
  Group names will be taken from the component list element or column
  names.
}
\value{
  Invisibly returns an object of class "venn", containing:
  \itemize{
    \item A matrix of all possible sets of groups, and the observed count
      of items belonging to each The fist column contains observed
      counts, subsequent columns contain 0-1 indicators of group
      intersectionship.
    \item If \code{intersections=TRUE}, the attribute \code{intersections}
      will be a list of vectors containing the names of the elements
      belonging to each subset.
  }
}
\author{
  Steffen Moeller \email{steffen\_moeller@gmx.de},
  with cleanup and packaging by Gregory R. Warnes
  \email{greg@warnes.net}.}
\examples{

##
## Example using a list of item names belonging to the
## specified group.
##

## construct some fake gene names..
oneName <- function() paste(sample(LETTERS,5,replace=TRUE),collapse="")
geneNames <- replicate(1000, oneName())

##
GroupA <- sample(geneNames, 400, replace=FALSE)
GroupB <- sample(geneNames, 750, replace=FALSE)
GroupC <- sample(geneNames, 250, replace=FALSE)
GroupD <- sample(geneNames, 300, replace=FALSE)
input  <-list(GroupA,GroupB,GroupC,GroupD)
input

tmp <- venn(input)
attr(tmp, "intersections")

##
## Example using a list of item indexes belonging to the
## specified group.
##
GroupA.i <- which(geneNames \%in\% GroupA)
GroupB.i <- which(geneNames \%in\% GroupB)
GroupC.i <- which(geneNames \%in\% GroupC)
GroupD.i <- which(geneNames \%in\% GroupD)
input.i  <-list(A=GroupA.i,B=GroupB.i,C=GroupC.i,D=GroupD.i)
input.i

venn(input.i)

##
## Example using a data frame of indicator ('f'lag) columns
##
GroupA.f <- geneNames \%in\% GroupA
GroupB.f <- geneNames \%in\% GroupB
GroupC.f <- geneNames \%in\% GroupC
GroupD.f <- geneNames \%in\% GroupD
input.df <- data.frame(A=GroupA.f,B=GroupB.f,C=GroupC.f,D=GroupD.f)
head(input.df)
venn(input.df)

## smaller set to create empty groupings
small <- input.df[1:20,]

venn(small, simplify=FALSE) # with empty groupings
venn(small, simplify=TRUE)  # without empty groupings

## Capture group counts, but don't plot
tmp <- venn(input, show.plot=FALSE)
tmp

## Show internal binary group labels
venn(input, showSetLogicLabel=TRUE)

## Limit  universe
tmp <- venn(input, universe=geneNames[1:100])
tmp

##
## Example to determine which elements are in A and B but not in
## C and D using the 'intersections' attribute.
##
tmp <- venn(input, intersection=TRUE)
isect <- attr(tmp, "intersection")

# Look at all of the subsets
str(isect)

# Extract and combine the subsets of interest..
AandB <- unique(c(isect$A, isect$B, isect$'A:B'))

# and look at the results
str(AandB)

##
## The full set of elements of each intersection is provided in the
## "interesections" attribute.
##
a<-venn(list(1:5,3:8), show.plot=FALSE)
intersections<-attr(a,"intersections")
print(intersections)
# $A
# [1] "1" "2"
#
# $B
# [1] "6" "7" "8"
#
# $`A:B`
# [1] "3" "4" "5"
}
\keyword{hplot}

% $Id: residplot.Rd 1441 2010-06-11 03:11:54Z warnes $

\name{residplot}
\alias{residplot}
%\alias{iapply}
\title{Undocumented functions}
\description{
  These functions are undocumented.  Some are internal and not intended
  for direct use.  Others simply haven't been documented yet.  
}
\usage{
residplot(model, formula, ...) 
}
\arguments{
  \item{model}{Undocumented}
  \item{formula}{Undocumented}
    \item{\dots}{ arguments to be passed to \code{fun} }
  }
\details{
 These functions are undocumented.  Some are internal and not intended
   for direct use.  Others simply haven't been documented yet.
   }
\author{Gregory R. Warnes \email{greg@warnes.net}}
\keyword{misc}

% $Id$
\name{ci2d}
\alias{ci2d}
\alias{print.ci2d}
\title{
  Create 2-dimensional empirical confidence regions
}
\description{
  Create 2-dimensional empirical confidence regions from provided data.
}
\usage{
ci2d(x, y = NULL,
     nbins=51, method=c("bkde2D","hist2d"),
     bandwidth, factor=1.0,
     ci.levels=c(0.50,0.75,0.90,0.95,0.975),
     show=c("filled.contour","contour","image","none"),
     col=topo.colors(length(breaks)-1),
     show.points=FALSE,
     pch=par("pch"),
     points.col="red",
     xlab, ylab, 
     ...)
\method{print}{ci2d}(x, ...)
}
\arguments{
  \item{x}{either a vector containing the x coordinates
    or a matrix with 2 columns. }
  \item{y}{a vector contianing the y coordinates, not required if `x'
    is matrix}
  \item{nbins}{number of bins in each dimension. May be a scalar or a
    2 element vector.  Defaults to 51.}
  \item{method}{One of "bkde2D" (for KernSmooth::bdke2d) or "hist2d"
    (for gplots::hist2d) specifyting the name of the method to create
    the 2-d density summarizing the data.  Defaults to "bkde2D".}
  \item{bandwidth}{Bandwidth to use for \code{KernSmooth::bkde2D}.
    See below for default value. }
  \item{factor}{Numeric scaling factor for bandwidth.  Useful for
    exploring effect of changing the bandwidth.  Defaults to 1.0.}
  \item{ci.levels}{Confidence level(s) to use for plotting
    data. Defaults to \code{c(0.5, 0.75, 0.9, 0.95, 0.975)} }
  \item{show}{Plot type to be displaed.  One of "filled.contour",
    "contour", "image", or "none".  Defaults to "filled.contour".}
  \item{show.points}{Boolean indicating whether original data values
    should be plotted.  Defaults to \code{TRUE}.}
  \item{pch}{Point type for plots.  See \code{points} for details.}
  \item{points.col}{Point color for plotting original data. Defaiults to
  "red".}
  \item{col}{Colors to use for plots.}
  \item{xlab, ylab}{Axis labels}
  \item{\dots}{Additional arguments passed to \code{KernSmooth::bkde2D}
    or \code{gplots::hist2d}. }
}
\details{
  This function utilizes either \code{KernSmooth::bkde2D} or
  \code{gplots::hist2d} to estmate a 2-dimensional density of the data
  passed as an argument.  This density is then used to create and
  (optionally) display confidence regions.

  When \code{bandwidth} is ommited and \code{method="bkde2d"},
  \code{KernSmooth::dpik} is appled in x and y dimensions to select the
  bandwidth.
  
}
\note{
  Confidence intervals generated by ci2d are \emph{approximate}, and
  are subject to biases and/or artifacts induced by the binning or
  kernel smoothing method, bin locations, bin sizes, and kernel bandwidth.

  The \code{\link[r2d2]{conf2d}} function in the \pkg{r2d2} package may create a more
  accurate confidence region, and reports the actual proportion of
  points inside the region.
  }
\value{
  A \code{ci2d} object consisting of a list containing (at least) the
  following elements:
  \item{nobs}{number of original data points}
  \item{x}{x position of each density estimate bin}
  \item{y}{y position of each density estimate bin}
  \item{density}{Matrix containing the probability density of each bin
    (count in bin/total count)}
  \item{cumDensity}{Matrix where each element contains the cumulative
    probability density of all elements with the same density (used to
    create the confidence region plots) }
  \item{contours}{List of contours of each confidence region.}
  \item{call}{Call used to create this object}
}
\author{ Gregory R. Warnes \email{greg@warnes.net}}
\seealso{
  \code{\link[KernSmooth]{bkde2D}}, \code{\link[r2d2]{conf2d}},
  \code{\link[KernSmooth]{dpik}}, \code{\link{hist2d}}
}
\examples{
   ####
   ## Basic usage 
   ####
   data(geyser, package="MASS")

   x <- geyser$duration
   y <- geyser$waiting

   # 2-d confidence intervals based on binned kernel density estimate
   ci2d(x,y)                   # filled contour plot
   ci2d(x,y, show.points=TRUE) # show original data


   # image plot
   ci2d(x,y, show="image")
   ci2d(x,y, show="image", show.points=TRUE)

   # contour plot
   ci2d(x,y, show="contour", col="black")
   ci2d(x,y, show="contour", col="black", show.points=TRUE)

   ####
   ## Control Axis scales
   ####
   x <- rnorm(2000, sd=4)
   y <- rnorm(2000, sd=1)

   # 2-d confidence intervals based on binned kernel density estimate
   ci2d(x,y)

   # 2-d confidence intervals based on 2d histogram
   ci2d(x,y, method="hist2d", nbins=25)
 
   # Require same scale for each axis, this looks oval
   ci2d(x,y, range.x=list(c(-20,20), c(-20,20)))
   ci2d(x,y, method="hist2d", same.scale=TRUE, nbins=25) # hist2d 

   ####
   ## Control smoothing and binning 
   ####
   x <- rnorm(2000, sd=4)
   y <- rnorm(2000, mean=x, sd=2)

   # Default 2-d confidence intervals based on binned kernel density estimate
   ci2d(x,y)

   # change the smoother bandwidth
   ci2d(x,y,
        bandwidth=c(sd(x)/8, sd(y)/8)
       )

   # change the smoother number of bins
   ci2d(x,y, nbins=10)
   ci2d(x,y)
   ci2d(x,y, nbins=100)

   # Default 2-d confidence intervals based on 2d histogram
   ci2d(x,y, method="hist2d", show.points=TRUE)

   # change the number of histogram bins
   ci2d(x,y, nbin=10, method="hist2d", show.points=TRUE )
   ci2d(x,y, nbin=25, method="hist2d", show.points=TRUE )

   ####
   ## Perform plotting manually
   ####
   data(geyser, package="MASS")

   # let ci2d handle plotting contours...
   ci2d(geyser$duration, geyser$waiting, show="contour", col="black")

   # call contour() directly, show the 90 percent CI, and the mean point 
   est <- ci2d(geyser$duration, geyser$waiting, show="none")
   contour(est$x, est$y, est$cumDensity,
           xlab="duration", ylab="waiting",
           levels=0.90, lwd=4, lty=2)
   points(mean(geyser$duration), mean(geyser$waiting),
         col="red", pch="X")


   ####
   ## Extract confidence region values
   ###
   data(geyser, package="MASS")

   ## Empirical 90 percent confidence limits
   quantile( geyser$duration, c(0.05, 0.95) )
   quantile( geyser$waiting, c(0.05, 0.95) )

   ## Bivariate 90 percent confidence region
   est <- ci2d(geyser$duration, geyser$waiting, show="none")
   names(est$contours) ## show available contours

   ci.90 <- est$contours[names(est$contours)=="0.9"]  # get region(s)
   ci.90 <- rbind(ci.90[[1]],NA, ci.90[[2]], NA, ci.90[[3]]) # join them

   print(ci.90)                  # show full contour
   range(ci.90$x, na.rm=TRUE)    # range for duration
   range(ci.90$y, na.rm=TRUE)    # range for waiting

   ####
   ## Visually compare confidence regions 
   ####
   data(geyser, package="MASS")

   ## Bivariate smoothed 90 percent confidence region
   est <- ci2d(geyser$duration, geyser$waiting, show="none")
   names(est$contours) ## show available contours

   ci.90 <- est$contours[names(est$contours)=="0.9"]  # get region(s)
   ci.90 <- rbind(ci.90[[1]],NA, ci.90[[2]], NA, ci.90[[3]]) # join them

   plot( waiting ~ duration, data=geyser,
         main="Comparison of 90 percent confidence regions" )
   polygon( ci.90, col="green", border="green", density=10)

   ## Univariate Normal-Theory 90 percent confidence region
   mean.x <- mean(geyser$duration)
   mean.y <- mean(geyser$waiting)
   sd.x <- sd(geyser$duration)
   sd.y <- sd(geyser$waiting)

   t.value <- qt(c(0.05,0.95), df=length(geyser$duration), lower=TRUE)
   ci.x <- mean.x +  t.value* sd.x
   ci.y <- mean.y +  t.value* sd.y

   plotCI(mean.x, mean.y,
          li=ci.x[1],
          ui=ci.x[2],
          barcol="blue", col="blue",
          err="x",
          pch="X",
          add=TRUE )

   plotCI(mean.x, mean.y,
          li=ci.y[1],
          ui=ci.y[2],
          barcol="blue", col="blue",
          err="y",
          pch=NA,
          add=TRUE )

#   rect(ci.x[1], ci.y[1], ci.x[2], ci.y[2], border="blue",
#        density=5,
#        angle=45,
#        col="blue" )


   ## Empirical univariate 90 percent confidence region
   box <- cbind( x=quantile( geyser$duration, c(0.05, 0.95 )), 
                 y=quantile( geyser$waiting, c(0.05, 0.95 )) )

   rect(box[1,1], box[1,2], box[2,1], box[2,2], border="red",
        density=5,
        angle=-45,
        col="red" )

   ## now a nice legend
   legend( "topright", legend=c("       Region type",
                                "Univariate Normal Theory",
                                "Univarite Empirical",
                                "Smoothed Bivariate"),
           lwd=c(NA,1,1,1),
           col=c("black","blue","red","green"),
           lty=c(NA,1,1,1)
         )

   ####
   ## Test with a large number of points
   ####
   \dontrun{
   x <- rnorm(60000, sd=1)
   y <- c( rnorm(40000, mean=x, sd=1),
           rnorm(20000, mean=x+4, sd=1) )

   hist2d(x,y)
   ci <- ci2d(x,y)
   ci
   }
}
\keyword{dplot}
\keyword{hplot}
\keyword{nonparametric}

\name{gplots-defunct}
\alias{boxplot.n}
\alias{plot.lm2}
\alias{smartlegend}
\title{Defunct functions}
\description{
  These functions are defunct and have been removed from the gplots package.
}
\usage{
   boxplot.n(..., top=FALSE, shrink=1, textcolor=NULL)
   plot.lm2(
            x,
            which = 1:5,
            caption = c("Residuals vs Fitted", "Normal Q-Q plot",
              "Scale-Location plot", "Cook's distance plot"),
            panel = panel.smooth,
            sub.caption = deparse(x$call),
            main = "",
            ask,
            ...,
            id.n = 3,
            labels.id = names(residuals(x)),
            cex.id = 0.75,
            band=TRUE,
            rug=TRUE,
            width=1/10,
            max.n=5000
            )
  smartlegend(x = c("left", "center", "right"),
             y = c("top", "center", "bottom"),
             ...,
             inset = 0.05)
}
\arguments{
  \item{ask,
        band,
        caption,
        cex.id,
        id.n,
        inset,
        labels.id,
        main,
        max.n,
        panel,
        rug,
        shrink,
        sub.caption,
        textcolor,
        top,
        which,
        width,
        x,
        y,
        \dots}{see man page for the corresponding replacement function}
}
\details{
These functions are no longer available.  Please refer to the manual page
for the replacement function:
  \itemize{
    \item \code{boxplot.n} has been replaced by \code{\link{boxplot2}}
    \item \code{plot.lm2} has been replaced by \code{\link{lmplot2}}
    \item \code{smartlegend} is no longer needed because relative
    positioning has been implemented in \code{\link[graphics]{legend}}.
  }
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{
  \code{\link{boxplot2}},
  \code{\link{lmplot2}},
  \code{\link[graphics]{legend}},
  \code{\link[base]{Defunct}}
}
\keyword{misc}

\name{bubbleplot}
\alias{bubbleplot}
\alias{bubbleplot.default}
\alias{bubbleplot.formula}
\title{Bubble Plot}
\description{
  Draw a bubble plot, a scatterplot with varying symbol sizes and
  colors, or add points to existing plots. A variety of input formats
  are supported, including vectors, matrices, data frames, formulas,
  etc.
}
\usage{
bubbleplot(x, \dots)

\method{bubbleplot}{default}(x, y, z, std=TRUE, pow=0.5, add=FALSE,
           rev=FALSE, type="p", ylim=NULL, xlab=NULL, ylab=NULL,
           pch=c(16,1), cex.points=1, col="black", bg=par("bg"), \dots)

\method{bubbleplot}{formula}(formula, data, subset, na.action=NULL, \dots)
}
\arguments{
  \item{x}{a vector of values for the horizontal axis. Can also be a
    2-dimensional matrix or table (x values in column names and y values
    in row names), or a data frame containing \code{x}, \code{y}, and
    \code{z} in that order. If the data frame contains column names
    \code{x}, \code{y}, and \code{z} then they will be used for
    plotting.}
  \item{\dots}{passed to \code{plot} and \code{points}.}
  \item{y}{a vector of values for the vertical axis.}
  \item{z}{a vector of values determining the bubble sizes.}
  \item{std}{whether to standardize the \code{z} values.}
  \item{pow}{a power coefficient for the bubble sizes.}
  \item{add}{whether to add bubbles to an existing plot.}
  \item{rev}{whether to reverse the y axis.}
  \item{type}{passed to \code{points}.}
  \item{ylim}{passed to \code{plot}.}
  \item{xlab, ylab}{passed to \code{plot}.}
  \item{pch}{passed to \code{points}.}
  \item{cex.points}{scales all bubble sizes.}
  \item{col, bg}{passed to \code{points}.}
  \item{formula}{has the form \code{z ~ x + y}, where \code{z}
    determines the bubble sizes and \code{x} and \code{y} determine
    bubble locations.}
  \item{data}{where formula terms are stored, e.g. data frame or list.}
  \item{subset}{a logical vector specifying which data to plot.}
  \item{na.action}{how \code{NA} values are handled.}
}
\details{
  The \code{std} standardization sets \code{z = abs(z) / mean(abs(z))}.

  The \code{pow = 0.5} (square root) is a good default, where a \code{z}
  value of 2 has twice the area of 1. See example #2 below for an
  exception, where the \code{z} value is tree circumference and
  therefore proportional to the tree diameter.

  The \code{pch}, \code{col}, and \code{bg} arguments can be be vectors
  of length 2, where positive \code{z} values are drawn with
  \code{pch[1]}, \code{col[1]}, \code{bg[1]} and negative \code{z}
  values are drawn with \code{pch[2]}, \code{col[2]}, and \code{bg[2]}.
}
\author{Arni Magnusson.}
\seealso{
  \code{\link{points}} is the underlying function used to draw the
  bubbles.

  \code{\link{symbols}} can also draw bubbles, but does not handle
  negative \code{z} values or have convenience features such as
  \code{pow} and \code{rev}.

  \code{\link{balloonplot}} provides an alternative interface and visual
  style based on tables instead of scatterplots.
}
\examples{
catch.t <- xtabs(Catch~Year+Age, catch.d)              # example table
catch.m <- as.matrix(as.data.frame(unclass(catch.t)))  # example matrix

# 1  Formula
bubbleplot(Catch~Age+Year, data=catch.d)
# Use rev=TRUE to get same layout as crosstab matrix:
print(catch.m)
bubbleplot(Catch~Age+Year, data=catch.d, rev=TRUE, las=1)

# 2  Data frame
bubbleplot(catch.d)
bubbleplot(Orange)
# Visualize tree transverse section at breast height
bubbleplot(Orange, pow=1, cex=2, pch=21,
           col="darkred", bg="peru", lwd=1.5)

# 3  Matrix or table
bubbleplot(catch.m)
bubbleplot(catch.t)

# 4  Positive and negative values
bubbleplot(catch.r)
bubbleplot(Resid~Age+Year, catch.r, subset=Age \%in\% 4:9,
           rev=TRUE, xlim=c(3.5,9.5), cex=1.3)
# Residuals from orange tree model
library(nlme)
fm <- nlme(circumference~phi1/(1+exp(-(age-phi2)/phi3)),
           fixed=phi1+phi2+phi3~1, random=phi1~1|Tree,
           data=Orange, start=c(phi1=200,phi2=800,phi3=400))
bubbleplot(residuals(fm)~Tree+age, Orange)
bubbleplot(residuals(fm)~Tree+age, Orange, cex=2.5, pch=16,
           col=c("dodgerblue","orange"))

# 5  Richter magnitude, amplitude, and energy release
bubbleplot(mag~long+lat, quakes, pch=1)
bubbleplot(10^mag~long+lat, quakes, cex=1.2, col=gray(0, 0.3))
bubbleplot(sqrt(1000)^mag~long+lat, quakes, cex=1.2, col=gray(0, 0.3))
bubbleplot(sqrt(1000)^mag~long+lat, quakes, cex=1.2, col="#FF00004D")
}

\name{bandplot}
\alias{bandplot}
\alias{bandplot.formula}
\alias{bandplot.default}
\title{Plot x-y Points with Locally Smoothed Mean and Standard Deviation}
\description{
  Plot x-y points with curves for locally smoothed mean and standard deviation.
}
\usage{
bandplot(x,...)
\method{bandplot}{formula}(x, data, subset, na.action, ...,
           xlab=NULL, ylab=NULL, add = FALSE, sd = c(-2:2),
           sd.col=c("magenta", "blue", "red", "blue", "magenta"),
           sd.lwd=c(2, 2, 3, 2, 2),  sd.lty=c(2, 1, 1, 1, 2),
           method = "frac", width = 1/5, n=50)
\method{bandplot}{default}(x, y, ..., add = FALSE, sd = c(-2:2),
           sd.col=c("magenta", "blue", "red", "blue", "magenta"),
           sd.lwd=c(2, 2, 3, 2, 2),  sd.lty=c(2, 1, 1, 1, 2),
           method = "frac", width = 1/5, n=50)
}
\arguments{
  \item{x}{either formula providing a single dependent variable (y) and
    an single independent variable (x) to use as coordinates in the
    scatter plot  or a numeric vector of x locations}
  \item{y}{numeric vector of y locations}
  \item{data}{an optional data.frame, list, or environment contianing
    the variables used in the model (and in \code{subset}).  If not found in
    data, the variables are taken from environment(formula),
    typically the environment from which lm is called.}
  \item{subset}{an optional vector specifying a subset of observations to be
    used in the fitting process.}
  \item{na.action}{a function which indicates what should happen when
    the data contain NAs.  The default is set by the na.action
    setting of options, and is na.fail if that is unset.  The
    factory-fresh default is na.omit.  Another possible value is
    NULL, no action.  Value na.exclude can be useful.  }
  \item{\dots}{Additional plotting parameters}
  \item{xlab, ylab}{x and y axis labels}
  \item{add}{ Boolean indicating whether the local mean and standard
    deviation lines should be added to an existing plot.  Defaults to
    FALSE.}
  \item{sd}{Vector of multiples of the standard devation that should be
    plotted.  \code{0} gives the mean, \code{-1} gives the mean minus
    one standard deviation, etc.  Defaults to -2:2.}
  \item{sd.col,sd.lwd,sd.lty}{Color, line width, and line type of each plotted line.}
  \item{method, width, n}{ Parameters controlling the smoothing. See the
    help page for \code{\link{wapply}} for details.}
}
\details{
  \code{bandplot} was created to look for changes in the mean or
  variance of scatter plots, particularly plots of regression residuals.

  The local mean and standard deviation are calculated by calling
  'wapply'.  By default, bandplot asks wapply to smooth using intervals
  that include the nearest 1/5 of the data.  See the documentation of
  that function for details on the algorithm.
}
\value{
  Invisibly returns a list containing the x,y points plotted for each line.
}
\author{ Gregory R. Warnes \email{greg@warnes.net}
}
\seealso{ \code{\link{wapply}}, \code{\link{lowess}}}
\examples{

# fixed mean, changing variance
x <- 1:1000
y <- rnorm(1000, mean=1, sd=1 + x/1000 )
bandplot(x,y)
bandplot(y~x)

# fixed varance, changing mean
x <- 1:1000
y <- rnorm(1000, mean=x/1000, sd=1)
bandplot(x,y)

#
# changing mean and variance
#
x <- abs(rnorm(500))
y <- rnorm(500, mean=2*x, sd=2+2*x)

# the changing mean and dispersion are hard to see whith the points alone:
plot(x,y )

# regression picks up the mean trend, but not the change in variance
reg <- lm(y~x)
summary(reg)
abline(reg=reg, col="blue", lwd=2)

# using bandplot on the original data helps to show the mean and
# variance trend
bandplot(y ~ x)

# using bandplot on the residuals helps to see that regression removes
# the mean trend but leaves the trend in variability
bandplot(predict(reg),resid(reg))

}
\keyword{ dplot }

\name{lmplot2}
\alias{lmplot2}
\title{
  Plots to assess the goodness of fit for the linear model objects
}
\description{
   Plots to assess the goodness of fit for the linear model objects
 }
 \usage{
   lmplot2(
            x,
            which = 1:5,
            caption = c("Residuals vs Fitted", "Normal Q-Q plot",
              "Scale-Location plot", "Cook's distance plot"),
            panel = panel.smooth,
            sub.caption = deparse(x$call),
            main = "",
            ask = interactive() && nb.fig < length(which)
            && .Device != "postscript",
            ...,
            id.n = 3,
            labels.id = names(residuals(x)),
            cex.id = 0.75,
            band=TRUE,
            rug=TRUE,
            width=1/10,
            max.n=5000
            )
}
\arguments{
  \item{x}{lm object}
  \item{which}{Numerical values between 1 and 5, indicating which plots
    to be shown. The codes are:
    \describe{
      \item{1}{Fitted vs residuals}
      \item{2}{Normal Q-Q}
      \item{3}{Scale-Location}
      \item{4}{Cook's distance}
      \item{5}{Residuals vs. predictor}
      }
  }
  \item{caption}{ Caption for each type of plot}
  \item{panel}{ function to draw on the existing plot}
  \item{sub.caption}{ SubCaption for the plots }
  \item{main}{Main title of the plot}
  \item{ask}{whether interactive graphics}
  \item{\dots}{ parameters passed to \code{lmplot2}. }
  \item{id.n}{ integer value, less than or equal to residuals of lm object }
  \item{labels.id}{Names of the residuals of the lm object}
  \item{cex.id}{Parameter to control the height of text stringsx}
  \item{band}{logical vector indicating whether bandplot should also be plotted }
  \item{rug}{logical vector indicating whether rug should be added to
    the existing plot }
  \item{width}{Fraction of the data to use for plot smooths}
  \item{max.n}{Maximum number of points to display in plots}
}
\note{
  This function replaces \code{plot.lm2}, which has been deprecated
  to avoid potential problems with S3 method dispatching.
}
\author{Gregory R. Warnes \email{greg@warnes.net} and Nitin
  Jain \email{nitin.jain@pfizer.com}}
\seealso{
  \code{\link[stats]{plot.lm}}
  }
\examples{
ctl <- rnorm(100, 4)
trt <- rnorm(100, 4.5)
group <- gl(2,100,200, labels=c("Ctl","Trt"))
weight <- c(ctl, trt)
wt.err <- rnorm(length(weight), mean=weight, sd=1/2)
x <- lm(weight ~ group + wt.err)

lmplot2(x)

lmplot2(x, which=1,   width=1/3)
lmplot2(x, which=1:3, width=1/3)
}
\keyword{hplot}

% $Id: wapply.Rd 1441 2010-06-11 03:11:54Z warnes $
%
% $Log$
% Revision 1.10  2005/12/01 16:46:52  nj7w
% Updated Greg's email address
%
% Revision 1.9  2005/06/09 14:20:28  nj7w
% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%
% Revision 1.1.1.1  2005/05/25 22:15:31  nj7w
% Initial submission as an individual package
%
% Revision 1.8  2003/12/03 02:46:51  warnes
% - match function argument defaults with 'usage'
%
% Revision 1.7  2003/01/20 17:13:06  warnes
%
% - Updated wapply.R to allow specification of evaluation points when
%   method is 'width' or 'range' using the 'pts' argument.
% - Updated wapply.Rd to add 'pts' argument
% - Fixed typos, spelling errors, gramatical errors and lack of clarity
%   in wapply.Rd help text.
%
% Revision 1.6  2002/04/09 00:51:33  warneg
%
% Checkin for version 0.5.3
%
% Revision 1.5  2001/12/07 23:34:35  warneg
%
% Fixed an problem where \code was incorrectly delimited by parens
% rather than curly brackets.
%
% Revision 1.4  2001/12/05 19:36:30  warneg
%  - Clarified how the width of the window is computed when method="nobs".
%
% Revision 1.3  2001/10/16 23:15:01  warneg
%
% Fixed unbalanced brace.
%
% Revision 1.2  2001/09/01 00:01:55  warneg
% Release 0.3.0
%
% Revision 1.1  2001/08/25 05:45:10  warneg
% Initial Checkin
%
%
\name{wapply}
\alias{wapply}
\title{Compute the Value of a Function Over a Local Region Of An X-Y Plot}

\description{
  This function applies the specified function to the sets of y values
  that are defined by overlapping "windows" in the x-dimension. For
  example, setting \code{fun=mean} returns local means, while setting
  \code{fun=function(x) sqrt(var(x))} returns local estimates of
  the standard deviation.
}

\usage{
wapply(x, y, fun=mean, method="range", width, n=50, drop.na=TRUE,
       pts, ...)
}
\arguments{
  \item{x}{ vector of x values for (x,y) pairs }
  \item{y}{ vector of y values for (x,y) pairs }
  \item{fun}{ function to be applied }
  \item{method}{ method of defining an x-neighborhood.  One of
    "width","nobs","range", or "fraction".  See details.}
  \item{width}{ width of an x-neighborhood. See details. }
  \item{n}{ Number of equally spaced points at which to compute local
    estimates. See details.}
  \item{drop.na}{ should points which result in missing values \code{NA}
    be omitted from the return value. Defaults to true. }
  \item{pts}{ \code{x} locations at which to compute the local mean when
    using the "width" or "range" methods. Ignored otherwise.}
  \item{\dots}{ arguments to be passed to \code{fun} }
}
\details{
  Two basic techniques are available for determining what points fall
  within the same x-neighborhood.  The first technique uses a window with
  a fixed width in the x-dimension and is is selected by
  setting \code{method="width"} or \code{method="range"}.  For
  \code{method="width"} the \code{width} argument is an absolute
  distance in the x-dimension.  For \code{method="range"}, the width is
  expressed as a fraction of the x-range.  In both cases, \code{pts}
  specifies the points at which evaluation of \code{fun} occurs.  When
  \code{pts} is omitted, \code{n} x values equally spaced along the x
  range are used.

  The second technique uses windows containing  k neighboring points.  The
  (x,y) pairs are sorted by the x-values and the nearest k/2 points with
  higher x values and the k/2 nearest points with lower x values are
  included in the window.  When \code{method="nobs"}, k equals
  \code{width} (actually 2*floor(\code{width}/2) ).  When
  \code{method="fraction"}, \code{width} specifies what fraction of the
  total number of points should be included. The actual number of points
  included in each window will be floor(n*frac/2)*2.  Regardless of the
  value of \code{pts}, the function \code{fun} will be evaluated at all
  x locations.
}
\value{
  Returns a list with components 
  \item{x }{x location'}
  \item{y }{Result of applying \code{fun} to the window about each x location}
}
\author{ Gregory R. Warnes \email{greg@warnes.net}
  }
\examples{

#show local mean and inner 2-sd interval to help diagnose changing mean
#or variance structure
x <- 1:1000
y <- rnorm(1000, mean=1, sd=1 + x/1000 )

plot(x,y)
lines(wapply(x,y,mean),col="red")

CL <- function(x,sd) mean(x)+sd*sqrt(var(x))

lines(wapply(x,y,CL,sd= 1),col="blue") 
lines(wapply(x,y,CL,sd=-1),col="blue") 
lines(wapply(x,y,CL,sd= 2),col="green")
lines(wapply(x,y,CL,sd=-2),col="green")

#show local mean and inner 2-sd interval to help diagnose changing mean
#or variance structure
x <- 1:1000
y <- rnorm(1000, mean=x/1000, sd=1)

plot(x,y)
lines(wapply(x,y,mean),col="red")

CL <- function(x,sd) mean(x)+sd*sqrt(var(x))

lines(wapply(x,y,CL,sd= 1,method="fraction",width=1/20),col="blue")
lines(wapply(x,y,CL,sd=-1,method="fraction",width=1/20),col="blue")
lines(wapply(x,y,CL,sd= 2,method="nobs",width=250),col="green")
lines(wapply(x,y,CL,sd=-2,method="nobs",width=250),col="green")


}
\keyword{ dplot }

\name{col2hex}
\alias{col2hex}
\title{Convert color names to hex RGB strings}
\description{
  Convert color names to hex RGB strings
}
\usage{
col2hex(cname)
}
\arguments{
  \item{cname}{Color name(s)}
}
\value{
  Character vector giving the hex color code translation of the provided
  color names.
}
\author{Gregory R. Warnes}
\seealso{
  \code{\link{col2rgb}}, \code{\link{colors}}, \code{\link{rgb}}
  }
\examples{
col2hex(c("red","yellow","lightgrey"))
}
\keyword{color}

\name{ooplot.default}
\alias{ooplot.default}
\alias{ooplot}
\title{Create an OpenOffice style plot}
\description{
  An extension of barplot2. Creates bar- and line-plots mimicking the
  style of OpenOffice plots.  This utility can plot the values next to
  each point or bar as well as confidence intervals.
}
\usage{
ooplot(data, ...)
\method{ooplot}{default}(data, width=1, space=NULL, names.arg=NULL, 
                           legend.text=NULL, horiz=FALSE, 
                           density=NULL, angle=45, kmg="fpnumkMGTP", 
                           kmglim=TRUE, 
                           type=c("xyplot", "linear", "barplot", "stackbar"), 
                           col=heat.colors(NC), prcol=NULL, 
                           border=par("fg"), main=NULL, sub=NULL, 
                           xlab=NULL, ylab=NULL, xlim=NULL, ylim=NULL, 
                           xpd=TRUE, log="", axes=TRUE, 
                           axisnames=TRUE, prval=TRUE, lm=FALSE,
                           cex.axis=par("cex.axis"), 
                           cex.names=par("cex.axis"),
                           cex.values=par("cex"),inside=TRUE, 
                           plot=TRUE, axis.lty=0, plot.ci=FALSE, 
                           ci.l=NULL, ci.u=NULL, ci.color="black", 
                           ci.lty="solid", ci.lwd=1, plot.grid=FALSE, 
                           grid.inc=NULL, grid.lty="dotted", 
                           grid.lwd=1, grid.col="black", add=FALSE, 
                           by.row=FALSE, ...)
}
\arguments{
  \item{data}{a matrix of values describing the values that make up the
    plot. The first column of \code{data} is taken as the axis against
    which all the other values are plotted. The first column of
    \code{data} may not be sparse.}
  \item{width}{optional vector of barwidths. Re-cycled to the number of
    bars drawn. A single value will have no visible effect. }
  \item{space}{ the amount of space left before each bar. May be given
    as a single number or one number per bar. If \code{type} is
    \code{stackbar}, \code{space} may be specified by two numbers, where
  the first is the space between bars in the same group, and the second
  the space between groups. Defaults to \code{c(0,1)} if \code{type} is
  a \code{stackbar}, and to 0.2 otherwise. }
  \item{names.arg}{a vector of names to be plotted below each bar or
    group of bars.  If this argument is omitted, then the names are
    taken from the  row names of \code{data}. }
  \item{legend.text}{a vector of text used to construct a legend for the
    plot, or a logical indicating whether a legend should be included;
    if \code{legend.text} is true, the row names of \code{data} will
    be used as labels if they are non-null.}
  \item{horiz}{a logical value.  If \code{FALSE}, the bars are drawn
    vertically with the first bar to the left.  If \code{TRUE}, the
    bars are drawn horizontally with the first at the bottom.}
  \item{density}{a vector giving the the density of shading lines, in
    lines per inch, for the bars or bar components.
    The default value of \code{NULL} means that no shading lines
    are drawn. Non-positive values of \code{density} also inhibit the
    drawing of shading lines.}
  \item{angle}{the slope of shading lines, given as an angle in
    degrees (counter-clockwise), for the bars or bar components.}
  \item{kmg}{the set of SI units to convert, defaults to
    "fpnumkMGTP". See below for details. }
  \item{kmglim}{logical. If \code{FALSE} the conversion to SI units is not
    performed. Default is \code{TRUE}.
  }
  \item{type}{a string indicating the preferred format of the plot,
    choices are:
    xyplot : plot where y is plotted against the x-value.
    linear : plot where y values are plotted against equidistant x-values.
    barplot : plot where y values are represented as bars against
    equidistant x-values.
    stackplot : plot where y values are stacked for identical x-values
    and bars are equidistant.
  }
  \item{col}{a vector of colors for the bars or bar components.}
  \item{prcol}{the color to be used for the plot region.}
  \item{border}{the color to be used for the border of the bars.}
  \item{main, sub}{overall and sub titles for the plot.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{xlim}{limits for the x axis.}
  \item{ylim}{limits for the y axis.}
 \item{xpd}{logical. Should bars be allowed to go outside region?}
  \item{log}{a character string which contains `"x"' if the x axis is
    to be logarithmic, `"y"' if the y axis is to be logarithmic and
    `"xy"' or `"yx"' if both axes are to be logarithmic.}
  \item{axes}{logical.  If \code{TRUE}, a vertical (or horizontal, if
    \code{horiz} is true) axis is drawn.}
  \item{axisnames}{logical.  If \code{TRUE}, and if there are
    \code{names.arg} (see above), the
    other axis is drawn (with \code{lty=0}) and labeled.}
  \item{prval}{logical. If \code{TRUE}, then values are plotted above
    all points and bars.
    }
  \item{lm}{logical. If \code{TRUE}, the linear fit is plotted.
    }
  \item{cex.axis, cex.names, cex.values}{character scaling factor for
    numeric axis labels,
    names, and displayed values, respectively.}
  \item{inside}{logical.  If \code{TRUE}, the lines which divide
    adjacent (non-stacked!) bars will be drawn.  Only applies when
    \code{space = 0} (which it partly is when \code{beside = TRUE}).}
  \item{plot}{logical.  If \code{FALSE}, nothing is plotted.}
  \item{axis.lty}{the graphics parameter \code{lty} applied to the axis
    and tick marks of the categorical (default horzontal) axis.  Note
    that by default the axis is suppressed.}
  \item{plot.ci}{logical. If \code{TRUE}, confidence intervals are plotted
   over the bars. Note that if a stacked bar plot is generated, confidence
   intervals will not be plotted even if \code{plot.ci = TRUE}}
  \item{ci.l,ci.u}{The confidence intervals (ci.l = lower bound, ci.u =
   upper bound) to be plotted if \code{plot.ci} = \code{TRUE}. Values must have
   the same dim structure as \code{height}.}
  \item{ci.color}{the color for the confidence interval line segments}
  \item{ci.lty}{the line type for the confidence interval line segments}
  \item{ci.lwd}{the line width for the confidence interval line segments}
  \item{plot.grid}{if \code{TRUE} a lined grid will be plotted behind the bars}
  \item{grid.inc}{the number of grid increments to be plotted}
  \item{grid.lty}{the line type for the grid}
  \item{grid.lwd}{the line width for the grid}
  \item{grid.col}{the line color for the grid}
  \item{add}{logical, if \code{TRUE} add barplot to current plot.}
  \item{by.row}{Logical value.  If \code{TRUE} the data matrix is
    organized with variables along rows rather than down colums.}
  \item{\dots}{further graphical parameters (\code{\link{par}}) are
    passed to \code{\link{plot.window}()}, \code{\link{title}()} and
    \code{\link{axis}}.}
}
\details{
  Plot units are automatically scaled to SI units based on the
  maximum value present, according to the set of units specified by
  characters in the \code{kmg} parameter.  These letters are interpreted
  as
  \describe{
    \item{P}{peta = 1E15}
    \item{T}{tera = 1E12}
    \item{G}{giga = 1E09}
    \item{M}{mega = 1E06}
    \item{k}{kilo = 1E03}
    \item{m}{milli= 1E-03}
    \item{u}{micro= 1E-06}
    \item{n}{nano = 1E-09}
    \item{p}{pico = 1E-12}
    \item{f}{femto= 1E-15}
  }
  with the default being "fpnumkMGTP" (all of these units).  For
  example, if the largest value plotted is 1243000, it would be
  presented as 1.234M.
  
}
\value{
   A numeric vector (or matrix, when \code{beside = TRUE}), say
  \code{mp}, giving the coordinates of \emph{all} the bar midpoints
  drawn, useful for adding to the graph.

  If \code{beside} is true, use \code{colMeans(mp)} for the
  midpoints of each \emph{group} of bars, see example.
}
\author{Lodewijk Bonebakker \email{bonebakker@comcast.net} with modifications
  by Gregory R. Warnes \email{greg@warnes.net}.
  Based on barplot2().
  }
\seealso{ \code{\link{plot}}, \code{\link{boxplot}} }
\examples{
     data(VADeaths, package = "datasets")

     VADeaths <- cbind( Age=c(50,55,60,65,70), VADeaths)

     mp <- ooplot(VADeaths) # default
     mp <- ooplot(VADeaths, type="xyplot")  # same as default
     mp <- ooplot(VADeaths, type="linear")  # linear scale
     mp <- ooplot(VADeaths, type="linear", log="y") # log scale on y axis
     mp <- ooplot(VADeaths, type="barplot") # barplot
     mp <- ooplot(VADeaths, type="stackbar") # stacked


     tot <- colMeans(VADeaths[,-1])
     ooplot(VADeaths, 
             col = c("lightblue", "mistyrose", "lightcyan", "lavender"),
             legend = colnames(VADeaths)[-1], ylim = c(0, 100),
             type="barplot", cex.values=0.75)
     title(main = "Death Rates in Virginia", font.main = 4)


     ##
     ## Capability demo
     ##
     ## examples for the ooplot routine
     ##
     ## create some test data
     test1 <- data.frame(x=c(0,1,2,3,4), lin=c(0,1,2,3,4))
     test2 <- data.frame(x=c(0,1,2,3,4), par=c(0,1,4,9,16))
     test3 <- data.frame(x=c(-2,-1,0,1,2),y2=c(4,1,0,1,4))
     ## single line test example
     test1f <- test1
     ## two column example
     test2f <- merge(test1,test2,by.x="x",all=TRUE,sort=TRUE)
     ## three column example
     test3f <- merge(test2f,test3,by.x="x",all=TRUE,sort=TRUE)
     ## subset, single row, example
     test5r <- test3f[5,]  
     
     ##
     ## xyplot, linear, barplot, stackbar
     dev.off()
     mat <- matrix(c(1:16),4,4,byrow=TRUE)
     layout(mat)
     
     ooplot(test1f,type="barplot",col=c("red"))
     title(main="barplot")
     ooplot(test2f,type="barplot",col=c("red","blue"))
     ooplot(test3f,type="barplot",col=c("red","blue","green"))
     ooplot(test5r,type="barplot",col=c("red","blue","green"))
     
     ooplot(test1f,type="xyplot",col=c("red"))
     title(main="xyplot")
     ooplot(test2f,type="xyplot",col=c("red","blue"))
     ooplot(test3f,type="xyplot",col=c("red","blue","green"))
     ooplot(test5r,type="xyplot",col=c("red","blue","green"))
     
     ooplot(test1f,type="linear",col=c("red"))
     title(main="linear")
     ooplot(test2f,type="linear",col=c("red","blue"))
     ooplot(test3f,type="linear",col=c("red","blue","green"))
     ooplot(test5r,type="linear",col=c("red","blue","green"))
     
     ooplot(test1f,type="stackbar",col=c("red"))
     title(main="stackbar")
     ooplot(test2f,type="stackbar",col=c("red","blue"))
     ooplot(test3f,type="stackbar",col=c("red","blue","green"))
     ooplot(test5r,type="stackbar",col=c("red","blue","green"))

     # restore default layout (1 plot/page)
     layout(1)
}
\keyword{hplot}

\name{colorpanel}
\alias{colorpanel}
\alias{redgreen}
\alias{greenred}
\alias{bluered}
\alias{redblue}
\title{Generate a smoothly varying set of colors}
\description{
  \code{colorpanel} generate a set of colors that varies
  smoothly. \code{redgreen}, \code{greenred}, \code{bluered}, and 
  \code{redblue} generate red-black-green, green-black-red,
  red-white-blue, and blue-white-red colorbars, respectively.
  colors 
}
\usage{
colorpanel(n, low, mid, high)
redgreen(n)
greenred(n)
bluered(n)
redblue(n)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{Desired number of color elements in the panel.}
  \item{low, mid, high}{Colors to use for the Lowest, middle,
                        and highest values. \code{mid} may be ommited.}
}
\details{
  The values for \code{low, mid, high} can be given as
  color names (\code{"red"}), plot color index (\code{2}), and
  HTML-style RGB, (\code{"#FF0000"}).

  If \code{mid} is supplied, then the returned color panel will consist
  of \code{n - floor(n/2)} HTML-style RGB elements which vary smoothly
  between \code{low} and \code{mid}, then between \code{mid} and
  \code{high}. Note that if \code{n} is even, the color \code{mid} will
  occur twice at the center of the sequence.

  If \code{mid} is omitted, the color panel will vary
  smoothly beween \code{low} and \code{high}. 
}
\value{
  Vector of HTML-style RGB colors.
}
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{\link{colors} } }
\examples{

showpanel <- function(col)
{
  image(z=matrix(1:100, ncol=1), col=col, xaxt="n", yaxt="n" )
}

par(mfrow=c(3,3))

# two colors only:
showpanel(colorpanel(8,low="red",high="green"))

# three colors
showpanel(colorpanel(8,"red","black","green"))
# note the duplicatation of black at the center, using an odd
# number of elements resolves this:
showpanel(colorpanel(9,"red","black","green"))

showpanel(greenred(64))
showpanel(redgreen(64))
showpanel(bluered(64))
showpanel(redblue(64))

}
\keyword{color}

\name{boxplot2}
\alias{boxplot2}
\title{Produce a Boxplot Annotated with the Number of Observations}
\description{
  This funcntion uses \code{boxplot} to produce a boxplot which is then 
  annotated with the number of observations in each group.
}
\usage{
boxplot2(..., top=FALSE, shrink=1, textcolor=NULL)
}
\arguments{
  \item{\dots}{ parameters passed to \code{boxplot}. }
  \item{top}{ logical indicating whether the number of observations
    should be added to the top or the bottom of the plotting
    region. Defaults to \code{FALSE}. }
  \item{shrink}{ value to shrink character size (cex) when annotating.}
  \item{textcolor}{ text color. }
}
\author{ Gregory R. Warnes \email{greg@warnes.net}}
\note{
  This function replaces \code{boxplot.n}, which has been deprecated
  avoid potential problems with S3 method dispatching.
}
\seealso{
  \code{\link{boxplot}},
  \code{\link{text}}
}

\examples{
data(state)

# n's at bottom
boxplot2( state.area ~ state.region)

# n's at top
boxplot2( state.area ~ state.region, top=TRUE)

# small red text
boxplot2( state.area ~ state.region, shrink=0.8, textcolor="red") 
}

\keyword{ hplot }

\name{overplot}
\alias{overplot}
\alias{panel.overplot}
\title{Plot multiple variables on the same region, with appropriate axes}
\description{
  \code{overplot} graphs a set of variables defined on the same x-range
  but which have varying y-ranges on the same plotting area.  For each
  set of y-values it uses a different color and line-type and and draws
  a correspondingly colored and line-typed axis.  \code{panel.overplot}
  is used by \code{overplot} to draw the individual graphs.
}
\usage{
overplot(formula, data = parent.frame(), same.scale = FALSE, xlab, ylab,
         xlim, ylim, min.y, max.y, log = "", panel = "panel.overplot",
         subset, plot = TRUE, groups, main, f = 2/3, ...)
}
\arguments{
  \item{formula}{Formula describing the x and y variables.  It should be
    of the form x ~ y|z. The conditioning variable (z) should be a factor.}
  \item{same.scale}{ Logical value indicating whether the plot region
    should have the same range for all plots.  Defaults to \code{FALSE}.}
  \item{xlab, ylab, xlim, ylim, main}{ Standard plotting parameters.  See
    \code{\link{plot}} for details}
  \item{min.y, max.y}{Scalar or vector values used to specify the y
    plotting limits for individual plots.  If a single scalar value is
    provided, it will be used for all plots. These parameters can be
    used specify one end of the individual plot ranges, while allowing
    the other end to vary with the data.  EG, to force 0 to always be
    within the plot region.}
  \item{log}{ character string '', 'x', 'y', or 'xy', indicating which axes
    should be plotted on a log scale. Defaults to '' (neither).}
  \item{panel}{ a plotting function to be called to draw the individual
    plots.  Defaults to \code{overplot.panel}, which plots the points
    and a \code{lowess} smooth. }
  \item{plot}{Logical value indicating whether to draw the plot.}
  \item{groups}{(optional) character vector giving the names of levels
    of the conditioning variable to plot. Defaults to all levels of the
    conditioning variable.}
  \item{f}{Smoothing parameter for \code{lowess}}
  \item{data, subset, \dots}{parameters passed to \code{model.frame} to
    obtain the data to be plotted from the formula.}
}
\details{
  This function essentially performs

  tmp <- split(data, z)
  
  for(i in levels(z))
  
  plot( x ~ y, data=tmp[[z]] )
  
  except that all of the plots are shown on the same plotting region and
  varying scales for each value of z are handled nicely.
}
\value{
  A copy of the data split by the conditioning variable.
}
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{
  \code{\link{interaction.plot}},
  \code{\link{coplot}} for alternative visualizations of 3-way data.}
\examples{

# Example teratogenicity rtPCR data
data(rtPCR)

# same scale
overplot( RQ ~ Conc..ug.ml. | Test.Substance,
         data=rtPCR,
         subset=Detector=="ProbeType 1" & Conc..ug.ml. > 0,
         same.scale=TRUE,
         log="xy",
         f=3/4,
         main="Detector=ProbeType 1",
         xlab="Concentration (ug/ml)",
         ylab="Relative Gene Quantification"
         )

# different scales, but force lower limit to 0.01 
overplot( RQ ~ Conc..ug.ml. | Test.Substance,
         data=rtPCR,
         subset=Detector=="ProbeType 8" & Conc..ug.ml. > 0,
         log="xy",
         f=3/4,
         main="Detector=ProbeType 8",
         xlab="Concentration (ug/ml)",
         ylab="Relative Gene Quantification",
         min.y=0.01
         )

}
\keyword{hplot}

\name{rich.colors}
\alias{rich.colors}
\title{Rich Color Palettes}
\description{
  Create a vector of \code{n} colors that are perceptually equidistant
  and in an order that is easy to interpret.
}
\usage{
rich.colors(n, palette="temperature", alpha=1.0, rgb=FALSE, plot=FALSE)
}
\arguments{
  \item{n}{number of colors to generate.}
  \item{palette}{palette to use: \code{"temperature"} contains
    blue-green-yellow-red, and \code{"blues"} contains
    black-blue-white.}
  \item{alpha}{alpha transparency, from 0 (fully transparent)  to 1
    (opaque).} 
  \item{rgb}{if \code{TRUE} then a matrix of RGBA values is included as an
          attribute.}
  \item{plot}{whether to plot a descriptive color diagram.}
}
\value{A character vector of color codes.}
\author{Arni Magnusson.}
\seealso{
  \code{\link{rgb}}, \code{\link{rainbow}}, \code{\link{heat.colors}}.
}
\examples{
m <- abs(matrix(1:120+rnorm(120), nrow=15, ncol=8))
opar <- par(bg="gray", mfrow=c(1,2))
matplot(m, type="l", lty=1, lwd=3, col=rich.colors(8))
matplot(m, type="l", lty=1, lwd=3, col=rich.colors(8,"blues"))
par(opar)

barplot(rep(1,100), col=rich.colors(100), space=0, border=0, axes=FALSE)
barplot(rep(1,20), col=rich.colors(40)[11:30]) # choose subset

plot(m, rev(m), ylim=c(120,0), pch=16, cex=2,
     col=rich.colors(200,"blues",alpha=0.6)[1:120]) # semitransparent

rich.colors(100, plot=TRUE)  # describe rgb recipe

par(mfrow=c(2,2))
barplot(m, col=heat.colors(15), main="\nheat.colors")
barplot(m, col=1:15, main="\ndefault palette")
barplot(m, col=rich.colors(15), main="\nrich.colors")
barplot(m, col=rainbow(15), main="\nrainbow")
par(opar)
}
% Graphics
\keyword{color}

\name{adjust_hsv}
\alias{adjust_hsv}
\title{Adjust Color in HSV Space}
\description{Adjust hue, saturation, value, and/or opacity.}
\usage{
adjust_hsv(col, h=NULL, s=NULL, v=NULL, alpha=NULL)
}
\arguments{
  \item{col}{a color or vector of colors.}
  \item{h}{the desired hue.}
  \item{s}{the desired saturation.}
  \item{v}{the desired value.}
  \item{alpha}{the desired opacity.}
}
\details{
  Colors can be specified as a color name, a hexadecimal string, or an integer.

  Hue, saturation, value, and opacity are specified as values from 0 to 1, or
  \code{NULL} to leave unchanged.
}
\value{Adjusted colors in hexadecimal string format.}
\author{Arni Magnusson.}
\seealso{
  \code{\link[grDevices]{adjustcolor}} adjusts colors in RGB space.

  \code{\link{col2rgb}}, \code{\link{rgb2hsv}}, and \code{\link{hsv}} are the
  underlying functions used to convert and adjust the colors.
}
\examples{
col <- "#123456"
col2 <- adjust_hsv(col, h=0.1)
col3 <- adjust_hsv(col, s=0.1)
col4 <- adjust_hsv(col, v=0.7)

barplot(rep(1, 4), col=c(col, col2, col3, col4),
        names=c("original", "h", "s", "v"))
}

\name{qqnorm.aov}
\alias{qqnorm.aov}

\title{ Makes a half or full normal plot for the effects from an aov model }
\description{
  Makes a half or full normal plot for the effects from a model
  inheriting from
  class \code{aov}. One can interactively label the points in the plot.
}
\usage{
\method{qqnorm}{aov}(y, full=FALSE, label=FALSE, omit=NULL,
           xlab=paste(if (full) "" else "Half", " Normal plot"), 
           ylab="Effects", ...)
}

\arguments{
  \item{y}{A model object inheriting from \code{aov}}
   \item{full}{Full or half normal plot (half is default) }
   \item{label}{If \code{TRUE}, function allows interactive
     labelling of points in plot, using the mouse }
   \item{omit}{Numeric or character vector of effects to omit,
     the intercept is always omitted}
   \item{xlab}{Horizontal axix label }
   \item{ylab}{Vertical axis label }
   \item{\dots}{Further arguments to be given to the plot function }
}
\details{
   Produces a (half) normal plot of the effects from an AOV model. The
   idea behind the plot is that most effects will be small or null, and
   this effects can be used as a basis for estimation of the
   experimental variance. This small effects will show up in the plot as
   a straight line, other effects can be judged against this as a
   background. Heavily used by Box, Hunter \& Hunter, which attributes
   the idea to Daniel.

   This is a simpler implementation than the one in S-Plus.
}
\value{
  If \code{label=TRUE}, the vector of points identified, else nothing of
   interest.
}

\references{
   Box, Hunter and Hunter: Statistics for Experimenters. An Introduction
   to Design, Data Analysis and Model Building. Wiley. \cr
   Daniel, C (1976): Applications of Statistics to Industrial
   Experimentation. Wiley. \cr
   Daniel, C (1959): Use of half-normal plot in interpreting factorial
   two-level experiments. \emph{Technometrics.}\bold{1}, 149.
}

\author{Kjetil Halvorsen \email{kjetil@entelnet.bo}}

\examples{
library(MASS)
data(npk)
npk.aov <- aov(yield ~ block + N*P*K, npk)
qqnorm(npk.aov)

## interactive labeling of points.  Click mouse on points to show label.
if (dev.interactive()) qqnorm(npk.aov, omit=2:6, label=TRUE)
}
\keyword{ hplot }%
\keyword{ design }

\name{sinkplot}
\alias{sinkplot}
\title{Send textual R output to a graphics device}
\description{
  Divert R's standard text output to a graphics device.
}
\usage{
sinkplot(operation = c("start", "plot", "cancel"), ...)
}
\arguments{
  \item{operation}{See below}
  \item{\dots}{Plot arguments. (Ignored unless \code{operation}="plot").}
}
\details{
  This function allows the printed output of R commands to be captured
  and displayed on a graphics device.

  The capture process is started by calling
  \code{sinkplot("start")}. Now R commands can be executed and all
  printed output (except errors) will be captured.  When the desired
  text has been captured \code{sinkplot("plot")} can be called to
  actually display the output.  \code{sinkplot("cancel")} can be used to
  abort the output capture without plotting.

  The current implementation does not allow \code{sinkplot} to be nested.
}
\value{
  Invisibly returns a character vector containing one element for each
  line of the captured output.
}

\references{Functionality requested by Kevin Wright
   \email{kwright@eskimo.com} in the R-devel newlist posting
   \url{https://stat.ethz.ch/pipermail/r-devel/2004-January/028483.html}.
 }
\author{ Gregory R. Warnes \email{greg@warnes.net}  }
\seealso{ \code{\link[utils]{capture.output}}, \code{\link{textplot}} }
\examples{
   set.seed(12456)
   x <- factor(sample( LETTERS[1:5], 50, replace=TRUE))
   y <- rnorm(50, mean=as.numeric(x), sd=1)

   ## construct a figure showing a box plot of the data, followed by an
   ## analysis of variance table for the data
   layout(cbind(1:2), heights=c(2,1))

   boxplot(y~x, col="darkgreen")

   sinkplot()
   anova(lm(y~x))
   sinkplot("plot",col="darkgreen")
}
\keyword{hplot}

\name{plotmeans}
\alias{plotmeans}
\title{Plot Group Means and Confidence Intervals}
\description{Plot group means and confidence intervals.}
\usage{
plotmeans(formula, data=NULL, subset, na.action,
          bars=TRUE, p=0.95, minsd=0, minbar, maxbar,
          xlab=names(mf)[2], ylab=names(mf)[1], mean.labels=FALSE,
          ci.label=FALSE, n.label=TRUE, text.n.label="n=",
          digits=getOption("digits"), col="black", barwidth=1,
          barcol="blue", connect=TRUE, ccol=
          col, legends=names(means), xaxt, use.t=TRUE,
          lwd=par("lwd"), ...)
}
\arguments{
  \item{formula}{symbolic expression specifying the outcome (continuous)
    and grouping variable (factor).  See lm() for details.}

  \item{data}{optional data frame containing the variables in the model.}

  \item{subset}{an optional vector specifying a subset of observations to be
    used in the fitting process.}

  \item{na.action}{a function which indicates what should happen when the data
    contain `NA's.  See lm() for details. }

  \item{bars}{a logical value indicating whether confidence interval
    bars should be plotted. Defaults to TRUE.}

  \item{p}{confidence level for error bars.  Defaults to 0.95.}

  \item{minsd}{minumum permitted value for the standard deviation within
    each factor level.  Any standard deviation estimates smaller than
    \code{minsd} will be replaced with \code{minsd}. Defaults to 0.}

  \item{minbar}{minumum allowed value for bar ends.  If specified,
    values smaller than \code{minbar} will be replaced with
    \code{minbar}. }

  \item{maxbar}{maximum allowed value for bar ends.  If specified,
    values larger than \code{maxbar} will be replaced with
    \code{maxbar}. }

  \item{xlab}{x-axis label.}
  \item{ylab}{y-axis label.}

  \item{mean.labels}{ either a logical value indicating whether the circles
    representing the group means should be replaced with text giving the
    actual mean values or a vector containing labels to use
    instead. Defaults to FALSE.}

  \item{ci.label}{ a logical value indicating whether text giving the
    actual interval end values should be placed at the end of each
    confidence interval bar. Defaults to FALSE.}

  \item{n.label}{ a logical value indicating whether text giving the number of
    observations in each group should should be added to the plot. }

  \item{text.n.label}{Prefix text for labeling observation counts.
    Defaults to "n=". }

  \item{digits}{ number of significant digits to use when displaying
    mean or confidince limit values.}

  \item{col}{ color of cicles marking group means.  Default is "black".}

  \item{barwidth}{ linewidth of interval bars and end marks. Default is
    1.}

  \item{barcol}{ color of interval bars and end marks.  Default is
    "blue".}

  \item{connect}{ either a logical value indicating whether the means of
    each group should be connected by a line, or a list of vectors giving the
    index of bars that should be connected by a line. Defaults to TRUE.}

  \item{ccol}{ color of lines used to connect means. Defaults to the
    same color as "col".}

  \item{legends}{ vector containing strings used to label groups along
    the x axis. Defaults to group names.}

  \item{xaxt}{A character which specifies the axis type. Specifying `"n"'
          causes an axis to be set up, but not plotted.}

  \item{use.t}{ a logical value indicating whether the t distribution
    should be used to compute confidence intervals.  If \code{TRUE}, the
    default, a t distribution will the correct number of degrees of
    freedom for each group be used.  If \code{FALSE}, the a normal
    distribution will be used.}

  \item{lwd}{Width of connecting lines  }

  \item{\dots}{ optional plotting parameters. }

  }

%\details{
%
%
%}


\examples{
  # library(gplots)
  # show comparison with boxplot
  data(state)
  plotmeans(state.area ~ state.region)

  # show some color and mean labels
  plotmeans(state.area ~ state.region,
             mean.labels=TRUE, digits=-3,
             col="red", connect=FALSE)

  # show how to specify which means should be connected
  plotmeans(state.area ~ state.region, connect=list(1:2, 3:4),
            ccol="red", pch=7 )

  # more complicated example showing how to show an interaction
  data(esoph)
  par(las=2,                        # use perpendicular axis labels
      mar=c(10.1,4.1,4.1,2.1),      # create enough space for long x labels
      mgp=c(8,1,0)                  # move x axis legend down to avoid overlap
      )
  plotmeans(ncases/ncontrols ~ interaction(agegp , alcgp, sep ="   "),
            connect=list(1:6,7:12,13:18,19:24),
            barwidth=2,
            col="dark green",
            data=esoph,
            xlab="Age Group and Alcohol Consumption",
            ylab="# Cases / # Controls",
            ylim = c(-.9,1.4),
            main=c("Fraction of Cases for by Age and Alcohol Consumption",
                   "Ile-et-Vilaine Esophageal Cancer Study")
            )
  abline(v=c(6.5, 12.5, 18.5), lty=2)

}

\author{Gregory R. Warnes \email{greg@warnes.net}}

\seealso{\code{\link{plotCI}}, \code{\link{boxplot}}}

\keyword{hplot}

%% Revision 2.1 2005/06/06
%% - Modified default behavior with 0's and NA's in
%%   'height' so that these values are not plotted.
%% - Warning messages added in the case of the above.

%% Revision 2.0 2005/04/27
%% - Added panel.first and panel.last arguments
%% - As per R 2.0.0, the default barplot() method by default uses a
%%   gamma-corrected grey palette (rather than the heat color
%%   palette) for coloring its output when given a matrix.

% $Id: barplot2.Rd 1646 2013-03-24 05:19:04Z warnes $
%% $Log$
%% Revision 1.12  2005/12/07 18:48:54  nj7w
%% Changed the default grey colors to heat colors, if height is matrix
%%
%% Revision 1.11  2005/12/06 22:41:08  nj7w
%% Fixed minor error in documentation of barplot2
%%
%% Revision 1.10  2005/06/09 14:20:28  nj7w
%% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%%
%% Revision 1.3  2005/06/06 21:44:51  nj7w
%% Changes based on Marc Schwartz's suggestions to handel 0's and NA's appropriately.
%%
%% Revision 1.9  2004/06/28 20:16:44  warnes
%% - Default 2-d color set to grey as in (v2.0.0) r-devel
%% - The default (v1.9.1) barplot method now handles vectors and 1-d arrays
%%   (e.g., obtained by table()) the same, and uses grey instead of
%%   heat color palettes in these cases.  (Also fixes PR#6776.)
%% - Updated to reflect the addition of 'offset' argument in base
%% - Fix bug in stacked barplots, which would not be plotted properly
%%
%
% Revision 1.8  2004/06/26
% - Default 2-d color set to grey as in (v2.0.0) r-devel
% - The default (v1.9.1) barplot method now handles vectors and 1-d arrays
%   (e.g., obtained by table()) the same, and uses grey instead of
%   heat color palettes in these cases.  (Also fixes PR#6776.)
% - Updated to reflect the addition of 'offset' argument in base
% - Fix bug in stacked barplots, which would not be plotted properly
%
% Revision 1.7  2003/12/02 16:58:46  warnes
%
% - Remove '%' character from example code to avoid errors.
%
% Revision 1.6  2003/12/01 16:00:08  warnes
%
% - updated to match changes to barplot() in base.
%
% Revision 1.5  2003/11/18 18:59:08  warnes
%
% - Tiny fix to remove R CMD check error.
%
% Revision 1.4  2003/01/30 21:43:05  warnes
%
% - Added argument 'add' to allow for the addition of a barplot to an
%   existing graphic. Default is FALSE
%
% Revision 1.2  2002/09/23 14:27:17  warnes
%
% - Fixed syntax errors in barplot2.Rd and CrossTable.Rd
% - Fixed incorrect translation of 'F' (distribution) to 'FALSE' in glh.test.Rd
%
% Revision 1.1  2002/09/23 13:38:53  warnes
%
% - Added CrossTable() and barplot2() code and docs contributed by Marc Schwartz.
% - Permit combinations() to be used when r>n provided repeat.allowed=TRUE
% - Bumped up version number
%
%

\name{barplot2}
\alias{barplot2}
\alias{barplot2.default}
\title{Enhanced Bar Plots}
\usage{
\method{barplot2}{default}(height, width = 1, space = NULL,
        names.arg = NULL, legend.text = NULL, beside = FALSE,
        horiz = FALSE, density = NULL, angle = 45,
        col = NULL, prcol = NULL, border = par("fg"),
        main = NULL, sub = NULL, xlab = NULL, ylab = NULL,
        xlim = NULL, ylim = NULL, xpd = TRUE, log = "",
        axes = TRUE, axisnames = TRUE,
        cex.axis = par("cex.axis"), cex.names = par("cex.axis"),
        inside = TRUE, plot = TRUE, axis.lty = 0, offset = 0,
        plot.ci = FALSE, ci.l = NULL, ci.u = NULL,
        ci.color = "black", ci.lty = "solid", ci.lwd = 1, ci.width = 0.5,
        plot.grid = FALSE, grid.inc = NULL,
        grid.lty = "dotted", grid.lwd = 1, grid.col = "black",
        add = FALSE, panel.first = NULL, panel.last = NULL, \dots)
}
\arguments{
  \item{height}{either a vector or matrix of values describing the
    bars which make up the plot.  If \code{height} is a vector, the
    plot consists of a sequence of rectangular bars with heights
    given by the values in the vector.  If \code{height} is a matrix
    and \code{beside} is \code{FALSE} then each bar of the plot
    corresponds to a column of \code{height}, with the values in the
    column giving the heights of stacked ``sub-bars'' making up the
    bar.  If \code{height} is a matrix and \code{beside} is
    \code{TRUE}, then the values in each column are juxtaposed
    rather than stacked.}
  \item{width}{optional vector of bar widths. Re-cycled to length the
    number of bars drawn.  Specifying a single value will no visible
    effect unless \code{xlim} is specified.}
  \item{space}{the amount of space (as a fraction of the average bar
    width) left before each bar.  May be given as a single number or
    one number per bar.  If \code{height} is a matrix and
    \code{beside} is \code{TRUE}, \code{space} may be specified by
    two numbers, where the first is the space between bars in the
    same group, and the second the space between the groups.  If not
    given explicitly, it defaults to \code{c(0,1)} if \code{height}
    is a matrix and \code{beside} is \code{TRUE}, and to 0.2
    otherwise.}
  \item{names.arg}{a vector of names to be plotted below each bar or
    group of bars.  If this argument is omitted, then the names are
    taken from the \code{names} attribute of \code{height} if this
    is a vector, or the column names if it is a matrix.}
  \item{legend.text}{a vector of text used to construct a legend for
    the plot, or a logical indicating whether a legend should be
    included.  This is only useful when \code{height} is a matrix.
    In that case given legend labels should correspond to the rows of
    \code{height}; if \code{legend.text} is true, the row names of
    \code{height} will be used as labels if they are non-null.}
  \item{beside}{a logical value.  If \code{FALSE}, the columns of
    \code{height} are portrayed as stacked bars, and if \code{TRUE}
    the columns are portrayed as juxtaposed bars.}
  \item{horiz}{a logical value.  If \code{FALSE}, the bars are drawn
    vertically with the first bar to the left.  If \code{TRUE}, the
    bars are drawn horizontally with the first at the bottom.}
  \item{density}{a vector giving the the density of shading lines, in
    lines per inch, for the bars or bar components.
    The default value of \code{NULL} means that no shading lines
    are drawn. Non-positive values of \code{density} also inhibit the
    drawing of shading lines.}
  \item{angle}{the slope of shading lines, given as an angle in
    degrees (counter-clockwise), for the bars or bar components.}
  \item{col}{a vector of colors for the bars or bar components.
    By default, grey is used if \code{height} is a vector, and
    \code{heat.colors(nrow(height))} if \code{height} is a matrix.}
  \item{prcol}{the color to be used for the plot region.}
  \item{border}{the color to be used for the border of the bars.}
  \item{main, sub}{overall and sub titles for the plot.}
  \item{xlab}{a label for the x axis.}
  \item{ylab}{a label for the y axis.}
  \item{xlim}{limits for the x axis.}
  \item{ylim}{limits for the y axis.}
  \item{xpd}{logical. Should bars be allowed to go outside region?}
  \item{log}{a character string which contains `"x"' if the x axis is
    to be logarithmic, `"y"' if the y axis is to be logarithmic and
    `"xy"' or `"yx"' if both axes are to be logarithmic.}
  \item{axes}{logical.  If \code{TRUE}, a vertical (or horizontal, if
    \code{horiz} is true) axis is drawn.}
  \item{axisnames}{logical.  If \code{TRUE}, and if there are
    \code{names.arg} (see above), the
    other axis is drawn (with \code{lty = 0}) and labeled.}
  \item{cex.axis}{expansion factor for numeric axis labels.}
  \item{cex.names}{expansion factor for names.}
  \item{inside}{logical.  If \code{TRUE}, the lines which divide
    adjacent (non-stacked!) bars will be drawn.  Only applies when
    \code{space = 0} (which it partly is when \code{beside = TRUE}).}
%MM: we still have "NotYetUsed" -- the above describe's S-plus 6.1 (not
% precisely documented!) behavior!
  \item{plot}{logical.  If \code{FALSE}, nothing is plotted.}
  \item{axis.lty}{the graphics parameter \code{lty} applied to the axis
    and tick marks of the categorical (default horzontal) axis.  Note
    that by default the axis is suppressed.}
  \item{offset}{a vector indicating how much the bars should be shifted
    relative to the x axis.}
  \item{plot.ci}{logical. If \code{TRUE}, confidence intervals are plotted
   over the bars. Note that if a stacked bar plot is generated, confidence
   intervals will not be plotted even if \code{plot.ci = TRUE}}
  \item{ci.l,ci.u}{The confidence intervals (ci.l = lower bound, ci.u =
   upper bound) to be plotted if \code{plot.ci} = \code{TRUE}. Values must have
   the same dim structure as \code{height}.}
  \item{ci.color}{the color for the confidence interval line segments}
  \item{ci.lty}{the line type for the confidence interval line segments}
  \item{ci.lwd}{the line width for the confidence interval line
    segments}
  \item{ci.width}{length of lines used for the "t" at the end of confidence
    interval line segments, as a multple of \code{width}.  Defaults to 0.5.}
  \item{plot.grid}{if \code{TRUE} a lined grid will be plotted behind the bars}
  \item{grid.inc}{the number of grid increments to be plotted}
  \item{grid.lty}{the line type for the grid}
  \item{grid.lwd}{the line width for the grid}
  \item{grid.col}{the line color for the grid}
  \item{add}{logical, if \code{TRUE} add barplot to current plot.}
  \item{panel.first}{An expression to be evaluated after the plot region
    coordinates have been set up, but prior to the drawing of the bars
    and other plot region contents. This can be useful to add additional
    plot region content behind the bars. This will also work if
    \code{add} = \code{TRUE}}
  \item{panel.last}{An expression to be evaluated after the bars have
    been drawn, but prior to the addition of confidence intervals, a
    legend and the axis annotation}
  \item{\dots}{further graphical parameters (\code{\link{par}}) are
    passed to \code{\link{plot.window}()}, \code{\link{title}()} and
    \code{\link{axis}}.}
}
\description{
  An enhancement of the standard barplot() function. Creates a bar plot
  with vertical or horizontal bars. Can plot confidence intervals for each
  bar, a lined grid behind the bars, change plot area color and
  logarithmic axes may be used. }
\details{
  This is a generic function, it currently only has a default method.
  A formula interface may be added eventually.
}
\value{
  A numeric vector (or matrix, when \code{beside = TRUE}), say
  \code{mp}, giving the coordinates of \emph{all} the bar midpoints
  drawn, useful for adding to the graph.

  If \code{beside} is true, use \code{colMeans(mp)} for the
  midpoints of each \emph{group} of bars, see example.
}
\author{Original barplot() by R-Core. Enhancements by Marc
  Schwartz.
}
\note{
  Prior to \R 1.6.0, \code{barplot} behaved as if \code{axis.lty = 1},
  unintentionally.
  0 (zero) and NA values in \code{height} will not be plotted if
  using logarithmic scales.
  If there are NA values in \code{height} and \code{beside = FALSE},
  values after the NA will not be plotted in stacked bars.
}
\seealso{
  \code{\link{plot}(\dots, type = "h")}, \code{\link{dotchart}},
  \code{\link{hist}}.
}
\examples{
tN <- table(Ni <- rpois(100, lambda = 5))
r <- barplot2(tN, col = 'gray')

#- type = "h" plotting *is* `bar'plot
lines(r, tN, type = 'h', col = 'red', lwd = 2)

barplot2(tN, space = 1.5, axisnames = FALSE,
        sub = "barplot2(..., space = 1.5, axisnames = FALSE)")

data(VADeaths, package = "datasets")
barplot2(VADeaths, plot = FALSE)
barplot2(VADeaths, plot = FALSE, beside = TRUE)

mp <- barplot2(VADeaths) # default
tot <- colMeans(VADeaths)
text(mp, tot + 3, format(tot), xpd = TRUE, col = "blue")
barplot2(VADeaths, beside = TRUE,
        col = c("lightblue", "mistyrose", "lightcyan",
                "lavender", "cornsilk"),
        legend = rownames(VADeaths), ylim = c(0, 100))
title(main = "Death Rates in Virginia", font.main = 4)

# Example with confidence intervals and grid
hh <- t(VADeaths)[, 5:1]
mybarcol <- "gray20"
ci.l <- hh * 0.85
ci.u <- hh * 1.15
mp <- barplot2(hh, beside = TRUE,
        col = c("lightblue", "mistyrose",
                "lightcyan", "lavender"),
        legend = colnames(VADeaths), ylim = c(0, 100),
        main = "Death Rates in Virginia", font.main = 4,
        sub = "Faked 95 percent error bars", col.sub = mybarcol,
        cex.names = 1.5, plot.ci = TRUE, ci.l = ci.l, ci.u = ci.u,
        plot.grid = TRUE)
mtext(side = 1, at = colMeans(mp), line = -2,
      text = paste("Mean", formatC(colMeans(hh))), col = "red")
box()

# Example with horizontal bars, grid and logarithmic x axis
barplot2(1:10 , log = "x", plot.grid = TRUE, grid.inc = 10,
        xlim = c(0.5, 20), horiz = TRUE, cex.axis = 0.9,
        prcol = "gray95")
box()

# Bar shading example
barplot2(VADeaths, angle = 15 + 10 * 1:5, density = 20, col = "black",
        legend = rownames(VADeaths))
title(main = list("Death Rates in Virginia", font = 4))

# border :
barplot2(VADeaths, border = "dark blue") % lwd = 2 << not passed
%notyet barplot(VADeaths, inside = FALSE, main = "barplot(*, inside=FALSE)")
}
\keyword{hplot}

% $Id: plotCI.Rd 1441 2010-06-11 03:11:54Z warnes $
%
% $Log$
% Revision 1.8  2005/11/08 16:19:42  nj7w
% Updated Greg's email
%
% Revision 1.7  2005/06/09 14:20:28  nj7w
% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%
% Revision 1.1.1.1  2005/05/25 22:15:31  nj7w
% Initial submission as an individual package
%
% Revision 1.6  2004/07/29 14:49:02  warnes
% Integrate changes from the version of plotCI maintained by Martin Maechler.
%
% Revision 1.5  2004/05/27 15:11:08  warnes
% Fix spelling error.
%
% Revision 1.4  2002/09/23 13:59:30  warnes
% - Modified all files to include CVS Id and Log tags.
%
%

\name{plotCI}
\alias{plotCI}
\title{Plot Error Bars and Confidence Intervals}
\description{
  Given a set of x and y values and interval width or upper and lower bounds,
  plot the points with error bars.  This can be a useful tool for
  visualizing confidence intervals.
  }
\usage{
plotCI(x, y = NULL, uiw, liw = uiw, ui, li, err='y', ylim=NULL,
       xlim=NULL, type="p",  col=par("col"), barcol=col,
       pt.bg = par("bg"),  sfrac = 0.01, gap=1, lwd=par("lwd"),
       lty=par("lty"), labels=FALSE, add=FALSE, xlab, ylab,  minbar,
       maxbar, ... )
}
\arguments{
  \item{x,y}{ coordinates for the center of error bars. \code{y}
    defaults to \code{1:n}.}

  \item{uiw}{ width of the upper or right error bar. Set to \code{NULL}
    or \code{NA} to omit upper bars.}

  \item{liw}{ width of the lower or left error bar.  Defaults to same value as
    \code{uiw}.  Set to \code{NULL} or \code{NA} to omit lower bars. }

  \item{ui}{ upper end of error bars.  Defaults to \code{y + uiw} or
    \code{x + uiw} depeding on \code{err}.  Set to \code{NULL}  or
    \code{NA} to omit upper bars. }

  \item{li}{ lower end of error bars.  Defaults to \code{y - liw} or
    \code{x - liw} depedning on \code{err}.  Set to \code{NULL}  or
    \code{NA} to omit lower bars.}

  \item{err}{ direction for error bars. Set to "y" for vertical bars.
    Set to "x" for horizontal bars. Defaults to "y".}

  \item{col}{ color of plotting character used center marker of error
    bars. Default is "black".}

  \item{xlim, ylim}{ range of x/y values to include in the plotting area. }

  \item{type}{point/line type; passed to \code{\link{points}}}

  \item{barcol}{color of the error bars.  Defaults to the same value as
    \code{col} }

  \item{pt.bg}{background color of points (use
    \code{pch=21, pt.bg=par("bg")} to get open points superimposed on
    error bars).}

  \item{sfrac}{ width of "crossbar" at the end of error bar as a fraction
    of the x plotting region. Defaults to 0.01. }

  \item{gap}{ space left between the center of the error bar and the
    lines marking the error bar in units of the height (width) of the
    letter "O".  Defaults to 1.0 }

  \item{lwd}{ width of bar lines. }

  \item{lty}{ line type of bar lines. }

  \item{labels}{ either a logical value indicating whether the circles
    representing the x values should be replaced with text giving the
    actual values or a vector containing labels to use
    instead. Defaults to \code{FALSE}. }

  \item{add}{logical indicating whether error bars should be added to
    the current plot.  If \code{FALSE} (the defailt), a new plot will be
    created and symbols/labels for the x values will be plotted before
    drawing error bars.}

  \item{minbar}{minumum allowed value for bar ends.  If specified,
    values smaller than \code{minbar} will be replaced with
    \code{minbar}. }

  \item{maxbar}{maximum allowed value for bar ends.  If specified,
    values larger than \code{maxbar} will be replaced with
    \code{maxbar}. }

  \item{\dots}{ optional plotting parameters }

  \item{xlab}{ label for x axis. }
  \item{ylab}{label for y axis. }

}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}

\author{ Original version by Bill Venables
  \email{wvenable@attunga.stats.adelaide.edu.au} posted to r-help on
  Sep. 20, 1997.  Enhanced version posted to r-help by Ben Bolker
  \email{ben@zoo.ufl.edu} on Apr. 16, 2001.  This version was modified
  and extended by Gregory R. Warnes
  \email{greg@warnes.net}.  Additional changes
  suggested by Martin Maechler \email{maechler@stat.math.ethz.ch}
  integrated on July 29, 2004.
  }

\seealso{ \code{\link{plotmeans}} provides an enhanced wrapper to
          \code{plotCI}. }

\examples{

  # plot means and
  data(state)
  tmp   <- split(state.area, state.region)
  means <- sapply(tmp, mean)
  stdev <- sqrt(sapply(tmp, var))
  n     <- sapply(tmp,length)
  ciw   <- qt(0.975, n) * stdev / sqrt(n)

  # plain
  plotCI(x=means, uiw=ciw)

  # prettier
  plotCI(x=means, uiw=ciw, col="black", barcol="blue", lwd=1)

  # give mean values
  plotCI(x=means, uiw=ciw, col="black", barcol="blue",
         labels=round(means,-3), xaxt="n", xlim=c(0,5) )
  axis(side=1, at=1:4, labels=names(tmp), cex=0.7)

  # better yet, just use plotmeans ... #
  plotmeans( state.area ~ state.region )

\dontshow{
## Just for testing
  plotCI(x=means, uiw=NA)
  plotCI(x=means, uiw=NULL)
  plotCI(x=means, uiw=ciw)
  plotCI(x=means, uiw=ciw, liw=NULL)
  plotCI(x=means, uiw=ciw, liw=NA)
  plotCI(x=means, liw=ciw, ciw=NULL)
  plotCI(x=means, liw=ciw, ciw=NA)
  ciw.na <- ciw
  ciw.na[3] <- NA
  plotCI(x=means, uiw=ciw.na, liw=ciw)
  plotCI(x=means, liw=ciw.na, uiw=ciw)
}


}

\keyword{ hplot }

\name{gplots-deprecated}
\alias{deprecated}
\title{Deprecated functions}
\description{
  These functions have been deprecated and will be removed in future
  releases of gplots.
}
\usage{
  ## No deprecated functions at this time ##
}
%\arguments{
%}
\details{
These functions have been deprecated.  Please refer to the manual page
for the replacement function:
  \itemize{
    \item (No deprecated functions at this time)
  }
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{
  \code{\link[base]{Deprecated}}
}
\keyword{misc}

\name{textplot}
\alias{textplot}
\alias{textplot.default}
\alias{textplot.character}
\alias{textplot.matrix}
\alias{textplot.data.frame}

\title{Display text information in a graphics plot.}

\description{
  This function displays text output in a graphics window.  It is the
  equivalent of 'print' except that the output is displayed as a plot.
}
\usage{
textplot(object, halign="center", valign="center", cex, ...)
\method{textplot}{default}(object, halign=c("center","left","right"),
         valign=c("center", "top", "bottom"), cex, ... )
\method{textplot}{character}(object, halign = c("center", "left", "right"),
         valign = c("center", "top", "bottom"), cex, fixed.width=TRUE,
         cspace=1, lspace=1, mar=c(0, 0, 3, 0) + 0.1,
         tab.width = 8, ...)
\method{textplot}{data.frame}(object, halign = c("center", "left", "right"),
         valign = c("center", "top", "bottom"), cex, ...)
\method{textplot}{matrix}(object, halign = c("center", "left", "right"),
         valign = c("center", "top", "bottom"), cex, cmar = 2,
         rmar = 0.5, show.rownames = TRUE, show.colnames = TRUE,
         hadj = 1, vadj = 1, mar = c(1, 1, 4, 1) + 0.1,
         col.data = par("col"), col.rownames = par("col"), 
         col.colnames = par("col"), ...) 
}
\arguments{
  \item{object}{Object to be displayed.}
  \item{halign}{Alignment in the x direction, one of "center", "left",
    or "right". }
  \item{valign}{Alignment in the y direction, one of "center", "top" ,
    or "bottom"}
  \item{cex}{Character size, see \code{\link{par}} for details. If
    unset, the code will attempt to use the largest value which allows
    the entire object to be displayed.}
  \item{fixed.width}{Logical value indicating whether to emulate a
    fixed-width font by aligning characters in each row of text. This is
    usually necessary for text-formatted tables display properly.
    Defaults to 'TRUE'.}
  \item{cspace}{Space between characters as a
    multiple of the width of the letter 'W'. This only applies when 
    \code{fixed.width==TRUE}. }
  \item{lspace}{Line spacing. This only applies when
    \code{fixed.width==TRUE}.}
  \item{mar}{Figure margins, see the documentation for \code{par}.}
  \item{rmar, cmar}{Space between rows or columns, in
    fractions of the size of the letter 'M'.}
  \item{show.rownames, show.colnames}{Logical value indicating whether row or
    column names will be displayed.}
  \item{hadj,vadj}{Vertical and horizontal location of elements within
    matrix cells.  These have the same meaning as the \code{adj} graphics
    paramter (see \code{\link{par}}).}
  \item{col.data}{Colors for data elements.  If a single value is
    provided, all data elements will be the same color.  If a matrix
    matching the dimensions of the data is provided, each data element
    will receive the specified color.}
  \item{col.rownames, col.colnames}{Colors for row names and column
    names, respectively.  Either may be specified as a scalar or a
    vector of appropriate length.}
  \item{tab.width}{Width of a single tab stop, in characters}
  \item{\dots}{ Optional arguments passed to the text plotting command
    or specialied object methods}
}
\details{
  A new plot is created and the object is displayed
  using the largest font that will fit on in the plotting region.  The
  \code{halign} and \code{valign} parameters can be used to control the
  location of the string within the plotting region.

  For matrixes and vectors a specialized textplot function is available,
  which plots each of the cells individually, with column widths set
  according to the sizes of the column elements.  If present, row and
  column labels will be displayed in a bold font.  
}
\value{
  The character scaling factor (\code{cex}) used.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{ \code{\link{plot}}, \code{\link{text}}, 
          \code{\link[utils]{capture.output}} 
        }

\examples{
\dontrun{
### simple examples
# show R version information
textplot(version)

# show the alphabet as a single string
textplot( paste(letters[1:26], collapse=" ") )

# show the alphabet as a matrix
textplot( matrix(letters[1:26], ncol=2))

### Make a nice 4 way display with two plots and two text summaries

data(iris)
par(mfrow=c(2,2))
plot( Sepal.Length ~ Species, data=iris, border="blue", col="cyan",
      main="Boxplot of Sepal Length by Species" )
plotmeans( Sepal.Length ~ Species, data=iris, barwidth=2, connect=FALSE,
           main="Means and 95\% Confidence Intervals\nof Sepal Length by Species")

info <- sapply( split(iris$Sepal.Length, iris$Species),
                function(x) round(c(Mean=mean(x), SD=sd(x), N=nrow(x)),2) )

textplot( info, valign="top"  )
title("Sepal Length by Species")

reg <- lm( Sepal.Length ~ Species, data=iris )
textplot( capture.output(summary(reg)), valign="top")
title("Regression of Sepal Length by Species")

par(mfrow=c(1,1))

### Show how to control text color
cols <- c("red", "green", "magenta", "forestgreen")
mat <- cbind(name=cols, t(col2rgb(cols)), hex=col2hex(cols))

textplot(mat,
         col.data=matrix(cols, nrow=length(cols), byrow=FALSE, ncol=5),
         )

### Show how to manually tune the character size
data(iris)
reg <- lm( Sepal.Length ~ Species, data=iris )
text <- capture.output(summary(reg))

# do the plot and capture the character size used
textplot(text, valign="top")

# see what size was used
cex

# now redo the plot at 80\% size
textplot( text, valign="top", cex=cex*0.80)



}
}
\keyword{hplot}

\name{angleAxis}
\alias{angleAxis}
\title{Add a Axis to a Plot with Rotated Labels}
\description{
Add a labeled axis to the current plot with rotated text
}
\usage{
angleAxis(side, labels, at = 1:length(labels), srt = 45, adj, xpd = TRUE, ...)
}
\arguments{
  \item{side}{
    an integer specifying which side of the plot the axis is to be
    drawn on. The axis is placed as follows: 1=below, 2=left, 3=above and 
    4=right.
    }
  \item{labels}{character or expression vector of labels to be placed at the
    tickpoints.
    }
  \item{at}{the points at which tick-marks are to be drawn. Non-finite
    (infinite, NaN or NA) values are omitted.
    }
  \item{srt}{
    The string rotation in degrees. Defaults to 45 degrees (clockwise).
    }
  \item{adj}{Text justification.  
    A value of 0 produces left-justified text, 0.5 centered text and 1 
    right-justified text. For \code{side=1} and \code{side=2}, the 
    default value is \code{adj=1}.  For \code{side=3} and \code{side=4} 
    the default value is \code{adj=0}.
    }
  \item{xpd}{A logical value or NA. If FALSE, labels are clipped to the 
    plot region, if TRUE, labels are clipped to the figure region, and
    if NA, labels are clipped to the device region.
    }
  \item{\dots}{optional arguments passed to \code{text}.  Common examples are \code{col}, \code{cex}.}
}
\details{
This function augments the feature of the \code{axis} functon by allowing the axis labels to be rotated.
}
\author{Gregory R. Warnes \email{greg@warnes.net} }
\seealso{
\code{\link{axis}}
}
\examples{
\dontshow{set.seed(42)}
# create a vector with some values and long labels
values <- sample(1:10)
names(values) <- sapply(letters[1:10], 
                        function(x) paste(rep(x, 10), sep="",collapse="") 
                        ) 

# barplot labels are too long for the available space, hence some are not plotted
barplot(values)

# to add angled labels, tell barplot not to label the x axis, and store the bar location
at <- barplot(values, xaxt="n")
# then use angleAxs
angleAxis(1, at=at, labels = names(values))

# angle counter-clockwise instead
at <- barplot(values, xaxt="n")
angleAxis(1, at=at, labels = names(values), srt=-45, adj=0)

# put labels at the top
oldpar <- par()$mar
par(mar=c(1,4,5,2)+0.1)
at <- barplot(values, xaxt="n")
angleAxis(3, at=at, labels = names(values))
par(oldpar)

# put labels on the left
oldpar <- par()$mar
par(mar=c(5,5,3,2)+0.1)
at <- barplot(values, yaxt="n", horiz=TRUE)
angleAxis(2, at=at, labels = names(values))
par(oldpar)

# put labels on the right
oldpar <- par()$mar
par(mar=c(2,5,3,5)+0.1)
at <- barplot(values, yaxt="n", horiz=TRUE)
angleAxis(4, at=at, labels = names(values))
par(oldpar)

# specify colors for bars and labels
at <- barplot(values, xaxt="n", col=1:10)
angleAxis(1, at=at, labels = names(values), col=1:10)
}
\keyword{aplot}

% $Id: lowess.Rd 2124 2016-03-30 15:29:44Z warnes $
%
% $Log$
% Revision 1.7  2005/06/09 14:20:28  nj7w
% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%
% Revision 1.1.1.1  2005/05/25 22:15:30  nj7w
% Initial submission as an individual package
%
% Revision 1.6  2003/12/03 02:46:51  warnes
% - match function argument defaults with 'usage'
%
% Revision 1.5  2002/09/23 13:59:30  warnes
% - Modified all files to include CVS Id and Log tags.
%
%

\name{lowess}
\alias{lowess}
\alias{lowess.default}
\alias{lowess.formula}
\alias{plot.lowess}
\alias{plotLowess}
\title{Scatter Plot Smoothing}
\usage{
lowess(x, ...)

\method{lowess}{default}(x, y=NULL, f=2/3, iter=3L, delta=0.01 *
       diff(range(x)), ...)

\method{lowess}{formula}(formula,data=parent.frame(), ..., subset, f=2/3,
       iter=3L, delta=.01*diff(range(mf[-response])))

\method{plot}{lowess}(x, y, ..., col.lowess="red", lty.lowess=2)

plotLowess(formula, data=parent.frame(), ..., subset=parent.frame(),
           col.lowess="red", lty.lowess=2  )
}
\alias{lowess}
\arguments{
  \item{formula}{ formula providing a single dependent variable (y) and
    an single independent variable (x) to use as coordinates in the
    scatter plot.}
  \item{data}{a data.frame (or list) from which the variables in `formula'
    should be taken.}
  \item{subset}{ an optional vector specifying a subset of observations
    to be used in the fitting process. }
  \item{x, y}{vectors giving the coordinates of the points in the scatter
    plot. Alternatively a single plotting structure can be specified.}
  \item{f}{the smoother span. This gives the proportion of points in
    the plot which influence the smooth at each value.
    Larger values give more smoothness.}
  \item{iter}{the number of robustifying iterations which should be
    performed.
    Using smaller values of \code{iter} will make \code{lowess} run
    faster.}
  \item{delta}{values of \code{x} which lie within \code{delta}
    of each other replaced by a single value in the output from
    \code{lowess}.}
  \item{...}{parameters for methods.}
  \item{col.lowess, lty.lowess}{color and line type for plotted line}
}
\description{
  The \code{lowess} function performs the computations for the
  \emph{LOWESS} smoother (see the reference below).
  \code{lowess} returns a an object containing components
  \code{x} and \code{y} which give the coordinates of the smooth.
  The smooth can then be added to a plot of the original
  points with the function \code{lines}.

  Alternatively, \code{plot} can be called directly on the object
  returned from \code{lowess} and the 'lowess' method for \code{plot}
  will generate a scatterplot of the original data with a \code{lowess}
  line superimposed.

  Finally, the \code{plotLowess} function both calculates the
  \code{lowess} smooth and plots the original data with a \code{lowess}
  smooth.
}
\references{
  Cleveland, W. S. (1979)
  Robust locally weighted regression and smoothing scatterplots.
  \emph{J. Amer. Statist. Assoc.} \bold{74}, 829--836.

  Cleveland, W. S. (1981)
  LOWESS: A program for smoothing scatterplots by robust locally weighted
  regression. \emph{The American Statistician}, \bold{35}, 54.
}
\seealso{\code{\link{loess}} (in package \code{modreg}), a newer
  formula based version of \code{lowess} (with different defaults!).
}
\examples{
data(cars)

#
# x,y method
#
plot(cars$speed, cars$dist, main="lowess(cars)")
lines(lowess(cars$speed, cars$dist), col=2)
lines(lowess(cars$speed, cars$dist, f=.2), col=3)
legend(5, 120, c(paste("f=", c("2/3", ".2"))), lty=1, col=2:3)

#
# formula method: plot, then calculate the lowess smoother,
#                 then add smooth to the plot
#
plot(dist ~ speed, data=cars, main="lowess(cars)")
lines(lowess(dist ~ speed, data=cars), col=2, lty=2)
lines(lowess(dist ~ speed, data=cars, f=.2), col=3) # smaller bandwith
legend(5, 120, c(paste("f=", c("2/3", ".2"))), lty=1, col=2:3)

#
# formula method: calculate lowess() smoother, then call plot()
#                  on the lowess object
#
lw <- lowess(dist ~ speed, data=cars)
plot(lw, main="lowess(cars)"  )

#
# formula method: calculate and plot in a single command
#
plotLowess(dist ~ speed, data=cars, main="lowess(cars)")

\dontshow{
# test non-standard evaluation

with(cars, plotLowess(dist ~ speed, main="lowess(cars)") )
with(cars, plot(lowess(dist ~ speed, main="lowess(cars)") ) )

idx <- cars$speed > 10
plotLowess(dist[idx] ~ speed[idx], data=cars, main="lowess(cars)")
plot(lowess(dist[idx] ~ speed[idx], data=cars, main="lowess(cars)"))

with(cars, plotLowess(dist[idx] ~ speed[idx], main="lowess(cars)") )
with(cars, plot(lowess(dist[idx] ~ speed[idx], main="lowess(cars)") ))

local({
dist <- cars$dist
speed <- cars$speed
plotLowess(dist[idx] ~ speed[idx], main="lowess(cars)")
plot(lowess(dist[idx] ~ speed[idx], main="lowess(cars)"))
})
} % dontshow

}
\keyword{smooth}

% $Id: space.Rd 1441 2010-06-11 03:11:54Z warnes $
%
% $Log$
% Revision 1.9  2005/09/12 15:44:37  nj7w
% Updated Greg's email
%
% Revision 1.8  2005/06/09 14:20:28  nj7w
% Updating the version number, and various help files to synchronize splitting of gregmisc bundle in 4 individual components.
%
% Revision 1.1.1.1  2005/05/25 22:15:31  nj7w
% Initial submission as an individual package
%
% Revision 1.7  2004/04/13 13:42:31  warnes
% Add ability to space points along 'y' direction.
%
% Revision 1.6  2003/01/03 19:23:34  warnes
% - Added reference to and comparison example of sunflowerplot.
% - Added code to put example plots on same plot window using par(mfrow=...).
%
% Revision 1.5  2002/09/23 13:59:30  warnes
% - Modified all files to include CVS Id and Log tags.
%
%

\name{space}
\alias{space}
\title{Space points in an x-y plot so they don't overlap.}
\description{
  Space points in an x-y plot so they don't overlap.
}
\usage{
space(x, y, s=1/50, na.rm=TRUE, direction="x")
}
\arguments{
  \item{x}{numeric vector of x coordonates.}
  \item{y}{numeric vector of x coordonates.}
  \item{s}{either a single numeric value or 2 element vector specifying
  the minimum distance between points in the x and y dimensions as a
  fraction of the x and y range.  Defaults to 1/50.}
  \item{na.rm}{logical indicating whether pairs where one or both
    elements are missing should be removed.  Defaults to TRUE.}
  \item{direction}{"x" or "y", indicating which direction points should
    be moved to accomplish spacine.}
}
\details{
  In an x-y plot where at least one variable has discrete levels several
  points may be plotted at or very near the same coordonates.
  This makes it difficult to guage the number of points in a specific
  region.  A common method of resolving this problem is to 'jitter' the
  points by adding random noise.

  This function takes a different approach to the same problem.  

  When there are two or more points with the same (x,y) value (or
  within x+-s[1] and x+-s[2]), it spaces these out in the x direction
  so that the points are separated by at least distance s.

  Another method for dealing with overploting is available in the
  \code{\link{sunflowerplot}} function.
}
\value{
  list with two components
  \item{x}{(modified) x location for each input point}
  \item{y}{y location of each input point}
}
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{\link{jitter}}, \code{\link{sunflowerplot}} }

\examples{
   x <- rep(1:5, 10)
   y <- round(rnorm(length(x),x))

   prepar <- par("mfrow")
   par(mfrow=c(1,3))

   # standard x-y plot: noverlapping points are hidden
   plot(x,y)
   title("Standard Plot") 

   # 'spaced' plot: overlapping points are spread out and visible
   plot(space(x,y))
   title("Plot with 'space'")


   # 'spaced' plot: overlapping points are spread out along y and visible
   plot(space(x,y, direction='y'))
   title("Plot with 'space', direction='y' ")


   # 'sunflower' plot, another approach, overlapping points are
   # indicated via petals
   sunflowerplot(x,y)
   title("Sunflower Plot")

   \testonly{
     # check that missign values correctly handled
     x <- c(x,NA)
     y <- c(y,NA)
     plot(space(x,y))
   }

   par(mfrow=prepar)
}
\keyword{ dplot }

\name{rtPCR}
\alias{rtPCR}
\docType{data}
\title{Teratogenesis rtPCR data}
\description{
  rtPCR data for experiments investigating a variety of markers for
  characterizing teratogenicity.
}
\usage{data(rtPCR)}
\format{
  A data frame with 1672 observations on the following 21 variables.
  \describe{
    \item{PlateID}{a factor with levels \code{A0027002} through
      \code{A0054019}}
    \item{Test.Substance}{a factor with levels \code{Compound A} through
      \code{Compound H}}
    \item{Teratogenicity.in.vivo}{a factor with levels \code{Non}
      \code{Strong} \code{Weak / Moderate}}
    \item{Sample}{a factor with levels \code{Sample 1} - \code{Sample 152}}
    \item{Rep..}{a factor with levels \code{Rep 1} - \code{Rep 21}}
    \item{Label}{a factor with levels \code{Ctrl}, \code{Neg. Ctrl}
      \code{P1} - \code{P9}, \code{No Vehicle Ctrl}, and \code{Pos. Ctrl}}
    \item{Conc..ug.ml.}{a numeric vector}
    \item{Detector}{a factor with levels \code{ProbeType 1} -
      \code{ProbeType 17}}
    \item{Avg.delta.Ct}{a numeric vector}
    \item{delta.Ct.SD}{a numeric vector}
    \item{delta.delta.Ct}{a numeric vector}
    \item{RQ}{a numeric vector}
    \item{X..RQ}{a numeric vector}
    \item{X100..Custom..}{a numeric vector}
    \item{X100...Custom..}{a numeric vector}
    \item{Custom..}{a numeric vector}
    \item{Custom...1}{a numeric vector}
    \item{RQ.Min}{a numeric vector}
    \item{RQ.Max}{a numeric vector}
    \item{Threshold}{a numeric vector}
  }
}
\details{
  TBA
}
\source{
  Anonymized data.
}
\examples{
data(rtPCR)

# same scale
overplot( RQ ~ Conc..ug.ml. | Test.Substance,
         data=rtPCR,
         subset=Detector=="ProbeType 7" & Conc..ug.ml. > 0,
         same.scale=TRUE,
         log="xy",
         f=3/4,
         main="Detector=ProbeType 7",
         xlab="Concentration (ug/ml)",
         ylab="Relative Gene Quantification"
         )

# different scales, but force lower limit to 0.01 
overplot( RQ ~ Conc..ug.ml. | Test.Substance,
         data=rtPCR,
         subset=Detector=="ProbeType 7" & Conc..ug.ml. > 0,
         log="xy",
         f=3/4,
         main="Detector=ProbeType 7",
         xlab="Concentration (ug/ml)",
         ylab="Relative Gene Quantification",
         min.y=0.01
         )
}
\keyword{datasets}

\name{heatmap.2}
\alias{heatmap.2}
\title{ Enhanced Heat Map }
\description{
  A heat map is a false color image (basically
  \code{\link{image}(t(x))}) with a dendrogram added to the left side
  and/or to the top.  Typically, reordering of the rows and columns
  according to some set of values (row or column means) within the
  restrictions imposed by the dendrogram is carried out.

  This heatmap provides a number of extensions to the standard R
  \code{\link[stats]{heatmap}} function.
}
\usage{
heatmap.2 (x,

           # dendrogram control
           Rowv = TRUE,
           Colv=if(symm)"Rowv" else TRUE,
           distfun = dist,
           hclustfun = hclust,
           dendrogram = c("both","row","column","none"),
           reorderfun = function(d, w) reorder(d, w),
           symm = FALSE,

           # data scaling
           scale = c("none","row", "column"),
           na.rm=TRUE,

           # image plot
           revC = identical(Colv, "Rowv"),
           add.expr,

           # mapping data to colors
           breaks,
           symbreaks=any(x < 0, na.rm=TRUE) || scale!="none",

           # colors
           col="heat.colors",

           # block sepration
           colsep,
           rowsep,
           sepcolor="white",
           sepwidth=c(0.05,0.05),

           # cell labeling
           cellnote,
           notecex=1.0,
           notecol="cyan",
           na.color=par("bg"),

           # level trace
           trace=c("column","row","both","none"),
           tracecol="cyan",
           hline=median(breaks),
           vline=median(breaks),
           linecol=tracecol,

           # Row/Column Labeling
           margins = c(5, 5),
           ColSideColors,
           RowSideColors,
           cexRow = 0.2 + 1/log10(nr),
           cexCol = 0.2 + 1/log10(nc),
           labRow = NULL,
           labCol = NULL,
           srtRow = NULL,
           srtCol = NULL,
           adjRow = c(0,NA),
           adjCol = c(NA,0),
           offsetRow = 0.5,
           offsetCol = 0.5,
           colRow = NULL,
           colCol = NULL,

           # color key + density info
           key = TRUE,
           keysize = 1.5,
           density.info=c("histogram","density","none"),
           denscol=tracecol,
           symkey = any(x < 0, na.rm=TRUE) || symbreaks,
           densadj = 0.25,
           key.title = NULL,
           key.xlab = NULL,
           key.ylab = NULL,
           key.xtickfun = NULL,
           key.ytickfun = NULL,
           key.par=list(),

           # plot labels
           main = NULL,
           xlab = NULL,
           ylab = NULL,

           # plot layout
           lmat = NULL,
           lhei = NULL,
           lwid = NULL,

           # extras
           extrafun=NULL,
           ...
           )
           }
\arguments{
  % Dendrogram Control
  \item{x}{numeric matrix of the values to be plotted. }
  \item{Rowv}{determines if and how the \emph{row} dendrogram should be
    reordered.	By default, it is TRUE, which implies dendrogram is
    computed and reordered based on row means. If NULL or FALSE, then no
    dendrogram is computed and no reordering is done. If a
    \code{\link{dendrogram}}, then it is used "as-is", ie
    without any reordering. If a vector of integers, then dendrogram is
    computed and reordered based on the order of the vector.}
  \item{Colv}{determines if and how the \emph{column} dendrogram should
    be reordered.	Has the options as the \code{Rowv} argument above and
    \emph{additionally} when \code{x} is a square matrix,
    \code{Colv="Rowv"} means that columns should be treated identically
    to the rows.}
  \item{distfun}{function used to compute the distance (dissimilarity)
    between both rows and columns.  Defaults to \code{\link{dist}}.}
  \item{hclustfun}{function used to compute the hierarchical clustering
    when \code{Rowv} or \code{Colv} are not dendrograms.  Defaults to
    \code{\link{hclust}}.}
  \item{dendrogram}{character string indicating whether to draw 'none',
    'row', 'column' or 'both' dendrograms.  Defaults to 'both'. However,
    if Rowv (or Colv) is FALSE or NULL and dendrogram is 'both', then a
    warning is issued and Rowv (or Colv) arguments are honoured.}
  \item{reorderfun}{\code{function(d, w)} of dendrogram and weights for
    reordering the row and column dendrograms.  The default uses
    \code{\link{stats}{reorder.dendrogram}} }.
  \item{symm}{logical indicating if \code{x} should be treated
    \bold{symm}etrically; can only be true when \code{x} is a
    square matrix.}
  % data scaling
  \item{scale}{character indicating if the values should be centered and
    scaled in either the row direction or the column direction, or
    none.  The default is \code{"none"}.}
  \item{na.rm}{logical indicating whether \code{NA}'s should be removed.}
  % image plot
  \item{revC}{logical indicating if the column order should be
    \code{\link{rev}}ersed for plotting, such that e.g., for the
    symmetric case, the symmetry axis is as usual.}
  \item{add.expr}{expression that will be evaluated after the call to
    \code{image}.  Can be used to add components to the plot.}
  \item{breaks}{(optional) Either a numeric vector indicating the
    splitting points for binning \code{x} into colors, or a integer
    number of break points to be used, in which case the break points
    will be spaced equally between \code{min(x)} and \code{max(x)}.}
  \item{symbreaks}{Boolean indicating whether breaks should be
    made symmetric about 0. Defaults to \code{TRUE} if the data includes
    negative values, and to \code{FALSE} otherwise.}
  \item{col}{colors used for the image. Defaults to heat colors
    (\code{heat.colors}).}
  % block separation
  \item{colsep, rowsep, sepcolor}{(optional) vector of integers
    indicating which columns or rows should be separated from the
    preceding columns or rows by a narrow space of color
    \code{sepcolor}.}
  \item{sepwidth}{(optional) Vector of length 2 giving the width
    (colsep) or height (rowsep) the separator box drawn by colsep and
    rowsep as a function of the width (colsep) or height (rowsep) of a
    cell. Defaults to \code{c(0.05, 0.05)}}
  % cell labeling
  \item{cellnote}{(optional) matrix of character strings which will be
    placed within each color cell, e.g. p-value symbols.}
  \item{notecex}{(optional) numeric scaling factor for \code{cellnote}
    items.}
  \item{notecol}{(optional) character string specifying the color for
    \code{cellnote} text.  Defaults to "cyan".}
  \item{na.color}{Color to use for missing value (\code{NA}). Defaults
    to the plot background color.}
  % level trace
  \item{trace}{character string indicating whether a solid "trace" line
    should be drawn across 'row's or down 'column's, 'both' or 'none'.
    The distance of the line from the center of each color-cell is
    proportional to the size of the measurement. Defaults to 'column'.}
  \item{tracecol}{character string giving the color for "trace"
    line. Defaults to "cyan".}
  \item{hline, vline, linecol}{Vector of values within cells where a
    horizontal or vertical dotted line should be drawn.  The color of
    the line is controlled by \code{linecol}.  Horizontal  lines are only
    plotted if \code{trace} is 'row' or 'both'.  Vertical lines are only
    drawn if \code{trace} 'column' or 'both'.   \code{hline} and
    \code{vline} default to the median of the breaks, \code{linecol}
    defaults to the value of \code{tracecol}.}
  % Row/Column Labeling
  \item{margins}{numeric vector of length 2 containing the margins
    (see \code{\link{par}(mar= *)}) for column and row names,
    respectively.}
  \item{ColSideColors}{(optional) character vector of length
    \code{ncol(x)} containing the color names for a horizontal side bar
    that may be used to annotate the columns of \code{x}.}
  \item{RowSideColors}{(optional) character vector of length
    \code{nrow(x)} containing the color names for a vertical side bar
    that may be used to annotate the rows of \code{x}.}
  \item{cexRow, cexCol}{positive numbers, used as \code{cex.axis} in
    for the row or column axis labeling.  The defaults currently only
    use number of rows or columns, respectively.}
  \item{labRow, labCol}{character vectors with row and column labels to
    use; these default to \code{rownames(x)} or \code{colnames(x)},
    respectively.}
  \item{srtRow, srtCol}{angle of row/column labels, in degrees from
    horizontal}
  \item{adjRow, adjCol}{2-element vector giving the (left-right,
    top-bottom) justification of row/column labels (relative to the text
    orientation).}
  \item{offsetRow, offsetCol}{Number of character-width spaces to
    place between row/column labels and the edge of the plotting
    region.}
  \item{colRow, colCol}{color of row/column labels, either a scalar to
    set the color of all labels the same, or a vector providing the
    colors of each label item}
  % Color key and density info
  \item{key}{logical indicating whether a color-key should be shown.}
  \item{keysize}{numeric value indicating the size of the key}
  \item{density.info}{character string indicating whether to superimpose
    a 'histogram', a 'density' plot, or no plot ('none') on the
    color-key.}
  \item{denscol}{character string giving the color for the density
    display specified by \code{density.info}, defaults to the same value
    as \code{tracecol}.}
  \item{symkey}{Boolean indicating whether the color key should be
    made symmetric about 0. Defaults to \code{TRUE} if the data includes
    negative values, and to \code{FALSE} otherwise.}
  \item{densadj}{Numeric scaling value for tuning the kernel width when
    a density plot is drawn on the color key.  (See the \code{adjust}
    parameter for the \code{density} function for details.)  Defaults to
    0.25.}
  \item{key.title}{main title of the color key. If set to NA no title
    will be plotted.}
  \item{key.xlab}{x axis label of the color key. If set to NA no label
    will be plotted.}
  \item{key.ylab}{y axis label of the color key. If set to NA no label
    will be plotted.}
  \item{key.xtickfun}{function computing tick location and labels for
    the xaxis of the color key. Returns a named list containing
    parameters that can be passed to \code{axis}. See examples.}
  \item{key.ytickfun}{function computing tick location and labels for
    the y axis of the color key. Returns a named list containing
    parameters that can be passed to \code{axis}.  See examples.}
  \item{key.par}{graphical parameters for the color key. Named list that
    can be passed to \code{par}.}
  % plot labels
  \item{main, xlab, ylab}{main, x- and y-axis titles; defaults to none.}
  % figure layout
  \item{lmat, lhei, lwid}{visual layout: position matrix, column height,
    column width.  See below for details}
  \item{extrafun}{A function to be called after all other work. See
    examples.}
  \item{...}{additional arguments passed on to \code{\link{image}} }
}
\details{
  If either \code{Rowv} or \code{Colv} are dendrograms they are honored
  (and not reordered).	Otherwise, dendrograms are computed as
  \code{dd <- as.dendrogram(hclustfun(distfun(X)))} where \code{X} is
  either \code{x} or \code{t(x)}.\cr

  If either is a vector (of \dQuote{weights}) then the appropriate
  dendrogram is reordered according to the supplied values subject to
  the constraints imposed by the dendrogram, by \code{\link{reorder}(dd,
    Rowv)}, in the row case.
  %%
  If either is missing, as by default, then the ordering of the
  corresponding dendrogram is by the mean value of the rows/columns,
  i.e., in the case of rows, \code{Rowv <- rowMeans(x, na.rm=na.rm)}.
  %%
  If either is \code{\link{NULL}}, \emph{no reordering} will be done for
  the corresponding side.

  If \code{scale="row"} (or \code{scale="col"}) the rows (columns) are
  scaled to have mean zero and standard deviation one.  There is some
  empirical evidence from genomic plotting that this is useful.

  The default colors range from red to white (\code{heat.colors}) and
  are not pretty.  Consider using enhancements such as the
  \pkg{RColorBrewer} package,
  \url{https://cran.r-project.org/package=RColorBrewer}
  to select better colors.

  By default four components will be displayed in the plot.  At the top
  left is the color key, top right is the column dendrogram, bottom left
  is the row dendrogram, bottom right is the image plot.  When
  RowSideColor or ColSideColor are provided, an additional row or column
  is inserted in the appropriate location.   This layout can be
  overriden by specifiying appropriate values for \code{lmat},
  \code{lwid}, and \code{lhei}.  \code{lmat} controls the relative
  postition of each element, while \code{lwid} controls the column
  width, and \code{lhei} controls the row height. See the help page for
  \code{\link[graphics]{layout}} for details on how to use these
  arguments.
}
\note{
  The original rows and columns are reordered to match the dendrograms
  \code{Rowv} and \code{Colv} (if present).

  \code{heatmap.2()} uses \code{\link{layout}} to arragent the plot
  elements. Consequentially, it can \bold{not} be used in a multi
  column/row layout using \code{\link{layout}(\dots)},
  \code{\link{par}(mfrow=\dots)} or \code{(mfcol=\dots)}.
}
\value{
  Invisibly, a list with components
  \item{rowInd}{row index permutation vector as returned by
    \code{\link{order.dendrogram}}.}
  \item{colInd}{column index permutation vector.}
  \item{call}{the matched call}
  \item{rowMeans, rowSDs}{mean and standard deviation of each row: only
    present if \code{scale="row"}}
  \item{colMeans, colSDs}{mean and standard deviation of each column: only
    present if \code{scale="column"}}
  \item{carpet}{reordered and scaled 'x' values used generate the main
    'carpet'}
  \item{rowDendrogram}{row dendrogram, if present}
  \item{colDendrogram}{column dendrogram, if present}
  \item{breaks}{values used for color break points}
  \item{col}{colors used}
  \item{vline}{center-line value used for column trace, present only if
    \code{trace="both"} or \code{trace="column"} }
  \item{hline}{center-line value used for row trace, present only if
    \code{trace="both"} or \code{trace="row"} }
  \item{colorTable}{A three-column data frame providing the lower and upper
    bound and color for each bin}
  \item{layout}{A named list containing the values used for
    \code{lmat}, \code{lhei}, and \code{lwid}. }
}
\author{Andy Liaw, original; R. Gentleman, M. Maechler, W. Huber,
  G. Warnes, revisions.}

\seealso{\code{\link{image}}, \code{\link{hclust}}}

\examples{
 data(mtcars)
 x  <- as.matrix(mtcars)
 rc <- rainbow(nrow(x), start=0, end=.3)
 cc <- rainbow(ncol(x), start=0, end=.3)

 ##
 ## demonstrate the effect of row and column dendrogram options
 ##
 heatmap.2(x)                    ## default - dendrogram plotted and reordering done.
 heatmap.2(x, dendrogram="none") ##  no dendrogram plotted, but reordering done.
 heatmap.2(x, dendrogram="row")  ## row dendrogram plotted and row reordering done.
 heatmap.2(x, dendrogram="col")  ## col dendrogram plotted and col reordering done.

 heatmap.2(x, keysize=2)         ## default - dendrogram plotted and reordering done.

 heatmap.2(x, Rowv=FALSE, dendrogram="both") ## generates a warning!
 heatmap.2(x, Rowv=NULL, dendrogram="both")  ## generates a warning!
 heatmap.2(x, Colv=FALSE, dendrogram="both") ## generates a warning!

 ## Reorder dendrogram by branch means rather than sums
 heatmap.2(x, reorderfun=function(d, w) reorder(d, w, agglo.FUN = mean) )

 ## plot a sub-cluster using the same color coding as for the full heatmap
 full <- heatmap.2(x)
 heatmap.2(x, Colv=full$colDendrogram[[2]], breaks=full$breaks)  # column subset
 heatmap.2(x, Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # row subset
 heatmap.2(x, Colv=full$colDendrogram[[2]],
              Rowv=full$rowDendrogram[[1]], breaks=full$breaks)  # both

 ## Show effect of row and column label rotation
 heatmap.2(x, srtCol=NULL)
 heatmap.2(x, srtCol=0,   adjCol = c(0.5,1) )
 heatmap.2(x, srtCol=45,  adjCol = c(1,1)   )
 heatmap.2(x, srtCol=135, adjCol = c(1,0)   )
 heatmap.2(x, srtCol=180, adjCol = c(0.5,0) )
 heatmap.2(x, srtCol=225, adjCol = c(0,0)   ) ## not very useful
 heatmap.2(x, srtCol=270, adjCol = c(0,0.5) )
 heatmap.2(x, srtCol=315, adjCol = c(0,1)   )
 heatmap.2(x, srtCol=360, adjCol = c(0.5,1) )

 heatmap.2(x, srtRow=45, adjRow=c(0, 1) )
 heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=45, adjCol=c(1,1) )
 heatmap.2(x, srtRow=45, adjRow=c(0, 1), srtCol=270, adjCol=c(0,0.5) )


 ## Show effect of offsetRow/offsetCol (only works when srtRow/srtCol is
 ## not also present)
 heatmap.2(x, offsetRow=0, offsetCol=0)
 heatmap.2(x, offsetRow=1, offsetCol=1)
 heatmap.2(x, offsetRow=2, offsetCol=2)
 heatmap.2(x, offsetRow=-1, offsetCol=-1)

 heatmap.2(x, srtRow=0, srtCol=90, offsetRow=0, offsetCol=0)
 heatmap.2(x, srtRow=0, srtCol=90, offsetRow=1, offsetCol=1)
 heatmap.2(x, srtRow=0, srtCol=90, offsetRow=2, offsetCol=2)
 heatmap.2(x, srtRow=0, srtCol=90, offsetRow=-1, offsetCol=-1)


 ## Show how to use 'extrafun' to replace the 'key' with a scatterplot
 lmat <- rbind( c(5,3,4), c(2,1,4) )
 lhei <- c(1.5, 4)
 lwid <- c(1.5, 4, 0.75)

 myplot <- function() {
             oldpar <- par("mar")
             par(mar=c(5.1, 4.1, 0.5, 0.5))
             plot(mpg ~ hp, data=x)
           }

 heatmap.2(x, lmat=lmat, lhei=lhei, lwid=lwid, key=FALSE, extrafun=myplot)

 ## show how to customize the color key
 heatmap.2(x,
           key.title=NA, # no title
           key.xlab=NA,  # no xlab
           key.par=list(mgp=c(1.5, 0.5, 0),
                        mar=c(2.5, 2.5, 1, 0)),
           key.xtickfun=function() {
                 breaks <- parent.frame()$breaks
                 return(list(
                      at=parent.frame()$scale01(c(breaks[1],
                                                  breaks[length(breaks)])),
                      labels=c(as.character(breaks[1]),
                               as.character(breaks[length(breaks)]))
                      ))
           })

 heatmap.2(x,
          breaks=256,
          key.title=NA,
          key.xlab=NA,
          key.par=list(mgp=c(1.5, 0.5, 0),
                       mar=c(1, 2.5, 1, 0)),
          key.xtickfun=function() {
               cex <- par("cex")*par("cex.axis")
               side <- 1
               line <- 0
               col <- par("col.axis")
               font <- par("font.axis")
               mtext("low", side=side, at=0, adj=0,
                     line=line, cex=cex, col=col, font=font)
               mtext("high", side=side, at=1, adj=1,
                     line=line, cex=cex, col=col, font=font)
               return(list(labels=FALSE, tick=FALSE))
          })


 ##
 ## Show effect of z-score scaling within columns, blue-red color scale
 ##
 hv <- heatmap.2(x, col=bluered, scale="column", tracecol="#303030")

 ###
 ## Look at the return values
 ###
 names(hv)

 ## Show the mapping of z-score values to color bins
 hv$colorTable

 ## Extract the range associated with white
 hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",]

 ## Determine the original data values that map to white
 whiteBin <- unlist(hv$colorTable[hv$colorTable[,"color"]=="#FFFFFF",1:2])
 rbind(whiteBin[1] * hv$colSDs + hv$colMeans,
       whiteBin[2] * hv$colSDs + hv$colMeans )
 ##
 ## A more decorative heatmap, with z-score scaling along columns
 ##
 hv <- heatmap.2(x, col=cm.colors(255), scale="column",
	       RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
	       xlab="specification variables", ylab= "Car Models",
	       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
         tracecol="green", density="density")
 ## Note that the breakpoints are now symmetric about 0

 ## Color the labels to match RowSideColors and ColSideColors
 hv <- heatmap.2(x, col=cm.colors(255), scale="column",
         RowSideColors=rc, ColSideColors=cc, margin=c(5, 10),
	       xlab="specification variables", ylab= "Car Models",
	       main="heatmap(<Mtcars data>, ..., scale=\"column\")",
         tracecol="green", density="density", colRow=rc, colCol=cc,
         srtCol=45, adjCol=c(0.5,1))



%% want example using the `add.exp' argument!

 data(attitude)
 round(Ca <- cor(attitude), 2)
 symnum(Ca) # simple graphic

 # with reorder
 heatmap.2(Ca, 		 symm=TRUE, margin=c(6, 6), trace="none" )

 # without reorder
 heatmap.2(Ca, Rowv=FALSE, symm=TRUE, margin=c(6, 6), trace="none" )

 ## Place the color key below the image plot
 heatmap.2(x, lmat=rbind( c(0, 3), c(2,1), c(0,4) ), lhei=c(1.5, 4, 2 ) )

 ## Place the color key to the top right of the image plot
 heatmap.2(x, lmat=rbind( c(0, 3, 4), c(2,1,0 ) ), lwid=c(1.5, 4, 2 ) )

 ## For variable clustering, rather use distance based on cor():
 data(USJudgeRatings)
 symnum( cU <- cor(USJudgeRatings) )

 hU <- heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=topo.colors(16),
              distfun=function(c) as.dist(1 - c), trace="none")

 ## The Correlation matrix with same reordering:
 hM <- format(round(cU, 2))
 hM

 # now with the correlation matrix on the plot itself

 heatmap.2(cU, Rowv=FALSE, symm=TRUE, col=rev(heat.colors(16)),
             distfun=function(c) as.dist(1 - c), trace="none",
             cellnote=hM)

 ## genechip data examples
 \dontrun{
 library(affy)
 data(SpikeIn)
 pms <- SpikeIn@pm

 # just the data, scaled across rows
 heatmap.2(pms, col=rev(heat.colors(16)), main="SpikeIn@pm",
              xlab="Relative Concentration", ylab="Probeset",
              scale="row")

 # fold change vs "12.50" sample
 data <- pms / pms[, "12.50"]
 data <- ifelse(data>1, data, -1/data)
 heatmap.2(data, breaks=16, col=redgreen, tracecol="blue",
               main="SpikeIn@pm Fold Changes\nrelative to 12.50 sample",
               xlab="Relative Concentration", ylab="Probeset")
 }

}
\keyword{hplot}

\name{catch.d}
\alias{catch.d}
\alias{catch.r}
\docType{data}
\title{Catch at Age and Residuals}
\description{
  Catch-at-age observed data and model residuals from Icelandic saithe
  assessment.
}
\usage{
catch.d
catch.r
}
\format{
  Data frame containing three columns:
  \tabular{ll}{
    \code{Year}  \tab year\cr
    \code{Age}   \tab age\cr
    and\cr
    \code{Catch} \tab catch (thousands of individuals)\cr
    or\cr
    \code{Resid} \tab standardized residual
  }
}
\details{
  The data are from Tables 8.2 and 8.6 in the ICES (2015) fish stock
  assessment of Icelandic saithe.
}
\source{
  ICES (2015) Report of the North-Western Working Group (NWWG).
  \emph{ICES CM 2015/ACOM:07}, pp. 240--246.
}
\seealso{
  \code{\link{bubbleplot}} is an effective way to visualize these data.
}
\examples{
catch.t <- xtabs(Catch~Year+Age, catch.d)
catch.m <- as.matrix(as.data.frame(unclass(catch.t)))

# 1  Formula
bubbleplot(Catch~Age+Year, data=catch.d)
# Use rev=TRUE to get same layout as crosstab matrix:
print(catch.m)
bubbleplot(Catch~Age+Year, data=catch.d, rev=TRUE, las=1)

# 2  Data frame
bubbleplot(catch.d)

# 3  Matrix or table
bubbleplot(catch.m)
bubbleplot(catch.t)

# 4  Positive and negative values
bubbleplot(catch.r)
bubbleplot(Resid~Age+Year, catch.r, subset=Age \%in\% 4:9,
           rev=TRUE, xlim=c(3.5,9.5), cex=1.3)
}

\name{balloonplot}
\alias{balloonplot}
\alias{balloonplot.default}
\alias{balloonplot.table}
\title{Plot a graphical matrix where each cell contains a dot whose size
  reflects the relative magnitude of the corresponding component.}
\description{
  Plot a graphical matrix where each cell contains a dot whose size
  reflects the relative magnitude of the corresponding component.
}
\usage{
balloonplot(x, ...)
\method{balloonplot}{table}(x, xlab, ylab, zlab, show.zeros=FALSE,show.margins=TRUE,...)
\method{balloonplot}{default}(x,y,z,
                                xlab,
                                ylab,
                                zlab=deparse(substitute(z)),
                                dotsize=2/max(strwidth(19),strheight(19)),
                                dotchar=19,
                                dotcolor="skyblue",
                                text.size=1,
                                text.color=par("fg"),
                                main,
                                label=TRUE,
                                label.digits=2,
                                label.size=1,
                                label.color=par("fg"),
                                scale.method=c("volume","diameter"),
                                scale.range=c("absolute","relative"),
                                colsrt=par("srt"),
                                rowsrt=par("srt"),
                                colmar=1,
                                rowmar=2,
                                show.zeros=FALSE,
                                show.margins=TRUE,
                                cum.margins=TRUE,
                                sorted=TRUE,
                                label.lines=TRUE,
                                fun=function(x)sum(x,na.rm=T),
                                hide.duplicates=TRUE,
                                ... )
}
\arguments{
  \item{x}{A table object, or either a vector or a list of several
    categorical vectors containing grouping variables for the first 
    (x) margin of the plotted matrix.}
  \item{y}{Vector or list of vectors for grouping variables for the
     second (y) dimension of the plotted matrix.}
  \item{z}{Vector of values for the size of the dots in the plotted matrix.}
  \item{xlab}{Text label for the x dimension. This will be displayed on the
    x axis and in the plot title.}
  \item{ylab}{Text label for the y dimension. This will be displayed on the
    y axis and in the plot title.}
  \item{zlab}{Text label for the dot size. This will be included in the
    plot title.}
  \item{dotsize}{Maximum dot size.  You may need to adjust this value
    for different plot devices and layouts.}
  \item{dotchar}{Plotting symbol or character used for dots.  See the help
    page for the points function for symbol codes.}
  \item{dotcolor}{Scalar or vector specifying the color(s) of the dots in
    the plot.}
  \item{text.size, text.color}{Character size and color for row and
    column headers}
  \item{main}{Plot title text.}
  \item{label}{Boolean flag indicating whether the actual value of the
    elements should be shown on the plot.}
  \item{label.digits}{Number of digits used in formatting value labels.}
  \item{label.size, label.color}{ Character size and color for value labels.}
  \item{scale.method}{Method of scaling the sizes of the dot, either
    "volume" or "diameter". See below.}
  \item{scale.range}{Method for scaling original data to compute
  circle diameter.  \code{scale.range="absolute"} scales the data
  relative to 0 (i.e, maps [0,max(z)] --> [0,1]), while
  \code{scale.range="relative"} scales the data relative to min(z)
  (i.e. maps [min(z), max(z)] --> [0,1]).}
  \item{rowsrt, colsrt}{Angle of rotation for row and column labels.}
  \item{rowmar, colmar}{Space allocated for row and column labels. Each
    unit is the width/height of one cell in the table.}
  \item{show.zeros}{boolean. If \code{FALSE}, entries containing zero will be left
    blank in the plotted matrix.  If \code{TRUE}, zeros will be
    displayed.}
  \item{show.margins}{boolean. If \code{TRUE}, row and column sums are
    printed in the bottom and right margins, respectively.}
  \item{cum.margins}{boolean. If \code{TRUE}, marginal fractions are
    graphically presented in grey behind the row/column label area.}
  \item{sorted}{boolean. If \code{TRUE}, the rows will be
    arranged in sorted order by using the levels of the first y factor,
    then the second y factor, etc. The same process is used for the
    columns, based on the x factors}
  \item{label.lines}{boolean. If \code{TRUE}, borders will be drawn for
    row and column level headers.}
  \item{hide.duplicates}{boolean. If \code{TRUE}, column and row headers
    will omit duplicates within row/column to reduce clutter.  Defaults
    to \code{TRUE}.}
  \item{fun}{function to be used to combine data elements with the same
    levels of the grouping variables \code{x} and \code{y}. Defaults to \code{sum}}
  \item{\dots}{ Additional arguments passed to \code{balloonplot.default}
    or \code{plot}, as appropriate.}
}
\details{
  This function plots a visual matrix.  In each \code{x},\code{y} cell a
  dot is plotted which reflects the relative size of the corresponding
  value of \code{z}.  When \code{scale.method="volume"} the volume of
  the dot is proportional to the relative size of \code{z}.  When
  \code{scale.method="diameter"}, the diameter of the dot is proportional to
  the the relative size of \code{z}.  The "volume" method is default
  because the "diameter" method visually exaggerates differences.
}
\value{
  Nothing of interest.
}
\note{
  \code{z} is expected to be non-negative.  The function will still
  operate correctly if there are negative values of \code{z}, but the
  corresponding dots will have 0 size and a warning will be generated.
}

\references{Function inspired by question posed on R-help by Ramon
  Alonso-Allende \email{allende@cnb.uam.es}.}

\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{
  \code{\link{plot.table}}.

  \code{\link{bubbleplot}} provides an alternative interface and visual
  style based on scatterplots instead of tables.
}
\examples{
\testonly{
set.seed(12425421)
}

# Create an Example Data Frame Containing Car x Color data
carnames <- c("bmw","renault","mercedes","seat")
carcolors <- c("red","white","silver","green")
datavals <- round(rnorm(16, mean=100, sd=60),1)
data <- data.frame(Car=rep(carnames,4),
                   Color=rep(carcolors, c(4,4,4,4) ),
                   Value=datavals )
# show the data
data

# generate balloon plot with default scaling
balloonplot( data$Car, data$Color, data$Value)


# show margin label rotation & space expansion, using some long labels
levels(data$Car) <- c("BMW: High End, German","Renault: Medium End, French",
 "Mercedes: High End, German", "Seat: Imaginary, Unknown Producer")

# generate balloon plot with default scaling
balloonplot( data$Car, data$Color, data$Value, colmar=3, colsrt=90)

# Create an example using table
xnames <- sample( letters[1:3], 50, replace=2)
ynames <- sample( 1:5, 50, replace=2)

tab <- table(xnames, ynames)

balloonplot(tab)

# Example of multiple classification variabls using the Titanic data
library(datasets)
data(Titanic)

dframe <- as.data.frame(Titanic) # convert to 1 entry per row format
attach(dframe)
balloonplot(x=Class, y=list(Survived, Age, Sex), z=Freq, sort=TRUE)

# colorize: surviors lightblue, non-survivors: grey
Colors <- Titanic
Colors[,,,"Yes"] <- "skyblue"
Colors[,,,"No"] <- "grey"
colors <- as.character(as.data.frame(Colors)$Freq)

balloonplot(x=list(Age,Sex),
            y=list(Class=Class,
                   Survived=reorder.factor(Survived,new.order=c(2,1))
                   ),
            z=Freq,
            zlab="Number of Passengers",
            sort=TRUE,
            dotcol = colors,
            show.zeros=TRUE,
            show.margins=TRUE)


}
\keyword{dplot}
\keyword{hplot}

% $Id: hist2d.Rd 1812 2014-04-17 19:05:11Z warnes $
\name{hist2d}
\alias{hist2d}
\alias{print.hist2d}
\title{ Compute and Plot a 2-Dimensional Histogram }
\description{
  Compute and plot a 2-dimensional histogram.
}
\usage{
hist2d(x,y=NULL, nbins=200, same.scale=FALSE, na.rm=TRUE, show=TRUE,
       col=c("black", heat.colors(12)), FUN=base::length, xlab, ylab,
       ... )
\method{print}{hist2d}(x, ...)
}
\arguments{
  \item{x}{either a vector containing the x coordinates
    or a matrix with 2 columns. }
  \item{y}{a vector contianing the y coordinates, not required if `x'
    is matrix}
  \item{nbins}{number of bins in each dimension. May be a scalar or a
    2 element vector.  Defaults to 200.}
  \item{same.scale}{use the same range for x and y. Defaults to FALSE.}
  \item{na.rm}{ Indicates whether missing values should be
    removed. Defaults to TRUE.}
  \item{show}{ Indicates whether the histogram be displayed using
    \code{image} once it has
    been computed. Defaults to TRUE.}
  \item{col}{ Colors for the histogram. Defaults to "black" for
    bins containing no elements, a set of 16 heat colors for other bins.}
  \item{FUN}{Function used to summarize bin contents.  Defaults to
    \code{base::length}. Use, e.g., \code{mean} to calculate means for each bin
    instead of counts.}
  \item{xlab,ylab}{(Optional) x and y axis labels}
  \item{\dots}{ Parameters passed to the image function. }
}
\details{
  This fucntion creates a 2-dimensional histogram by cutting the x and
  y dimensions into \code{nbins} sections.  A 2-dimensional matrix is
  then constucted which holds the counts of the number of observed (x,y) pairs
  that fall into each bin.  If \code{show=TRUE}, this matrix is then
  then passed to \code{image} for display.
}
\value{
  A list containing 5 elements:
  \item{counts}{Matrix containing the number of points falling into each
    bin}
  \item{x.breaks, y.breaks}{Lower and upper limits of each bin}
  \item{x,y}{midpoints of each bin}
  }
\author{ Gregory R. Warnes \email{greg@warnes.net}}

\seealso{
  \code{\link{image}}, \code{\link{persp}}, \code{\link{hist}},
  \code{\link[r2d2]{freq2d}} 
  }

\examples{
   ## example data, bivariate normal, no correlation
   x <- rnorm(2000, sd=4)
   y <- rnorm(2000, sd=1)

   ## separate scales for each axis, this looks circular
   hist2d(x,y)

   ## same scale for each axis, this looks oval
   hist2d(x,y, same.scale=TRUE)

   ## use different ## bins in each dimension
   hist2d(x,y, same.scale=TRUE, nbins=c(100,200) )

   ## use the hist2d function to create an h2d object
   h2d <- hist2d(x,y,show=FALSE, same.scale=TRUE, nbins=c(20,30))

   ## show object summary
   h2d

   ## object contents
   str(h2d)

   ## perspective plot
   persp( h2d$x, h2d$y, h2d$counts,
          ticktype="detailed", theta=30, phi=30,
          expand=0.5, shade=0.5, col="cyan", ltheta=-30)

   ## for contour (line) plot ...
   contour( h2d$x, h2d$y, h2d$counts, nlevels=4 )

   ## for a filled contour plot ...
   filled.contour( h2d$x, h2d$y, h2d$counts, nlevels=4,
                   col=gray((4:0)/4) )



}
\keyword{dplot}
\keyword{hplot}
